@@ -260,10 +260,6 @@ data BCInstr
    -- Breakpoints
    | BRK_FUN          !InternalBreakpointId
 
-   -- An internal breakpoint for triggering a break on any case alternative
-   -- See Note [Debugger: BRK_ALTS]
-   | BRK_ALTS         !Bool {- enabled? -}
-
 #if MIN_VERSION_rts(1,0,3)
    -- | A "meta"-instruction for recording the name of a BCO for debugging purposes.
    -- These are ignored by the interpreter but helpfully printed by the disassmbler.
@@ -458,7 +454,6 @@ instance Outputable BCInstr where
                              = text "BRK_FUN" <+> text "<breakarray>"
                                <+> ppr info_mod <+> ppr infox
                                <+> text "<cc>"
-   ppr (BRK_ALTS active)     = text "BRK_ALTS" <+> ppr active
 #if MIN_VERSION_rts(1,0,3)
    ppr (BCO_NAME nm)         = text "BCO_NAME" <+> text (show nm)
 #endif
@@ -584,7 +579,6 @@ bciStackUse OP_INDEX_ADDR{}         = 0
 
 bciStackUse SWIZZLE{}             = 0
 bciStackUse BRK_FUN{}             = 0
-bciStackUse BRK_ALTS{}            = 0
 
 -- These insns actually reduce stack use, but we need the high-tide level,
 -- so can't use this info.  Not that it matters much.
