@@ -1,11 +1,10 @@
-{-# LANGUAGE CPP #-}
+
 module GHC.Cmm.Info (
   mkEmptyContInfoTable,
   cmmToRawCmm,
   srtEscape,
 
   -- info table accessors
-  PtrOpts (..),
   closureInfoPtr,
   entryCode,
   getConstrTag,
@@ -32,30 +31,27 @@ module GHC.Cmm.Info (
   stdPtrsOffset, stdNonPtrsOffset,
 ) where
 
-#include "HsVersions.h"
-
 import GHC.Prelude
 
 import GHC.Cmm
 import GHC.Cmm.Utils
 import GHC.Cmm.CLabel
+import GHC.StgToCmm.CgUtils (CgStream)
 import GHC.Runtime.Heap.Layout
 import GHC.Data.Bitmap
-import GHC.Data.Stream (Stream)
 import qualified GHC.Data.Stream as Stream
-import GHC.Cmm.Dataflow.Collections
+import GHC.Cmm.Dataflow.Label
 
 import GHC.Platform
 import GHC.Platform.Profile
 import GHC.Data.Maybe
-import GHC.Driver.Session
 import GHC.Utils.Error (withTimingSilent)
 import GHC.Utils.Panic
-import GHC.Types.Unique.Supply
 import GHC.Utils.Logger
 import GHC.Utils.Monad
 import GHC.Utils.Misc
 import GHC.Utils.Outputable
+import GHC.Types.Unique.DSM
 
 import Data.ByteString (ByteString)
 
@@ -68,20 +64,15 @@ mkEmptyContInfoTable info_lbl
                  , cit_srt  = Nothing
                  , cit_clo  = Nothing }
 
-cmmToRawCmm :: Logger -> DynFlags -> Stream IO CmmGroupSRTs a
-            -> IO (Stream IO RawCmmGroup a)
-cmmToRawCmm logger dflags cmms
-  = do {
-       ; let do_one :: [CmmDeclSRTs] -> IO [RawCmmDecl]
-             do_one cmm = do
-               uniqs <- mkSplitUniqSupply 'i'
+cmmToRawCmm :: Logger -> Profile -> CgStream CmmGroupSRTs a
+            -> IO (CgStream RawCmmGroup a)
+cmmToRawCmm logger profile cmms
+  = do { let do_one :: [CmmDeclSRTs] -> UniqDSMT IO [RawCmmDecl]
+             do_one cmm = setTagUDSMT 'i' $ do
                -- NB. strictness fixes a space leak.  DO NOT REMOVE.
-               withTimingSilent logger dflags (text "Cmm -> Raw Cmm")
-                          (\x -> seqList x ())
-                  -- TODO: It might be better to make `mkInfoTable` run in
-                  -- IO as well so we don't have to pass around
-                  -- a UniqSupply (see #16843)
-                 (return $ initUs_ uniqs $ concatMapM (mkInfoTable dflags) cmm)
+               withTimingSilent logger (text "Cmm -> Raw Cmm") (\x -> seqList x ()) $ do
+                 liftUniqDSM $
+                   concatMapM (mkInfoTable profile) cmm
        ; return (Stream.mapM do_one cmms)
        }
 
@@ -101,7 +92,7 @@ cmmToRawCmm logger dflags cmms
 --      <normal forward rest of StgInfoTable>
 --      <forward variable part>
 --
---      See includes/rts/storage/InfoTables.h
+--      See rts/include/rts/storage/InfoTables.h
 --
 -- For return-points these are as follows
 --
@@ -119,15 +110,15 @@ cmmToRawCmm logger dflags cmms
 --
 --  * The SRT slot is only there if there is SRT info to record
 
-mkInfoTable :: DynFlags -> CmmDeclSRTs -> UniqSM [RawCmmDecl]
+mkInfoTable :: Profile -> CmmDeclSRTs -> UniqDSM [RawCmmDecl]
 mkInfoTable _ (CmmData sec dat) = return [CmmData sec dat]
 
-mkInfoTable dflags proc@(CmmProc infos entry_lbl live blocks)
+mkInfoTable profile proc@(CmmProc infos entry_lbl live blocks)
   --
   -- in the non-tables-next-to-code case, procs can have at most a
   -- single info table associated with the entry label of the proc.
   --
-  | not (platformTablesNextToCode (targetPlatform dflags))
+  | not (platformTablesNextToCode platform)
   = case topInfoTable proc of   --  must be at most one
       -- no info table
       Nothing ->
@@ -135,7 +126,7 @@ mkInfoTable dflags proc@(CmmProc infos entry_lbl live blocks)
 
       Just info@CmmInfoTable { cit_lbl = info_lbl } -> do
         (top_decls, (std_info, extra_bits)) <-
-             mkInfoTableContents dflags info Nothing
+             mkInfoTableContents profile info Nothing
         let
           rel_std_info   = map (makeRelativeRefTo platform info_lbl) std_info
           rel_extra_bits = map (makeRelativeRefTo platform info_lbl) extra_bits
@@ -162,10 +153,10 @@ mkInfoTable dflags proc@(CmmProc infos entry_lbl live blocks)
             [CmmProc (mapFromList raw_infos) entry_lbl live blocks])
 
   where
-   platform = targetPlatform dflags
+   platform = profilePlatform profile
    do_one_info (lbl,itbl) = do
      (top_decls, (std_info, extra_bits)) <-
-         mkInfoTableContents dflags itbl Nothing
+         mkInfoTableContents profile itbl Nothing
      let
         info_lbl = cit_lbl itbl
         rel_std_info   = map (makeRelativeRefTo platform info_lbl) std_info
@@ -179,20 +170,20 @@ type InfoTableContents = ( [CmmLit]          -- The standard part
                          , [CmmLit] )        -- The "extra bits"
 -- These Lits have *not* had mkRelativeTo applied to them
 
-mkInfoTableContents :: DynFlags
+mkInfoTableContents :: Profile
                     -> CmmInfoTable
                     -> Maybe Int               -- Override default RTS type tag?
-                    -> UniqSM ([RawCmmDecl],             -- Auxiliary top decls
+                    -> UniqDSM ([RawCmmDecl],             -- Auxiliary top decls
                                InfoTableContents)       -- Info tbl + extra bits
 
-mkInfoTableContents dflags
+mkInfoTableContents profile
                     info@(CmmInfoTable { cit_lbl  = info_lbl
                                        , cit_rep  = smrep
                                        , cit_prof = prof
                                        , cit_srt = srt })
                     mb_rts_tag
   | RTSRep rts_tag rep <- smrep
-  = mkInfoTableContents dflags info{cit_rep = rep} (Just rts_tag)
+  = mkInfoTableContents profile info{cit_rep = rep} (Just rts_tag)
     -- Completely override the rts_tag that mkInfoTableContents would
     -- otherwise compute, with the rts_tag stored in the RTSRep
     -- (which in turn came from a handwritten .cmm file)
@@ -200,9 +191,9 @@ mkInfoTableContents dflags
   | StackRep frame <- smrep
   = do { (prof_lits, prof_data) <- mkProfLits platform prof
        ; let (srt_label, srt_bitmap) = mkSRTLit platform info_lbl srt
-       ; (liveness_lit, liveness_data) <- mkLivenessBits dflags frame
+       ; (liveness_lit, liveness_data) <- mkLivenessBits platform frame
        ; let
-             std_info = mkStdInfoTable dflags prof_lits rts_tag srt_bitmap liveness_lit
+             std_info = mkStdInfoTable profile prof_lits rts_tag srt_bitmap liveness_lit
              rts_tag | Just tag <- mb_rts_tag = tag
                      | null liveness_data     = rET_SMALL -- Fits in extra_bits
                      | otherwise              = rET_BIG   -- Does not; extra_bits is
@@ -215,18 +206,18 @@ mkInfoTableContents dflags
        ; let (srt_label, srt_bitmap) = mkSRTLit platform info_lbl srt
        ; (mb_srt_field, mb_layout, extra_bits, ct_data)
                                 <- mk_pieces closure_type srt_label
-       ; let std_info = mkStdInfoTable dflags prof_lits
+       ; let std_info = mkStdInfoTable profile prof_lits
                                        (mb_rts_tag   `orElse` rtsClosureType smrep)
                                        (mb_srt_field `orElse` srt_bitmap)
                                        (mb_layout    `orElse` layout)
        ; return (prof_data ++ ct_data, (std_info, extra_bits)) }
   where
-    platform = targetPlatform dflags
+    platform = profilePlatform profile
     mk_pieces :: ClosureTypeInfo -> [CmmLit]
-              -> UniqSM ( Maybe CmmLit  -- Override the SRT field with this
-                        , Maybe CmmLit  -- Override the layout field with this
-                        , [CmmLit]           -- "Extra bits" for info table
-                        , [RawCmmDecl])      -- Auxiliary data decls
+              -> UniqDSM ( Maybe CmmLit  -- Override the SRT field with this
+                         , Maybe CmmLit  -- Override the layout field with this
+                         , [CmmLit]           -- "Extra bits" for info table
+                         , [RawCmmDecl])      -- Auxiliary data decls
     mk_pieces (Constr con_tag con_descr) _no_srt    -- A data constructor
       = do { (descr_lit, decl) <- newStringLit con_descr
            ; return ( Just (CmmInt (fromIntegral con_tag)
@@ -246,7 +237,7 @@ mkInfoTableContents dflags
            ; return (Nothing, Nothing,  extra_bits, []) }
 
     mk_pieces (Fun arity (ArgGen arg_bits)) srt_label
-      = do { (liveness_lit, liveness_data) <- mkLivenessBits dflags arg_bits
+      = do { (liveness_lit, liveness_data) <- mkLivenessBits platform arg_bits
            ; let fun_type | null liveness_data = aRG_GEN
                           | otherwise          = aRG_GEN_BIG
                  extra_bits = [ packIntsCLit platform fun_type arity ]
@@ -257,7 +248,7 @@ mkInfoTableContents dflags
         slow_entry = CmmLabel (toSlowEntryLbl platform info_lbl)
         srt_lit = case srt_label of
                     []          -> mkIntCLit platform 0
-                    (lit:_rest) -> ASSERT( null _rest ) lit
+                    (lit:_rest) -> assert (null _rest) lit
 
     mk_pieces other _ = pprPanic "mk_pieces" (ppr other)
 
@@ -286,8 +277,7 @@ mkSRTLit platform _ (Just lbl) = ([CmmLabel lbl], CmmInt 1 (halfWordWidth platfo
 -- See the section "Referring to an SRT from the info table" in
 -- Note [SRTs] in "GHC.Cmm.Info.Build"
 inlineSRT :: Platform -> Bool
-inlineSRT platform = platformArch platform == ArchX86_64
-  && platformTablesNextToCode platform
+inlineSRT = pc_USE_INLINE_SRT_FIELD . platformConstants
 
 -------------------------------------------------------------------------
 --
@@ -344,14 +334,14 @@ makeRelativeRefTo platform info_lbl lit
 -- The head of the stack layout is the top of the stack and
 -- the least-significant bit.
 
-mkLivenessBits :: DynFlags -> Liveness -> UniqSM (CmmLit, [RawCmmDecl])
+mkLivenessBits :: Platform -> Liveness -> UniqDSM (CmmLit, [RawCmmDecl])
               -- ^ Returns:
               --   1. The bitmap (literal value or label)
               --   2. Large bitmap CmmData if needed
 
-mkLivenessBits dflags liveness
+mkLivenessBits platform liveness
   | n_bits > mAX_SMALL_BITMAP_SIZE platform -- does not fit in one word
-  = do { uniq <- getUniqueM
+  = do { uniq <- getUniqueDSM
        ; let bitmap_lbl = mkBitmapLabel uniq
        ; return (CmmLabel bitmap_lbl,
                  [mkRODataLits bitmap_lbl lits]) }
@@ -359,7 +349,6 @@ mkLivenessBits dflags liveness
   | otherwise -- Fits in one word
   = return (mkStgWordCLit platform bitmap_word, [])
   where
-    platform = targetPlatform dflags
     n_bits = length liveness
 
     bitmap :: Bitmap
@@ -375,7 +364,7 @@ mkLivenessBits dflags liveness
     lits = mkWordCLit platform (fromIntegral n_bits)
          : map (mkStgWordCLit platform) bitmap
       -- The first word is the size.  The structure must match
-      -- StgLargeBitmap in includes/rts/storage/InfoTable.h
+      -- StgLargeBitmap in rts/include/rts/storage/InfoTable.h
 
 -------------------------------------------------------------------------
 --
@@ -385,20 +374,20 @@ mkLivenessBits dflags liveness
 
 -- The standard bits of an info table.  This part of the info table
 -- corresponds to the StgInfoTable type defined in
--- includes/rts/storage/InfoTables.h.
+-- rts/include/rts/storage/InfoTables.h.
 --
 -- Its shape varies with ticky/profiling/tables next to code etc
 -- so we can't use constant offsets from Constants
 
 mkStdInfoTable
-   :: DynFlags
+   :: Profile
    -> (CmmLit,CmmLit)   -- Closure type descr and closure descr  (profiling)
    -> Int               -- Closure RTS tag
    -> CmmLit            -- SRT length
    -> CmmLit            -- layout field
    -> [CmmLit]
 
-mkStdInfoTable dflags (type_descr, closure_descr) cl_type srt layout_lit
+mkStdInfoTable profile (type_descr, closure_descr) cl_type srt layout_lit
  =      -- Parallel revertible-black hole field
     prof_info
         -- Ticky info (none at present)
@@ -406,9 +395,9 @@ mkStdInfoTable dflags (type_descr, closure_descr) cl_type srt layout_lit
  ++ [layout_lit, tag, srt]
 
  where
-    platform = targetPlatform dflags
+    platform = profilePlatform profile
     prof_info
-        | sccProfilingEnabled dflags = [type_descr, closure_descr]
+        | profileIsProfiling profile = [type_descr, closure_descr]
         | otherwise = []
 
     tag = CmmInt (fromIntegral cl_type) (halfWordWidth platform)
@@ -419,16 +408,16 @@ mkStdInfoTable dflags (type_descr, closure_descr) cl_type srt layout_lit
 --
 -------------------------------------------------------------------------
 
-mkProfLits :: Platform -> ProfilingInfo -> UniqSM ((CmmLit,CmmLit), [RawCmmDecl])
+mkProfLits :: Platform -> ProfilingInfo -> UniqDSM ((CmmLit,CmmLit), [RawCmmDecl])
 mkProfLits platform NoProfilingInfo = return ((zeroCLit platform, zeroCLit platform), [])
 mkProfLits _ (ProfilingInfo td cd)
   = do { (td_lit, td_decl) <- newStringLit td
        ; (cd_lit, cd_decl) <- newStringLit cd
        ; return ((td_lit,cd_lit), [td_decl,cd_decl]) }
 
-newStringLit :: ByteString -> UniqSM (CmmLit, GenCmmDecl RawCmmStatics info stmt)
+newStringLit :: ByteString -> UniqDSM (CmmLit, GenCmmDecl RawCmmStatics info stmt)
 newStringLit bytes
-  = do { uniq <- getUniqueM
+  = do { uniq <- getUniqueDSM
        ; return (mkByteStringCLit (mkStringLitLabel uniq) bytes) }
 
 
@@ -444,25 +433,19 @@ srtEscape platform = toStgHalfWord platform (-1)
 --
 -------------------------------------------------------------------------
 
-data PtrOpts = PtrOpts
-   { po_profile     :: !Profile -- ^ Platform profile
-   , po_align_check :: !Bool    -- ^ Insert alignment check (cf @-falignment-sanitisation@)
-   }
-
 -- | Wrap a 'CmmExpr' in an alignment check when @-falignment-sanitisation@ is
 -- enabled.
-wordAligned :: PtrOpts -> CmmExpr -> CmmExpr
-wordAligned opts e
-  | po_align_check opts
+wordAligned :: Platform -> DoAlignSanitisation -> CmmExpr -> CmmExpr
+wordAligned platform align_check e
+  | align_check
   = CmmMachOp (MO_AlignmentCheck (platformWordSizeInBytes platform) (wordWidth platform)) [e]
   | otherwise
   = e
-  where platform = profilePlatform (po_profile opts)
 
 -- | Takes a closure pointer and returns the info table pointer
-closureInfoPtr :: PtrOpts -> CmmExpr -> CmmExpr
-closureInfoPtr opts@(PtrOpts profile _) e =
-    cmmLoadBWord  (profilePlatform profile) (wordAligned opts e)
+closureInfoPtr :: Platform -> DoAlignSanitisation -> CmmExpr -> CmmExpr
+closureInfoPtr platform align_check e =
+    CmmMachOp (MO_RelaxedRead (wordWidth platform)) [wordAligned platform align_check e]
 
 -- | Takes an info pointer (the first word of a closure) and returns its entry
 -- code
@@ -476,23 +459,21 @@ entryCode platform e =
 -- constructor tag obtained from the info table
 -- This lives in the SRT field of the info table
 -- (constructors don't need SRTs).
-getConstrTag :: PtrOpts -> CmmExpr -> CmmExpr
-getConstrTag opts closure_ptr
+getConstrTag :: Profile -> DoAlignSanitisation -> CmmExpr -> CmmExpr
+getConstrTag profile align_check closure_ptr
   = CmmMachOp (MO_UU_Conv (halfWordWidth platform) (wordWidth platform)) [infoTableConstrTag profile info_table]
   where
-    info_table = infoTable profile (closureInfoPtr opts closure_ptr)
+    info_table = infoTable profile (closureInfoPtr platform align_check closure_ptr)
     platform   = profilePlatform profile
-    profile    = po_profile opts
 
 -- | Takes a closure pointer, and return the closure type
 -- obtained from the info table
-cmmGetClosureType :: PtrOpts -> CmmExpr -> CmmExpr
-cmmGetClosureType opts closure_ptr
+cmmGetClosureType :: Profile -> DoAlignSanitisation -> CmmExpr -> CmmExpr
+cmmGetClosureType profile align_check closure_ptr
   = CmmMachOp (MO_UU_Conv (halfWordWidth platform) (wordWidth platform)) [infoTableClosureType profile info_table]
   where
-    info_table = infoTable profile (closureInfoPtr opts closure_ptr)
+    info_table = infoTable profile (closureInfoPtr platform align_check closure_ptr)
     platform   = profilePlatform profile
-    profile    = po_profile opts
 
 -- | Takes an info pointer (the first word of a closure)
 -- and returns a pointer to the first word of the standard-form
