@@ -1,4 +1,3 @@
-{-# LANGUAGE CPP                        #-}
 {-# LANGUAGE DataKinds                  #-}
 {-# LANGUAGE GeneralizedNewtypeDeriving #-}
 {-# LANGUAGE Rank2Types                 #-}
@@ -42,17 +41,13 @@ module GHC.CmmToAsm.CFG
      )
 where
 
-#include "HsVersions.h"
-
 import GHC.Prelude
 import GHC.Platform
 
 import GHC.Cmm.BlockId
 import GHC.Cmm as Cmm
 
-import GHC.Cmm.Utils
 import GHC.Cmm.Switch
-import GHC.Cmm.Dataflow.Collections
 import GHC.Cmm.Dataflow.Label
 import GHC.Cmm.Dataflow.Block
 import qualified GHC.Cmm.Dataflow.Graph as G
@@ -64,12 +59,16 @@ import GHC.Data.Maybe
 import GHC.Types.Unique
 import qualified GHC.CmmToAsm.CFG.Dominators as Dom
 import GHC.CmmToAsm.CFG.Weight
+import GHC.Data.Word64Map.Strict (Word64Map)
+import GHC.Data.Word64Set (Word64Set)
 import Data.IntMap.Strict (IntMap)
 import Data.IntSet (IntSet)
 
 import qualified Data.IntMap.Strict as IM
+import qualified GHC.Data.Word64Map.Strict as WM
 import qualified Data.Map as M
 import qualified Data.IntSet as IS
+import qualified GHC.Data.Word64Set as WS
 import qualified Data.Set as S
 import Data.Tree
 import Data.Bifunctor
@@ -80,7 +79,6 @@ import GHC.Utils.Panic
 --import GHC.Cmm.DebugBlock
 --import GHC.Data.OrdList
 --import GHC.Cmm.DebugBlock.Trace
-import GHC.Cmm.Ppr () -- For Outputable instances
 
 import Data.List (sort, nub, partition)
 import Data.STRef.Strict
@@ -94,6 +92,7 @@ import Data.Array.Base (unsafeRead, unsafeWrite)
 
 import Control.Monad
 import GHC.Data.UnionFind
+import Data.Word
 
 type Prob = Double
 
@@ -152,7 +151,7 @@ instance Outputable CfgEdge where
 -- or has it been introduced during assembly codegen. We use this to maintain
 -- some information which would otherwise be lost during the
 -- Cmm \<-> asm transition.
--- See also Note [Inverting Conditional Branches]
+-- See also Note [Inverting conditions]
 data TransitionSource
   = CmmSource { trans_cmmNode :: (CmmNode O C)
               , trans_info :: BranchInfo }
@@ -215,7 +214,7 @@ getCfgNodes m =
 hasNode :: CFG -> BlockId -> Bool
 hasNode m node =
   -- Check the invariant that each node must exist in the first map or not at all.
-  ASSERT( found || not (any (mapMember node) m))
+  assert (found || not (any (mapMember node) m))
   found
     where
       found = mapMember node m
@@ -223,7 +222,7 @@ hasNode m node =
 
 
 -- | Check if the nodes in the cfg and the set of blocks are the same.
---   In a case of a missmatch we panic and show the difference.
+--   In a case of a mismatch we panic and show the difference.
 sanityCheckCfg :: CFG -> LabelSet -> SDoc -> Bool
 sanityCheckCfg m blockSet msg
     | blockSet == cfgNodes
@@ -240,7 +239,7 @@ sanityCheckCfg m blockSet msg
       diff = (setUnion cfgNodes blockSet) `setDifference` (setIntersection cfgNodes blockSet) :: LabelSet
 
 -- | Filter the CFG with a custom function f.
---   Paramaeters are `f from to edgeInfo`
+--   Parameters are `f from to edgeInfo`
 filterEdges :: (BlockId -> BlockId -> EdgeInfo -> Bool) -> CFG -> CFG
 filterEdges f cfg =
     mapMapWithKey filterSources cfg
@@ -250,7 +249,7 @@ filterEdges f cfg =
 
 
 {- Note [Updating the CFG during shortcutting]
-
+   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 See Note [What is shortcutting] in the control flow optimization
 code (GHC.Cmm.ContFlowOpt) for a slightly more in depth explanation on shortcutting.
 
@@ -311,9 +310,9 @@ shortcutWeightMap cuts cfg
         -- Create a unification variable for each of the nodes in a rewrite
         cuts_vars <- traverse (\p -> (p,) <$> fresh (Just p)) (concatMap (\(a, b) -> [a] ++ maybe [] (:[]) b) cuts_list)
         let cuts_map = mapFromList cuts_vars :: LabelMap (Point s (Maybe BlockId))
-        -- Then unify according the the rewrites in the cuts map
-        mapM_ (\(from, to) -> expectJust "shortcutWeightMap" (mapLookup from cuts_map)
-                              `union` expectJust "shortcutWeightMap" (maybe (Just null) (flip mapLookup cuts_map) to) ) cuts_list
+        -- Then unify according to the rewrites in the cuts map
+        mapM_ (\(from, to) -> expectJust (mapLookup from cuts_map)
+                              `union` expectJust (maybe (Just null) (flip mapLookup cuts_map) to) ) cuts_list
         -- Then recover the unique representative, which is the result of following
         -- the chain to the end.
         mapM find cuts_map
@@ -389,10 +388,7 @@ addWeightEdge from to weight cfg =
 
 delEdge :: BlockId -> BlockId -> CFG -> CFG
 delEdge from to m =
-    mapAlter remDest from m
-    where
-        remDest Nothing = Nothing
-        remDest (Just wm) = Just $ mapDelete to wm
+    mapAdjust (mapDelete to) from m
 
 
 -- | Destinations from bid ordered by weight (descending)
@@ -420,13 +416,10 @@ getEdgeInfo from to m
     = Nothing
 
 getEdgeWeight :: CFG -> BlockId -> BlockId -> EdgeWeight
-getEdgeWeight cfg from to =
-    edgeWeight $ expectJust "Edgeweight for noexisting block" $
-                 getEdgeInfo from to cfg
+getEdgeWeight cfg from to = edgeWeight $ expectJust $ getEdgeInfo from to cfg
 
 getTransitionSource :: BlockId -> BlockId -> CFG -> TransitionSource
-getTransitionSource from to cfg = transitionSource $ expectJust "Source info for noexisting block" $
-                        getEdgeInfo from to cfg
+getTransitionSource from to cfg = transitionSource $ expectJust $ getEdgeInfo from to cfg
 
 reverseEdges :: CFG -> CFG
 reverseEdges cfg = mapFoldlWithKey (\cfg from toMap -> go (addNode cfg from) from toMap) mapEmpty cfg
@@ -604,7 +597,7 @@ addNodesBetween weights m updates =
 
 -}
 -- | Generate weights for a Cmm proc based on some simple heuristics.
-getCfgProc :: Platform -> Weights -> RawCmmDecl -> CFG
+getCfgProc :: Platform -> Weights -> GenCmmDecl d h CmmGraph -> CFG
 getCfgProc _        _       (CmmData {}) = mapEmpty
 getCfgProc platform weights (CmmProc _info _lab _live graph) = getCfg platform weights graph
 
@@ -667,8 +660,8 @@ getCfg platform weights graph =
         (CmmCall { cml_cont = Nothing })   -> []
         other ->
             panic "Foo" $
-            ASSERT2(False, ppr "Unknown successor cause:" <>
-              (pdoc platform branch <+> text "=>" <> pdoc platform (G.successors other)))
+            assertPpr False (text "Unknown successor cause:" <>
+              (pdoc platform branch <+> text "=>" <> pdoc platform (G.successors other))) $
             map (\x -> ((bid,x),mkEdgeInfo 0)) $ G.successors other
       where
         bid = G.entryLabel block
@@ -714,7 +707,7 @@ optHsPatterns weights (CmmProc info _lab _live graph) cfg =
     increaseBackEdgeWeight (g_entry graph) $ cfg
   where
 
-    -- | Increase the weight of all backedges in the CFG
+    -- Increase the weight of all backedges in the CFG
     -- this helps to make loop jumpbacks the heaviest edges
     increaseBackEdgeWeight :: BlockId -> CFG -> CFG
     increaseBackEdgeWeight root cfg =
@@ -727,7 +720,7 @@ optHsPatterns weights (CmmProc info _lab _live graph) cfg =
         in  foldl'  (\cfg edge -> updateEdgeWeight update edge cfg)
                     cfg backedges
 
-    -- | Since we cant fall through info tables we penalize these.
+    -- Since we cant fall through info tables we penalize these.
     penalizeInfoTables :: LabelMap a -> CFG -> CFG
     penalizeInfoTables info cfg =
         mapWeights fupdate cfg
@@ -738,7 +731,7 @@ optHsPatterns weights (CmmProc info _lab _live graph) cfg =
           = weight - (fromIntegral $ infoTablePenalty weights)
           | otherwise = weight
 
-    -- | If a block has two successors, favour the one with fewer
+    -- If a block has two successors, favour the one with fewer
     -- predecessors and/or the one allowing fall through.
     favourFewerPreds :: CFG -> CFG
     favourFewerPreds cfg =
@@ -858,7 +851,7 @@ loopInfo cfg root = LoopInfo  { liBackEdges = backEdges
 
     --TODO - This should be a no op: Export constructors? Use unsafeCoerce? ...
     rooted = ( fromBlockId root
-              , toIntMap $ fmap toIntSet graph) :: (Int, IntMap IntSet)
+              , toWord64Map $ fmap toWord64Set graph) :: (Word64, Word64Map Word64Set)
     tree = fmap toBlockId $ Dom.domTree rooted :: Tree BlockId
 
     -- Map from Nodes to their dominators
@@ -905,10 +898,10 @@ loopInfo cfg root = LoopInfo  { liBackEdges = backEdges
           loopCount n = length $ nub . map fst . filter (setMember n . snd) $ bodies
       in  map (\n -> (n, loopCount n)) $ nodes :: [(BlockId, Int)]
 
-    toIntSet :: LabelSet -> IntSet
-    toIntSet s = IS.fromList . map fromBlockId . setElems $ s
-    toIntMap :: LabelMap a -> IntMap a
-    toIntMap m = IM.fromList $ map (\(x,y) -> (fromBlockId x,y)) $ mapToList m
+    toWord64Set :: LabelSet -> Word64Set
+    toWord64Set s = WS.fromList . map fromBlockId . setElems $ s
+    toWord64Map :: LabelMap a -> Word64Map a
+    toWord64Map m = WM.fromList $ map (\(x,y) -> (fromBlockId x,y)) $ mapToList m
 
     mkDomMap :: Tree BlockId -> LabelMap LabelSet
     mkDomMap root = mapFromList $ go setEmpty root
@@ -923,10 +916,10 @@ loopInfo cfg root = LoopInfo  { liBackEdges = backEdges
                             (\n -> go (setInsert (rootLabel n) parents) n)
                             leaves
 
-    fromBlockId :: BlockId -> Int
+    fromBlockId :: BlockId -> Word64
     fromBlockId = getKey . getUnique
 
-    toBlockId :: Int -> BlockId
+    toBlockId :: Word64 -> BlockId
     toBlockId = mkBlockId . mkUniqueGrimily
 
 -- We make the CFG a Hoopl Graph, so we can reuse revPostOrder.
@@ -1008,14 +1001,13 @@ mkGlobalWeights root localCfg
     blockMapping = listArray (0,mapSize vertexMapping - 1) revOrder :: Array Int BlockId
     -- Map from blockId to indices starting at zero
     toVertex :: BlockId -> Int
-    toVertex   blockId  = expectJust "mkGlobalWeights" $ mapLookup blockId vertexMapping
+    toVertex   blockId  = expectJust $ mapLookup blockId vertexMapping
     -- Map from indices starting at zero to blockIds
     fromVertex :: Int -> BlockId
     fromVertex vertex   = blockMapping ! vertex
 
 {- Note [Static Branch Prediction]
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-
 The work here has been based on the paper
 "Static Branch Prediction and Program Profile Analysis" by Y Wu, JR Larus.
 
@@ -1028,11 +1020,11 @@ Currently implemented is a heuristic to predict that we do not exit
 loops (lehPredicts) and one to predict that backedges are more likely
 than any other edge.
 
-The back edge case is special as it superceeds any other heuristic if it
+The back edge case is special as it supersedes any other heuristic if it
 applies.
 
 Do NOT rely solely on nofib results for benchmarking this. I recommend at least
-comparing megaparsec and container benchmarks. Nofib does not seeem to have
+comparing megaparsec and container benchmarks. Nofib does not seem to have
 many instances of "loopy" Cmm where these make a difference.
 
 TODO:
@@ -1353,7 +1345,7 @@ calcFreqs graph backEdges loops revPostOrder = runST $ do
                                 vcat (map (\(k,m) -> ppr (k,m :: IM.IntMap Double)) $ IM.toList g)
                             )
 
-    nodeCount = IM.foldl' (\count toMap -> IM.foldlWithKey' countTargets count toMap) (IM.size graph) graph
+    nodeCount = IM.foldl' (\count toMap -> IM.foldlWithKey' countTargets (count + 1) toMap) 0 graph
       where
         countTargets = (\count k _ -> countNode k + count )
         countNode n = if IM.member n graph then 0 else 1
