@@ -22,23 +22,20 @@ import GHC.CmmToAsm.PPC.Cond
 import GHC.CmmToAsm.Ppr
 import GHC.CmmToAsm.Format
 import GHC.Platform.Reg
-import GHC.Platform.Reg.Class
+import GHC.Platform.Reg.Class.Unified
 import GHC.CmmToAsm.Reg.Target
 import GHC.CmmToAsm.Config
 import GHC.CmmToAsm.Types
 import GHC.CmmToAsm.Utils
 
 import GHC.Cmm hiding (topInfoTable)
-import GHC.Cmm.Dataflow.Collections
 import GHC.Cmm.Dataflow.Label
 
 import GHC.Cmm.BlockId
 import GHC.Cmm.CLabel
-import GHC.Cmm.Ppr.Expr () -- For Outputable instances
 
 import GHC.Types.Unique ( pprUniqueAlways, getUnique )
 import GHC.Platform
-import GHC.Data.FastString
 import GHC.Utils.Outputable
 import GHC.Utils.Panic
 
@@ -48,7 +45,7 @@ import Data.Int
 -- -----------------------------------------------------------------------------
 -- Printing this stuff out
 
-pprNatCmmDecl :: NCGConfig -> NatCmmDecl RawCmmStatics Instr -> SDoc
+pprNatCmmDecl :: IsDoc doc => NCGConfig -> NatCmmDecl RawCmmStatics Instr -> doc
 pprNatCmmDecl config (CmmData section dats) =
   pprSectionAlign config section
   $$ pprDatas (ncgPlatform config) dats
@@ -65,15 +62,15 @@ pprNatCmmDecl config proc@(CmmProc top_info lbl _ (ListGraph blocks)) =
             _ -> pprLabel platform lbl) $$ -- blocks guaranteed not null,
                                            -- so label needed
          vcat (map (pprBasicBlock config top_info) blocks) $$
-         ppWhen (ncgDwarfEnabled config) (pdoc platform (mkAsmTempEndLabel lbl)
-                                          <> char ':' $$
-                                          pprProcEndLabel platform lbl) $$
+         ppWhen (ncgDwarfEnabled config) (line (pprAsmLabel platform (mkAsmTempEndLabel lbl)
+                                                <> char ':') $$
+                                          line (pprProcEndLabel platform lbl)) $$
          pprSizeDecl platform lbl
 
     Just (CmmStaticsRaw info_lbl _) ->
       pprSectionAlign config (Section Text info_lbl) $$
       (if platformHasSubsectionsViaSymbols platform
-          then pdoc platform (mkDeadStripPreventer info_lbl) <> char ':'
+          then line (pprAsmLabel platform (mkDeadStripPreventer info_lbl) <> char ':')
           else empty) $$
       vcat (map (pprBasicBlock config top_info) blocks) $$
       -- above: Even the first block gets a label, because with branch-chain
@@ -81,66 +78,66 @@ pprNatCmmDecl config proc@(CmmProc top_info lbl _ (ListGraph blocks)) =
       (if platformHasSubsectionsViaSymbols platform
        then
        -- See Note [Subsections Via Symbols] in X86/Ppr.hs
-                text "\t.long "
-            <+> pdoc platform info_lbl
-            <+> char '-'
-            <+> pdoc platform (mkDeadStripPreventer info_lbl)
+                line (text "\t.long "
+                      <+> pprAsmLabel platform info_lbl
+                      <+> char '-'
+                      <+> pprAsmLabel platform (mkDeadStripPreventer info_lbl))
        else empty) $$
       pprSizeDecl platform info_lbl
+{-# SPECIALIZE pprNatCmmDecl :: NCGConfig -> NatCmmDecl RawCmmStatics Instr -> SDoc #-}
+{-# SPECIALIZE pprNatCmmDecl :: NCGConfig -> NatCmmDecl RawCmmStatics Instr -> HDoc #-} -- see Note [SPECIALIZE to HDoc] in GHC.Utils.Outputable
 
 -- | Output the ELF .size directive.
-pprSizeDecl :: Platform -> CLabel -> SDoc
+pprSizeDecl :: IsDoc doc => Platform -> CLabel -> doc
 pprSizeDecl platform lbl
  = if osElfTarget (platformOS platform)
-   then text "\t.size" <+> prettyLbl <> text ", .-" <> codeLbl
+   then line (text "\t.size" <+> prettyLbl <> text ", .-" <> codeLbl)
    else empty
   where
-    prettyLbl = pdoc platform lbl
+    prettyLbl = pprAsmLabel platform lbl
     codeLbl
       | platformArch platform == ArchPPC_64 ELF_V1 = char '.' <> prettyLbl
       | otherwise                                  = prettyLbl
 
-pprFunctionDescriptor :: Platform -> CLabel -> SDoc
-pprFunctionDescriptor platform lab = pprGloblDecl platform lab
-                        $$  text "\t.section \".opd\", \"aw\""
-                        $$  text "\t.align 3"
-                        $$  pdoc platform lab <> char ':'
-                        $$  text "\t.quad ."
-                        <>  pdoc platform lab
-                        <>  text ",.TOC.@tocbase,0"
-                        $$  text "\t.previous"
-                        $$  text "\t.type"
-                        <+> pdoc platform lab
-                        <>  text ", @function"
-                        $$  char '.' <> pdoc platform lab <> char ':'
+pprFunctionDescriptor :: IsDoc doc => Platform -> CLabel -> doc
+pprFunctionDescriptor platform lab =
+  vcat [pprGloblDecl platform lab,
+        line (text "\t.section \".opd\", \"aw\""),
+        line (text "\t.align 3"),
+        line (pprAsmLabel platform lab <> char ':'),
+        line (text "\t.quad ."
+              <>  pprAsmLabel platform lab
+              <>  text ",.TOC.@tocbase,0"),
+        line (text "\t.previous"),
+        line (text "\t.type"
+              <+> pprAsmLabel platform lab
+              <>  text ", @function"),
+        line (char '.' <> pprAsmLabel platform lab <> char ':')]
 
-pprFunctionPrologue :: Platform -> CLabel ->SDoc
-pprFunctionPrologue platform lab =  pprGloblDecl platform lab
-                        $$  text ".type "
-                        <> pdoc platform lab
-                        <> text ", @function"
-                        $$ pdoc platform lab <> char ':'
-                        $$ text "0:\taddis\t" <> pprReg toc
-                        <> text ",12,.TOC.-0b@ha"
-                        $$ text "\taddi\t" <> pprReg toc
-                        <> char ',' <> pprReg toc <> text ",.TOC.-0b@l"
-                        $$ text "\t.localentry\t" <> pdoc platform lab
-                        <> text ",.-" <> pdoc platform lab
+pprFunctionPrologue :: IsDoc doc => Platform -> CLabel -> doc
+pprFunctionPrologue platform lab =
+  vcat [pprGloblDecl platform lab,
+        line (text ".type " <> pprAsmLabel platform lab <> text ", @function"),
+        line (pprAsmLabel platform lab <> char ':'),
+        line (text "0:\taddis\t" <> pprReg toc <> text ",12,.TOC.-0b@ha"),
+        line (text "\taddi\t" <> pprReg toc <> char ',' <> pprReg toc <> text ",.TOC.-0b@l"),
+        line (text "\t.localentry\t" <> pprAsmLabel platform lab <>
+              text ",.-" <> pprAsmLabel platform lab)]
 
-pprProcEndLabel :: Platform -> CLabel -- ^ Procedure name
-                -> SDoc
+pprProcEndLabel :: IsLine doc => Platform -> CLabel -- ^ Procedure name
+                -> doc
 pprProcEndLabel platform lbl =
-    pdoc platform (mkAsmTempProcEndLabel lbl) <> char ':'
+    pprAsmLabel platform (mkAsmTempProcEndLabel lbl) <> char ':'
 
-pprBasicBlock :: NCGConfig -> LabelMap RawCmmStatics -> NatBasicBlock Instr
-              -> SDoc
+pprBasicBlock :: IsDoc doc => NCGConfig -> LabelMap RawCmmStatics -> NatBasicBlock Instr
+              -> doc
 pprBasicBlock config info_env (BasicBlock blockid instrs)
   = maybe_infotable $$
     pprLabel platform asmLbl $$
     vcat (map (pprInstr platform) instrs) $$
     ppWhen (ncgDwarfEnabled config) (
-      pdoc platform (mkAsmTempEndLabel asmLbl) <> char ':'
-      <> pprProcEndLabel platform asmLbl
+      line (pprAsmLabel platform (mkAsmTempEndLabel asmLbl) <> char ':'
+            <> pprProcEndLabel platform asmLbl)
     )
   where
     asmLbl = blockLbl blockid
@@ -154,8 +151,8 @@ pprBasicBlock config info_env (BasicBlock blockid instrs)
 
 
 
-pprDatas :: Platform -> RawCmmStatics -> SDoc
--- See note [emit-time elimination of static indirections] in "GHC.Cmm.CLabel".
+pprDatas :: IsDoc doc => Platform -> RawCmmStatics -> doc
+-- See Note [emit-time elimination of static indirections] in "GHC.Cmm.CLabel".
 pprDatas platform (CmmStaticsRaw alias [CmmStaticLit (CmmLabel lbl), CmmStaticLit ind, _, _])
   | lbl == mkIndStaticInfoLabel
   , let labelInd (CmmLabelOff l _) = Just l
@@ -164,50 +161,49 @@ pprDatas platform (CmmStaticsRaw alias [CmmStaticLit (CmmLabel lbl), CmmStaticLi
   , Just ind' <- labelInd ind
   , alias `mayRedirectTo` ind'
   = pprGloblDecl platform alias
-    $$ text ".equiv" <+> pdoc platform alias <> comma <> pdoc platform (CmmLabel ind')
+    $$ line (text ".equiv" <+> pprAsmLabel platform alias <> comma <> pprAsmLabel platform ind')
 pprDatas platform (CmmStaticsRaw lbl dats) = vcat (pprLabel platform lbl : map (pprData platform) dats)
 
-pprData :: Platform -> CmmStatic -> SDoc
+pprData :: IsDoc doc => Platform -> CmmStatic -> doc
 pprData platform d = case d of
-   CmmString str          -> pprString str
-   CmmFileEmbed path      -> pprFileEmbed path
-   CmmUninitialised bytes -> text ".space " <> int bytes
+   CmmString str          -> line (pprString str)
+   CmmFileEmbed path _    -> line (pprFileEmbed path)
+   CmmUninitialised bytes -> line (text ".space " <> int bytes)
    CmmStaticLit lit       -> pprDataItem platform lit
 
-pprGloblDecl :: Platform -> CLabel -> SDoc
+pprGloblDecl :: IsDoc doc => Platform -> CLabel -> doc
 pprGloblDecl platform lbl
   | not (externallyVisibleCLabel lbl) = empty
-  | otherwise = text ".globl " <> pdoc platform lbl
+  | otherwise = line (text ".globl " <> pprAsmLabel platform lbl)
 
-pprTypeAndSizeDecl :: Platform -> CLabel -> SDoc
+pprTypeAndSizeDecl :: IsLine doc => Platform -> CLabel -> doc
 pprTypeAndSizeDecl platform lbl
   = if platformOS platform == OSLinux && externallyVisibleCLabel lbl
     then text ".type " <>
-         pdoc platform lbl <> text ", @object"
+         pprAsmLabel platform lbl <> text ", @object"
     else empty
 
-pprLabel :: Platform -> CLabel -> SDoc
+pprLabel :: IsDoc doc => Platform -> CLabel -> doc
 pprLabel platform lbl =
    pprGloblDecl platform lbl
-   $$ pprTypeAndSizeDecl platform lbl
-   $$ (pdoc platform lbl <> char ':')
+   $$ line (pprTypeAndSizeDecl platform lbl)
+   $$ line (pprAsmLabel platform lbl <> char ':')
 
 -- -----------------------------------------------------------------------------
 -- pprInstr: print an 'Instr'
 
-pprReg :: Reg -> SDoc
+pprReg :: forall doc. IsLine doc => Reg -> doc
 
 pprReg r
   = case r of
-      RegReal    (RealRegSingle i) -> ppr_reg_no i
-      RegReal    (RealRegPair{})   -> panic "PPC.pprReg: no reg pairs on this arch"
-      RegVirtual (VirtualRegI  u)  -> text "%vI_"   <> pprUniqueAlways u
-      RegVirtual (VirtualRegHi u)  -> text "%vHi_"  <> pprUniqueAlways u
-      RegVirtual (VirtualRegF  u)  -> text "%vF_"   <> pprUniqueAlways u
-      RegVirtual (VirtualRegD  u)  -> text "%vD_"   <> pprUniqueAlways u
+      RegReal    (RealRegSingle  i) -> ppr_reg_no i
+      RegVirtual (VirtualRegI    u) -> text "%vI_"   <> pprUniqueAlways u
+      RegVirtual (VirtualRegHi   u) -> text "%vHi_"  <> pprUniqueAlways u
+      RegVirtual (VirtualRegD    u) -> text "%vD_"   <> pprUniqueAlways u
+      RegVirtual (VirtualRegV128 u) -> text "%vV128_" <> pprUniqueAlways u
 
   where
-    ppr_reg_no :: Int -> SDoc
+    ppr_reg_no :: Int -> doc
     ppr_reg_no i
          | i <= 31   = int i      -- GPRs
          | i <= 63   = int (i-32) -- FPRs
@@ -215,35 +211,35 @@ pprReg r
 
 
 
-pprFormat :: Format -> SDoc
+pprFormat :: IsLine doc => Format -> doc
 pprFormat x
- = ptext (case x of
-                II8  -> sLit "b"
-                II16 -> sLit "h"
-                II32 -> sLit "w"
-                II64 -> sLit "d"
-                FF32 -> sLit "fs"
-                FF64 -> sLit "fd")
+ = case x of
+                II8  -> text "b"
+                II16 -> text "h"
+                II32 -> text "w"
+                II64 -> text "d"
+                FF32 -> text "fs"
+                FF64 -> text "fd"
+                VecFormat {} -> panic "PPC pprFormat: VecFormat"
 
-
-pprCond :: Cond -> SDoc
+pprCond :: IsLine doc => Cond -> doc
 pprCond c
- = ptext (case c of {
-                ALWAYS  -> sLit "";
-                EQQ     -> sLit "eq";  NE    -> sLit "ne";
-                LTT     -> sLit "lt";  GE    -> sLit "ge";
-                GTT     -> sLit "gt";  LE    -> sLit "le";
-                LU      -> sLit "lt";  GEU   -> sLit "ge";
-                GU      -> sLit "gt";  LEU   -> sLit "le"; })
+ = case c of {
+                ALWAYS  -> text "";
+                EQQ     -> text "eq";  NE    -> text "ne";
+                LTT     -> text "lt";  GE    -> text "ge";
+                GTT     -> text "gt";  LE    -> text "le";
+                LU      -> text "lt";  GEU   -> text "ge";
+                GU      -> text "gt";  LEU   -> text "le"; }
 
 
-pprImm :: Platform -> Imm -> SDoc
+pprImm :: IsLine doc => Platform -> Imm -> doc
 pprImm platform = \case
    ImmInt i       -> int i
    ImmInteger i   -> integer i
-   ImmCLbl l      -> pdoc platform l
-   ImmIndex l i   -> pdoc platform l <> char '+' <> int i
-   ImmLit s       -> s
+   ImmCLbl l      -> pprAsmLabel platform l
+   ImmIndex l i   -> pprAsmLabel platform l <> char '+' <> int i
+   ImmLit s       -> ftext s
    ImmFloat f     -> float $ fromRational f
    ImmDouble d    -> double $ fromRational d
    ImmConstantSum a b   -> pprImm platform a <> char '+' <> pprImm platform b
@@ -267,7 +263,7 @@ pprImm platform = \case
    HIGHESTA i  -> pprImm platform i <> text "@highesta"
 
 
-pprAddr :: Platform -> AddrMode -> SDoc
+pprAddr :: IsLine doc => Platform -> AddrMode -> doc
 pprAddr platform = \case
    AddrRegReg r1 r2             -> pprReg r1 <> char ',' <+> pprReg r2
    AddrRegImm r1 (ImmInt i)     -> hcat [ int i, char '(', pprReg r1, char ')' ]
@@ -275,40 +271,41 @@ pprAddr platform = \case
    AddrRegImm r1 imm            -> hcat [ pprImm platform imm, char '(', pprReg r1, char ')' ]
 
 
-pprSectionAlign :: NCGConfig -> Section -> SDoc
+pprSectionAlign :: IsDoc doc => NCGConfig -> Section -> doc
 pprSectionAlign config sec@(Section seg _) =
-   pprSectionHeader config sec $$
+   line (pprSectionHeader config sec) $$
    pprAlignForSection (ncgPlatform config) seg
 
 -- | Print appropriate alignment for the given section type.
-pprAlignForSection :: Platform -> SectionType -> SDoc
-pprAlignForSection platform seg =
+pprAlignForSection :: IsDoc doc => Platform -> SectionType -> doc
+pprAlignForSection platform seg = line $
  let ppc64    = not $ target32Bit platform
- in ptext $ case seg of
-       Text              -> sLit ".align 2"
+ in case seg of
+       Text              -> text ".align 2"
        Data
-        | ppc64          -> sLit ".align 3"
-        | otherwise      -> sLit ".align 2"
+        | ppc64          -> text ".align 3"
+        | otherwise      -> text ".align 2"
        ReadOnlyData
-        | ppc64          -> sLit ".align 3"
-        | otherwise      -> sLit ".align 2"
+        | ppc64          -> text ".align 3"
+        | otherwise      -> text ".align 2"
        RelocatableReadOnlyData
-        | ppc64          -> sLit ".align 3"
-        | otherwise      -> sLit ".align 2"
+        | ppc64          -> text ".align 3"
+        | otherwise      -> text ".align 2"
        UninitialisedData
-        | ppc64          -> sLit ".align 3"
-        | otherwise      -> sLit ".align 2"
-       ReadOnlyData16    -> sLit ".align 4"
+        | ppc64          -> text ".align 3"
+        | otherwise      -> text ".align 2"
        -- TODO: This is copied from the ReadOnlyData case, but it can likely be
        -- made more efficient.
+       InitArray         -> text ".align 3"
+       FiniArray         -> text ".align 3"
        CString
-        | ppc64          -> sLit ".align 3"
-        | otherwise      -> sLit ".align 2"
+        | ppc64          -> text ".align 3"
+        | otherwise      -> text ".align 2"
        OtherSection _    -> panic "PprMach.pprSectionAlign: unknown section"
 
-pprDataItem :: Platform -> CmmLit -> SDoc
+pprDataItem :: IsDoc doc => Platform -> CmmLit -> doc
 pprDataItem platform lit
-  = vcat (ppr_item (cmmTypeFormat $ cmmLitType platform lit) lit)
+  = lines_ (ppr_item (cmmTypeFormat $ cmmLitType platform lit) lit)
     where
         imm = litToImm lit
         archPPC_64 = not $ target32Bit platform
@@ -335,22 +332,21 @@ pprDataItem platform lit
                 = panic "PPC.Ppr.pprDataItem: no match"
 
 
-pprInstr :: Platform -> Instr -> SDoc
+asmComment :: IsLine doc => doc -> doc
+asmComment c = whenPprDebug $ text "#" <+> c
+
+
+pprInstr :: IsDoc doc => Platform -> Instr -> doc
 pprInstr platform instr = case instr of
 
-   COMMENT _
-      -> empty -- nuke 'em
+   COMMENT s
+      -> line (asmComment (ftext s))
 
-   -- COMMENT s
-   --    -> if platformOS platform == OSLinux
-   --          then text "# " <> ftext s
-   --          else text "; " <> ftext s
-
-   LOCATION file line col _name
-      -> text "\t.loc" <+> ppr file <+> ppr line <+> ppr col
+   LOCATION file line' col _name
+      -> line (text "\t.loc" <+> int file <+> int line' <+> int col)
 
    DELTA d
-      -> pprInstr platform (COMMENT (mkFastString ("\tdelta = " ++ show d)))
+      -> line (asmComment $ text ("\tdelta = " ++ show d))
 
    NEWBLOCK _
       -> panic "PprMach.pprInstr: NEWBLOCK"
@@ -377,16 +373,17 @@ pprInstr platform instr = case instr of
 -}
 
    LD fmt reg addr
-      -> hcat [
+      -> line $ hcat [
            char '\t',
            text "l",
-           ptext (case fmt of
-               II8  -> sLit "bz"
-               II16 -> sLit "hz"
-               II32 -> sLit "wz"
-               II64 -> sLit "d"
-               FF32 -> sLit "fs"
-               FF64 -> sLit "fd"
+           (case fmt of
+               II8  -> text "bz"
+               II16 -> text "hz"
+               II32 -> text "wz"
+               II64 -> text "d"
+               FF32 -> text "fs"
+               FF64 -> text "fd"
+               VecFormat {} -> panic "PPC pprInstr: VecFormat"
                ),
            case addr of AddrRegImm _ _ -> empty
                         AddrRegReg _ _ -> char 'x',
@@ -406,7 +403,7 @@ pprInstr platform instr = case instr of
       -> panic "PPC.Ppr.pprInstr LDFAR: no match"
 
    LDR fmt reg1 addr
-      -> hcat [
+      -> line $ hcat [
            text "\tl",
            case fmt of
              II32 -> char 'w'
@@ -419,16 +416,17 @@ pprInstr platform instr = case instr of
            ]
 
    LA fmt reg addr
-      -> hcat [
+      -> line $ hcat [
            char '\t',
            text "l",
-           ptext (case fmt of
-               II8  -> sLit "ba"
-               II16 -> sLit "ha"
-               II32 -> sLit "wa"
-               II64 -> sLit "d"
-               FF32 -> sLit "fs"
-               FF64 -> sLit "fd"
+           (case fmt of
+               II8  -> text "ba"
+               II16 -> text "ha"
+               II32 -> text "wa"
+               II64 -> text "d"
+               FF32 -> text "fs"
+               FF64 -> text "fd"
+               VecFormat {} -> panic "PPC pprInstr: VecFormat"
                ),
            case addr of AddrRegImm _ _ -> empty
                         AddrRegReg _ _ -> char 'x',
@@ -439,7 +437,7 @@ pprInstr platform instr = case instr of
            ]
 
    ST fmt reg addr
-      -> hcat [
+      -> line $ hcat [
            char '\t',
            text "st",
            pprFormat fmt,
@@ -460,7 +458,7 @@ pprInstr platform instr = case instr of
       -> panic "PPC.Ppr.pprInstr STFAR: no match"
 
    STU fmt reg addr
-      -> hcat [
+      -> line $ hcat [
            char '\t',
            text "st",
            pprFormat fmt,
@@ -474,7 +472,7 @@ pprInstr platform instr = case instr of
            ]
 
    STC fmt reg1 addr
-      -> hcat [
+      -> line $ hcat [
            text "\tst",
            case fmt of
              II32 -> char 'w'
@@ -487,7 +485,7 @@ pprInstr platform instr = case instr of
            ]
 
    LIS reg imm
-      -> hcat [
+      -> line $ hcat [
            char '\t',
            text "lis",
            char '\t',
@@ -497,7 +495,7 @@ pprInstr platform instr = case instr of
            ]
 
    LI reg imm
-      -> hcat [
+      -> line $ hcat [
            char '\t',
            text "li",
            char '\t',
@@ -508,11 +506,11 @@ pprInstr platform instr = case instr of
 
    MR reg1 reg2
     | reg1 == reg2 -> empty
-    | otherwise    -> hcat [
+    | otherwise    -> line $ hcat [
         char '\t',
         case targetClassOfReg platform reg1 of
             RcInteger -> text "mr"
-            _ -> text "fmr",
+            RcFloatOrVector -> text "fmr",
         char '\t',
         pprReg reg1,
         text ", ",
@@ -520,7 +518,7 @@ pprInstr platform instr = case instr of
         ]
 
    CMP fmt reg ri
-      -> hcat [
+      -> line $ hcat [
            char '\t',
            op,
            char '\t',
@@ -538,7 +536,7 @@ pprInstr platform instr = case instr of
                ]
 
    CMPL fmt reg ri
-      -> hcat [
+      -> line $ hcat [
            char '\t',
            op,
            char '\t',
@@ -556,13 +554,13 @@ pprInstr platform instr = case instr of
                   ]
 
    BCC cond blockid prediction
-      -> hcat [
+      -> line $ hcat [
            char '\t',
            text "b",
            pprCond cond,
            pprPrediction prediction,
            char '\t',
-           pdoc platform lbl
+           pprAsmLabel platform lbl
            ]
          where lbl = mkLocalBlockLabel (getUnique blockid)
                pprPrediction p = case p of
@@ -571,7 +569,7 @@ pprInstr platform instr = case instr of
                  Just False -> char '-'
 
    BCCFAR cond blockid prediction
-      -> vcat [
+      -> lines_ [
            hcat [
                text "\tb",
                pprCond (condNegate cond),
@@ -580,7 +578,7 @@ pprInstr platform instr = case instr of
            ],
            hcat [
                text "\tb\t",
-               pdoc platform lbl
+               pprAsmLabel platform lbl
            ]
           ]
           where lbl = mkLocalBlockLabel (getUnique blockid)
@@ -590,18 +588,20 @@ pprInstr platform instr = case instr of
                   Just False -> char '+'
 
    JMP lbl _
-     -- We never jump to ForeignLabels; if we ever do, c.f. handling for "BL"
-     | isForeignLabel lbl -> panic "PPC.Ppr.pprInstr: JMP to ForeignLabel"
+     | OSAIX <- platformOS platform ->
+       line $ hcat [ -- an alias for b that takes a CLabel
+           text "\tb.\t", -- add the ".", cf Note [AIX function descriptors and entry-code addresses]
+           pprAsmLabel platform lbl
+       ]
+
      | otherwise ->
-       hcat [ -- an alias for b that takes a CLabel
-           char '\t',
-           text "b",
-           char '\t',
-           pdoc platform lbl
+       line $ hcat [ -- an alias for b that takes a CLabel
+           text "\tb\t",
+           pprAsmLabel platform lbl
        ]
 
    MTCTR reg
-      -> hcat [
+      -> line $ hcat [
            char '\t',
            text "mtctr",
            char '\t',
@@ -609,7 +609,7 @@ pprInstr platform instr = case instr of
         ]
 
    BCTR _ _ _
-      -> hcat [
+      -> line $ hcat [
            char '\t',
            text "bctr"
          ]
@@ -617,6 +617,8 @@ pprInstr platform instr = case instr of
    BL lbl _
       -> case platformOS platform of
            OSAIX ->
+             -- Note [AIX function descriptors and entry-code addresses]
+             -- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
              -- On AIX, "printf" denotes a function-descriptor (for use
              -- by function pointers), whereas the actual entry-code
              -- address is denoted by the dot-prefixed ".printf" label.
@@ -626,27 +628,27 @@ pprInstr platform instr = case instr of
              -- but when profiling the codegen inserts calls via
              -- 'emitRtsCallGen' which are 'CmmLabel's even though
              -- they'd technically be more like 'ForeignLabel's.
-             hcat [
+             line $ hcat [
                text "\tbl\t.",
-               pdoc platform lbl
+               pprAsmLabel platform lbl
              ]
            _ ->
-             hcat [
+             line $ hcat [
                text "\tbl\t",
-               pdoc platform lbl
+               pprAsmLabel platform lbl
              ]
 
    BCTRL _
-      -> hcat [
+      -> line $ hcat [
              char '\t',
              text "bctrl"
          ]
 
    ADD reg1 reg2 ri
-      -> pprLogic platform (sLit "add") reg1 reg2 ri
+      -> pprLogic platform (text "add") reg1 reg2 ri
 
    ADDIS reg1 reg2 imm
-      -> hcat [
+      -> line $ hcat [
            char '\t',
            text "addis",
            char '\t',
@@ -658,25 +660,25 @@ pprInstr platform instr = case instr of
            ]
 
    ADDO reg1 reg2 reg3
-      -> pprLogic platform (sLit "addo") reg1 reg2 (RIReg reg3)
+      -> pprLogic platform (text "addo") reg1 reg2 (RIReg reg3)
 
    ADDC reg1 reg2 reg3
-      -> pprLogic platform (sLit "addc") reg1 reg2 (RIReg reg3)
+      -> pprLogic platform (text "addc") reg1 reg2 (RIReg reg3)
 
    ADDE reg1 reg2 reg3
-      -> pprLogic platform (sLit "adde") reg1 reg2 (RIReg reg3)
+      -> pprLogic platform (text "adde") reg1 reg2 (RIReg reg3)
 
    ADDZE reg1 reg2
-      -> pprUnary (sLit "addze") reg1 reg2
+      -> pprUnary (text "addze") reg1 reg2
 
    SUBF reg1 reg2 reg3
-      -> pprLogic platform (sLit "subf") reg1 reg2 (RIReg reg3)
+      -> pprLogic platform (text "subf") reg1 reg2 (RIReg reg3)
 
    SUBFO reg1 reg2 reg3
-      -> pprLogic platform (sLit "subfo") reg1 reg2 (RIReg reg3)
+      -> pprLogic platform (text "subfo") reg1 reg2 (RIReg reg3)
 
    SUBFC reg1 reg2 ri
-      -> hcat [
+      -> line $ hcat [
            char '\t',
            text "subf",
            case ri of
@@ -691,13 +693,13 @@ pprInstr platform instr = case instr of
            ]
 
    SUBFE reg1 reg2 reg3
-      -> pprLogic platform (sLit "subfe") reg1 reg2 (RIReg reg3)
+      -> pprLogic platform (text "subfe") reg1 reg2 (RIReg reg3)
 
    MULL fmt reg1 reg2 ri
       -> pprMul platform fmt reg1 reg2 ri
 
    MULLO fmt reg1 reg2 reg3
-      -> hcat [
+      -> line $ hcat [
              char '\t',
              text "mull",
              case fmt of
@@ -714,13 +716,13 @@ pprInstr platform instr = case instr of
 
    MFOV fmt reg
       -> vcat [
-           hcat [
+           line $ hcat [
                char '\t',
                text "mfxer",
                char '\t',
                pprReg reg
                ],
-           hcat [
+           line $ hcat [
                char '\t',
                text "extr",
                case fmt of
@@ -740,7 +742,7 @@ pprInstr platform instr = case instr of
            ]
 
    MULHU fmt reg1 reg2 reg3
-      -> hcat [
+      -> line $ hcat [
             char '\t',
             text "mulh",
             case fmt of
@@ -761,7 +763,7 @@ pprInstr platform instr = case instr of
         -- for some reason, "andi" doesn't exist.
         -- we'll use "andi." instead.
    AND reg1 reg2 (RIImm imm)
-      -> hcat [
+      -> line $ hcat [
             char '\t',
             text "andi.",
             char '\t',
@@ -773,22 +775,22 @@ pprInstr platform instr = case instr of
         ]
 
    AND reg1 reg2 ri
-      -> pprLogic platform (sLit "and") reg1 reg2 ri
+      -> pprLogic platform (text "and") reg1 reg2 ri
 
    ANDC reg1 reg2 reg3
-      -> pprLogic platform (sLit "andc") reg1 reg2 (RIReg reg3)
+      -> pprLogic platform (text "andc") reg1 reg2 (RIReg reg3)
 
    NAND reg1 reg2 reg3
-      -> pprLogic platform (sLit "nand") reg1 reg2 (RIReg reg3)
+      -> pprLogic platform (text "nand") reg1 reg2 (RIReg reg3)
 
    OR reg1 reg2 ri
-      -> pprLogic platform (sLit "or") reg1 reg2 ri
+      -> pprLogic platform (text "or") reg1 reg2 ri
 
    XOR reg1 reg2 ri
-      -> pprLogic platform (sLit "xor") reg1 reg2 ri
+      -> pprLogic platform (text "xor") reg1 reg2 ri
 
    ORIS reg1 reg2 imm
-      -> hcat [
+      -> line $ hcat [
             char '\t',
             text "oris",
             char '\t',
@@ -800,7 +802,7 @@ pprInstr platform instr = case instr of
         ]
 
    XORIS reg1 reg2 imm
-      -> hcat [
+      -> line $ hcat [
             char '\t',
             text "xoris",
             char '\t',
@@ -812,7 +814,7 @@ pprInstr platform instr = case instr of
         ]
 
    EXTS fmt reg1 reg2
-      -> hcat [
+      -> line $ hcat [
            char '\t',
            text "exts",
            pprFormat fmt,
@@ -823,7 +825,7 @@ pprInstr platform instr = case instr of
          ]
 
    CNTLZ fmt reg1 reg2
-      -> hcat [
+      -> line $ hcat [
            char '\t',
            text "cntlz",
            case fmt of
@@ -837,10 +839,10 @@ pprInstr platform instr = case instr of
          ]
 
    NEG reg1 reg2
-      -> pprUnary (sLit "neg") reg1 reg2
+      -> pprUnary (text "neg") reg1 reg2
 
    NOT reg1 reg2
-      -> pprUnary (sLit "not") reg1 reg2
+      -> pprUnary (text "not") reg1 reg2
 
    SR II32 reg1 reg2 (RIImm (ImmInt i))
     -- Handle the case where we are asked to shift a 32 bit register by
@@ -864,27 +866,27 @@ pprInstr platform instr = case instr of
 
    SL fmt reg1 reg2 ri
       -> let op = case fmt of
-                       II32 -> "slw"
-                       II64 -> "sld"
+                       II32 -> text "slw"
+                       II64 -> text "sld"
                        _    -> panic "PPC.Ppr.pprInstr: shift illegal size"
-         in pprLogic platform (sLit op) reg1 reg2 (limitShiftRI fmt ri)
+         in pprLogic platform op reg1 reg2 (limitShiftRI fmt ri)
 
    SR fmt reg1 reg2 ri
       -> let op = case fmt of
-                       II32 -> "srw"
-                       II64 -> "srd"
+                       II32 -> text "srw"
+                       II64 -> text "srd"
                        _    -> panic "PPC.Ppr.pprInstr: shift illegal size"
-         in pprLogic platform (sLit op) reg1 reg2 (limitShiftRI fmt ri)
+         in pprLogic platform op reg1 reg2 (limitShiftRI fmt ri)
 
    SRA fmt reg1 reg2 ri
       -> let op = case fmt of
-                       II32 -> "sraw"
-                       II64 -> "srad"
+                       II32 -> text "sraw"
+                       II64 -> text "srad"
                        _    -> panic "PPC.Ppr.pprInstr: shift illegal size"
-         in pprLogic platform (sLit op) reg1 reg2 (limitShiftRI fmt ri)
+         in pprLogic platform op reg1 reg2 (limitShiftRI fmt ri)
 
    RLWINM reg1 reg2 sh mb me
-      -> hcat [
+      -> line $ hcat [
              text "\trlwinm\t",
              pprReg reg1,
              text ", ",
@@ -898,7 +900,7 @@ pprInstr platform instr = case instr of
          ]
 
    CLRLI fmt reg1 reg2 n
-      -> hcat [
+      -> line $ hcat [
             text "\tclrl",
             pprFormat fmt,
             text "i ",
@@ -910,7 +912,7 @@ pprInstr platform instr = case instr of
         ]
 
    CLRRI fmt reg1 reg2 n
-      -> hcat [
+      -> line $ hcat [
             text "\tclrr",
             pprFormat fmt,
             text "i ",
@@ -922,25 +924,28 @@ pprInstr platform instr = case instr of
         ]
 
    FADD fmt reg1 reg2 reg3
-      -> pprBinaryF (sLit "fadd") fmt reg1 reg2 reg3
+      -> pprBinaryF (text "fadd") fmt reg1 reg2 reg3
 
    FSUB fmt reg1 reg2 reg3
-      -> pprBinaryF (sLit "fsub") fmt reg1 reg2 reg3
+      -> pprBinaryF (text "fsub") fmt reg1 reg2 reg3
 
    FMUL fmt reg1 reg2 reg3
-      -> pprBinaryF (sLit "fmul") fmt reg1 reg2 reg3
+      -> pprBinaryF (text "fmul") fmt reg1 reg2 reg3
 
    FDIV fmt reg1 reg2 reg3
-      -> pprBinaryF (sLit "fdiv") fmt reg1 reg2 reg3
+      -> pprBinaryF (text "fdiv") fmt reg1 reg2 reg3
 
    FABS reg1 reg2
-      -> pprUnary (sLit "fabs") reg1 reg2
+      -> pprUnary (text "fabs") reg1 reg2
 
    FNEG reg1 reg2
-      -> pprUnary (sLit "fneg") reg1 reg2
+      -> pprUnary (text "fneg") reg1 reg2
+
+   FMADD signs fmt dst ra rc rb
+     -> pprTernaryF (pprFMASign signs) fmt dst ra rc rb
 
    FCMP reg1 reg2
-      -> hcat [
+      -> line $ hcat [
            char '\t',
            text "fcmpu\t0, ",
                -- Note: we're using fcmpu, not fcmpo
@@ -956,19 +961,19 @@ pprInstr platform instr = case instr of
          ]
 
    FCTIWZ reg1 reg2
-      -> pprUnary (sLit "fctiwz") reg1 reg2
+      -> pprUnary (text "fctiwz") reg1 reg2
 
    FCTIDZ reg1 reg2
-      -> pprUnary (sLit "fctidz") reg1 reg2
+      -> pprUnary (text "fctidz") reg1 reg2
 
    FCFID reg1 reg2
-      -> pprUnary (sLit "fcfid") reg1 reg2
+      -> pprUnary (text "fcfid") reg1 reg2
 
    FRSP reg1 reg2
-      -> pprUnary (sLit "frsp") reg1 reg2
+      -> pprUnary (text "frsp") reg1 reg2
 
    CRNOR dst src1 src2
-      -> hcat [
+      -> line $ hcat [
            text "\tcrnor\t",
            int dst,
            text ", ",
@@ -978,7 +983,7 @@ pprInstr platform instr = case instr of
          ]
 
    MFCR reg
-      -> hcat [
+      -> line $ hcat [
              char '\t',
              text "mfcr",
              char '\t',
@@ -986,7 +991,7 @@ pprInstr platform instr = case instr of
          ]
 
    MFLR reg
-      -> hcat [
+      -> line $ hcat [
            char '\t',
            text "mflr",
            char '\t',
@@ -994,27 +999,27 @@ pprInstr platform instr = case instr of
          ]
 
    FETCHPC reg
-      -> vcat [
+      -> lines_ [
              text "\tbcl\t20,31,1f",
              hcat [ text "1:\tmflr\t", pprReg reg ]
          ]
 
    HWSYNC
-      -> text "\tsync"
+      -> line $ text "\tsync"
 
    ISYNC
-      -> text "\tisync"
+      -> line $ text "\tisync"
 
    LWSYNC
-      -> text "\tlwsync"
+      -> line $ text "\tlwsync"
 
    NOP
-      -> text "\tnop"
+      -> line $ text "\tnop"
 
-pprLogic :: Platform -> PtrString -> Reg -> Reg -> RI -> SDoc
-pprLogic platform op reg1 reg2 ri = hcat [
+pprLogic :: IsDoc doc => Platform -> Line doc -> Reg -> Reg -> RI -> doc
+pprLogic platform op reg1 reg2 ri = line $ hcat [
         char '\t',
-        ptext op,
+        op,
         case ri of
             RIReg _ -> empty
             RIImm _ -> char 'i',
@@ -1027,8 +1032,8 @@ pprLogic platform op reg1 reg2 ri = hcat [
     ]
 
 
-pprMul :: Platform -> Format -> Reg -> Reg -> RI -> SDoc
-pprMul platform fmt reg1 reg2 ri = hcat [
+pprMul :: IsDoc doc => Platform -> Format -> Reg -> Reg -> RI -> doc
+pprMul platform fmt reg1 reg2 ri = line $ hcat [
         char '\t',
         text "mull",
         case ri of
@@ -1046,8 +1051,8 @@ pprMul platform fmt reg1 reg2 ri = hcat [
     ]
 
 
-pprDiv :: Format -> Bool -> Reg -> Reg -> Reg -> SDoc
-pprDiv fmt sgn reg1 reg2 reg3 = hcat [
+pprDiv :: IsDoc doc => Format -> Bool -> Reg -> Reg -> Reg -> doc
+pprDiv fmt sgn reg1 reg2 reg3 = line $ hcat [
         char '\t',
         text "div",
         case fmt of
@@ -1064,10 +1069,10 @@ pprDiv fmt sgn reg1 reg2 reg3 = hcat [
     ]
 
 
-pprUnary :: PtrString -> Reg -> Reg -> SDoc
-pprUnary op reg1 reg2 = hcat [
+pprUnary :: IsDoc doc => Line doc -> Reg -> Reg -> doc
+pprUnary op reg1 reg2 = line $ hcat [
         char '\t',
-        ptext op,
+        op,
         char '\t',
         pprReg reg1,
         text ", ",
@@ -1075,10 +1080,10 @@ pprUnary op reg1 reg2 = hcat [
     ]
 
 
-pprBinaryF :: PtrString -> Format -> Reg -> Reg -> Reg -> SDoc
-pprBinaryF op fmt reg1 reg2 reg3 = hcat [
+pprBinaryF :: IsDoc doc => Line doc -> Format -> Reg -> Reg -> Reg -> doc
+pprBinaryF op fmt reg1 reg2 reg3 = line $ hcat [
         char '\t',
-        ptext op,
+        op,
         pprFFormat fmt,
         char '\t',
         pprReg reg1,
@@ -1088,12 +1093,27 @@ pprBinaryF op fmt reg1 reg2 reg3 = hcat [
         pprReg reg3
     ]
 
-pprRI :: Platform -> RI -> SDoc
+pprTernaryF :: IsDoc doc => Line doc -> Format -> Reg -> Reg -> Reg -> Reg -> doc
+pprTernaryF op fmt rt ra rc rb = line $ hcat [
+        char '\t',
+        op,
+        pprFFormat fmt,
+        char '\t',
+        pprReg rt,
+        text ", ",
+        pprReg ra,
+        text ", ",
+        pprReg rc,
+        text ", ",
+        pprReg rb
+    ]
+
+pprRI :: IsLine doc => Platform -> RI -> doc
 pprRI _        (RIReg r) = pprReg r
 pprRI platform (RIImm r) = pprImm platform r
 
 
-pprFFormat :: Format -> SDoc
+pprFFormat :: IsLine doc => Format -> doc
 pprFFormat FF64     = empty
 pprFFormat FF32     = char 's'
 pprFFormat _        = panic "PPC.Ppr.pprFFormat: no match"
