@@ -461,7 +461,7 @@ pprFormat x = case x of
   -- TODO: this is shady because it only works for certain instructions
   VecFormat _ FmtInt8   -> text "b"
   VecFormat _ FmtInt16  -> text "w"
-  VecFormat _ FmtInt32  -> text "l"
+  VecFormat _ FmtInt32  -> text "d"
   VecFormat _ FmtInt64  -> text "q"
 
 pprFormat_x87 :: IsLine doc => Format -> doc
@@ -915,7 +915,7 @@ pprInstr platform i = case i of
       -> pprFormatOp (text "mul") format op
 
    FDIV format op1 op2
-      -> pprFormatOpOp (text "div") format op1 op2
+      -> pprFormatOpReg (text "div") format op1 op2
 
    FMA3 format var perm op1 op2 op3
       -> let mnemo = case var of
@@ -990,8 +990,22 @@ pprInstr platform i = case i of
      -> pprFormatOpRegReg (text "vmul") format s1 s2 dst
    VDIV format s1 s2 dst
      -> pprFormatOpRegReg (text "vdiv") format s1 s2 dst
-   VBROADCAST format from to
-     -> pprBroadcast (text "vbroadcast") format from to
+   PADD format src dst
+     -> pprFormatOpReg (text "padd") format src dst
+   PSUB format src dst
+     -> pprFormatOpReg (text "psub") format src dst
+   PMULL format src dst
+     -> pprFormatOpReg (text "pmull") format src dst
+   PMULUDQ format src dst
+     -> pprOpReg (text "pmuludq") format src dst
+   PCMPGT format src dst
+     -> pprFormatOpReg (text "pcmpgt") format src dst
+   VBROADCAST format@(VecFormat _ sFmt) from to
+     -> pprBroadcast (text "vbroadcast") (scalarFormatFormat sFmt) format from to
+   VBROADCAST format _ _
+     -> pprPanic "VBROADCAST: expected vector format" (ppr format)
+   VPBROADCAST scalarFormat format from to
+     -> pprBroadcast (text "vpbroadcast") scalarFormat format from to
    VMOVU format from to
      -> pprFormatOpOp (text "vmovu") format from to
    MOVU format from to
@@ -1012,39 +1026,103 @@ pprInstr platform i = case i of
         VecFormat 32 FmtInt16 -> text "vmovdqu32" -- NB: not using vmovdqu16/8, as they
         VecFormat 64 FmtInt8  -> text "vmovdqu32" -- require the additional AVX512BW extension
         _ -> text "vmovdqu"
+   VMOV_MERGE format src2 src1 dst
+     -> pprRegRegReg instr format src2 src1 dst
+     where instr = case format of
+             VecFormat _ FmtFloat -> text "vmovss"
+             VecFormat _ FmtDouble -> text "vmovsd"
+             _ -> pprPanic "invalid format for VMOV_MERGE" (ppr format)
 
    PXOR format src dst
      -> pprPXor (text "pxor") format src dst
    VPXOR format s1 s2 dst
      -> pprXor (text "vpxor") format s1 s2 dst
+   PAND format src dst
+     -> pprOpReg (text "pand") format src dst
+   PANDN format src dst
+     -> pprOpReg (text "pandn") format src dst
+   POR format src dst
+     -> pprOpReg (text "por") format src dst
    VEXTRACT format offset from to
      -> pprFormatImmRegOp (text "vextract") format offset from to
    INSERTPS format offset addr dst
      -> pprInsert (text "insertps") format offset addr dst
+   VINSERTPS format offset src2 src1 dst
+     -> pprImmOpRegReg (text "vinsertps") format offset src2 src1 dst
+   PINSR scalarFormat vectorFormat offset src dst
+     -> pprPinsr (text "pinsr") scalarFormat vectorFormat offset src dst
+   PEXTR scalarFormat vectorFormat offset src dst
+     -> pprPextr (text "pextr") scalarFormat vectorFormat offset src dst
 
    SHUF format offset src dst
      -> pprShuf (text "shuf" <> pprFormat format) format offset src dst
    VSHUF format offset src1 src2 dst
      -> pprVShuf (text "vshuf" <> pprFormat format) format offset src1 src2 dst
+   PSHUFB format mask dst
+     -> pprOpReg (text "pshufb") format mask dst
+   PSHUFLW format offset src dst
+     -> pprShuf (text "pshuflw") format offset src dst
+   PSHUFHW format offset src dst
+     -> pprShuf (text "pshufhw") format offset src dst
    PSHUFD format offset src dst
      -> pprShuf (text "pshufd") format offset src dst
    VPSHUFD format offset src dst
      -> pprShuf (text "vpshufd") format offset src dst
+   BLEND format mask src dst
+     -> pprFormatImmOpReg (text "blend") format mask src dst
+   VBLEND format mask src2 src1 dst
+     -> pprFormatImmOpRegReg (text "vblend") format mask src2 src1 dst
+   PBLENDW format mask src dst
+     -> pprShuf (text "pblendw") format mask src dst
 
+   PSLL format offset dst
+     -> pprFormatOpReg (text "psll") format offset dst
    PSLLDQ format offset dst
      -> pprDoubleShift (text "pslldq") format offset dst
+   PSRL format offset dst
+     -> pprFormatOpReg (text "psrl") format offset dst
    PSRLDQ format offset dst
      -> pprDoubleShift (text "psrldq") format offset dst
+   PALIGNR format offset src dst
+     -> pprImmOpReg (text "palignr") format offset src dst
 
    MOVHLPS format from to
      -> pprOpReg (text "movhlps") format (OpReg from) to
+   VMOVHLPS format src2 src1 dst
+     -> pprRegRegReg (text "vmovhlps") format src2 src1 dst
+   MOVLHPS format from to
+     -> pprOpReg (text "movlhps") format (OpReg from) to
+   VMOVLHPS format src2 src1 dst
+     -> pprRegRegReg (text "vmovlhps") format src2 src1 dst
    UNPCKL format src dst
      -> pprFormatOpReg (text "unpckl") format src dst
+   VUNPCKL format src2 src1 dst
+     -> pprFormatOpRegReg (text "vunpckl") format src2 src1 dst
+   UNPCKH format src dst
+     -> pprFormatOpReg (text "unpckh") format src dst
+   VUNPCKH format src2 src1 dst
+     -> pprFormatOpRegReg (text "vunpckh") format src2 src1 dst
    PUNPCKLQDQ format from to
      -> pprOpReg (text "punpcklqdq") format from to
+   PUNPCKLDQ format from to
+     -> pprOpReg (text "punpckldq") format from to
+   PUNPCKLWD format from to
+     -> pprOpReg (text "punpcklwd") format from to
+   PUNPCKLBW format from to
+     -> pprOpReg (text "punpcklbw") format from to
+   PUNPCKHQDQ format from to
+     -> pprOpReg (text "punpckhqdq") format from to
+   PUNPCKHDQ format from to
+     -> pprOpReg (text "punpckhdq") format from to
+   PUNPCKHWD format from to
+     -> pprOpReg (text "punpckhwd") format from to
+   PUNPCKHBW format from to
+     -> pprOpReg (text "punpckhbw") format from to
+   PACKUSWB format from to
+     -> pprOpReg (text "packuswb") format from to
 
    MINMAX minMax ty fmt src dst
-     -> pprMinMax False minMax ty fmt [src, dst]
+     -> pprMinMax False minMax ty fmt [src, OpReg dst]
    VMINMAX minMax ty fmt src1 src2 dst
      -> pprMinMax True minMax ty fmt [src1, OpReg src2, OpReg dst]
 
@@ -1201,6 +1279,17 @@ pprInstr platform i = case i of
            pprReg platform (archWordFormat (target32Bit platform)) reg2
        ]
 
+   pprRegRegReg :: Line doc -> Format -> Reg -> Reg -> Reg -> doc
+   pprRegRegReg name format reg1 reg2 reg3
+     = line $ hcat [
+           pprMnemonic_ name,
+           pprReg platform format reg1,
+           comma,
+           pprReg platform format reg2,
+           comma,
+           pprReg platform format reg3
+       ]
+
    pprOpReg :: Line doc -> Format -> Operand -> Reg -> doc
    pprOpReg name format op reg
      = line $ hcat [
@@ -1299,16 +1388,14 @@ pprInstr platform i = case i of
    -- Custom pretty printers
    -- These instructions currently don't follow a uniform suffix pattern
    -- in their names, so we have custom pretty printers for them.
-   pprBroadcast :: Line doc -> Format -> Operand -> Reg -> doc
-   pprBroadcast name fmt@(VecFormat _ sFmt) op dst
+   pprBroadcast :: Line doc -> Format -> Format -> Operand -> Reg -> doc
+   pprBroadcast name scalarFormat vectorFormat op dst
      = line $ hcat [
-           pprBroadcastMnemonic name fmt,
-           pprOperand platform (scalarFormatFormat sFmt) op,
+           pprBroadcastMnemonic name vectorFormat,
+           pprOperand platform scalarFormat op,
            comma,
-           pprReg platform fmt dst
+           pprReg platform vectorFormat dst
        ]
-   pprBroadcast _ fmt _ _ =
-     pprPanic "pprBroadcast: expected vector format" (ppr fmt)
 
    pprXor :: Line doc -> Format -> Reg -> Reg -> Reg -> doc
    pprXor name format reg1 reg2 reg3
@@ -1360,6 +1447,28 @@ pprInstr platform i = case i of
            pprReg platform format dst
        ]
 
+   pprPinsr :: Line doc -> Format -> Format -> Imm -> Operand -> Reg -> doc
+   pprPinsr name scalarFormat vectorFormat imm src dst
+     = line $ hcat [
+           pprMnemonic name vectorFormat,
+           pprDollImm imm,
+           comma,
+           pprOperand platform scalarFormat src,
+           comma,
+           pprReg platform vectorFormat dst
+       ]
+
+   pprPextr :: Line doc -> Format -> Format -> Imm -> Reg -> Operand -> doc
+   pprPextr name scalarFormat vectorFormat imm src dst
+     = line $ hcat [
+           pprMnemonic name vectorFormat,
+           pprDollImm imm,
+           comma,
+           pprReg platform vectorFormat src,
+           comma,
+           pprOperand platform scalarFormat dst
+       ]
+
    pprShuf :: Line doc -> Format -> Imm -> Operand -> Reg -> doc
    pprShuf name format imm1 op2 reg3
      = line $ hcat [
@@ -1384,15 +1493,63 @@ pprInstr platform i = case i of
            pprReg platform format reg4
        ]
 
-   pprDoubleShift :: Line doc -> Format -> Operand -> Reg -> doc
+   pprDoubleShift :: Line doc -> Format -> Imm -> Reg -> doc
    pprDoubleShift name format off reg
      = line $ hcat [
            pprGenMnemonic name format,
-           pprOperand platform format off,
+           pprDollImm off,
            comma,
            pprReg platform format reg
        ]
 
+   pprImmOpReg :: Line doc -> Format -> Imm -> Operand -> Reg -> doc
+   pprImmOpReg name format imm1 op2 reg3
+     = line $ hcat [
+           pprGenMnemonic name format,
+           pprDollImm imm1,
+           comma,
+           pprOperand platform format op2,
+           comma,
+           pprReg platform format reg3
+       ]
+
+   pprFormatImmOpReg :: Line doc -> Format -> Imm -> Operand -> Reg -> doc
+   pprFormatImmOpReg name format imm1 op2 reg3
+     = line $ hcat [
+           pprMnemonic name format,
+           pprDollImm imm1,
+           comma,
+           pprOperand platform format op2,
+           comma,
+           pprReg platform format reg3
+       ]
+
+   pprImmOpRegReg :: Line doc -> Format -> Imm -> Operand -> Reg -> Reg -> doc
+   pprImmOpRegReg name format imm1 op2 reg3 reg4
+     = line $ hcat [
+           pprGenMnemonic name format,
+           pprDollImm imm1,
+           comma,
+           pprOperand platform format op2,
+           comma,
+           pprReg platform format reg3,
+           comma,
+           pprReg platform format reg4
+       ]
+
+   pprFormatImmOpRegReg :: Line doc -> Format -> Imm -> Operand -> Reg -> Reg -> doc
+   pprFormatImmOpRegReg name format imm1 op2 reg3 reg4
+     = line $ hcat [
+           pprMnemonic name format,
+           pprDollImm imm1,
+           comma,
+           pprOperand platform format op2,
+           comma,
+           pprReg platform format reg3,
+           comma,
+           pprReg platform format reg4
+       ]
+
    pprMinMax :: Bool -> MinOrMax -> MinMaxType -> Format -> [Operand] -> doc
    pprMinMax wantV minOrMax mmTy fmt regs
      = line $ hcat ( instr : intersperse comma ( map ( pprOperand platform fmt ) regs ) )
