@@ -2,7 +2,6 @@
 -- Copyright (c) 2014 Joachim Breitner
 --
 
-{-# LANGUAGE BangPatterns #-}
 
 module GHC.Core.Opt.CallArity
     ( callArityAnalProgram
@@ -13,7 +12,6 @@ import GHC.Prelude
 
 import GHC.Types.Var.Set
 import GHC.Types.Var.Env
-import GHC.Driver.Session ( DynFlags )
 
 import GHC.Types.Basic
 import GHC.Core
@@ -25,6 +23,7 @@ import GHC.Types.Demand
 import GHC.Utils.Misc
 
 import Control.Arrow ( first, second )
+import Data.List.NonEmpty ( NonEmpty (..) )
 
 
 {-
@@ -100,7 +99,7 @@ The two analysis are not completely independent, as a higher arity can improve
 the information about what variables are being called once or multiple times.
 
 Note [Analysis I: The arity analysis]
-------------------------------------
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
 The arity analysis is quite straightforward: The information about an
 expression is an
@@ -116,7 +115,7 @@ minimum (considering Nothing an infinity).
 
 
 Note [Analysis II: The Co-Called analysis]
-------------------------------------------
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
 The second part is more sophisticated. For reasons explained below, it is not
 sufficient to simply know how often an expression evaluates a variable. Instead
@@ -151,7 +150,7 @@ The interesting cases of the analysis:
    any useful co-call information.
    Return (fv e)²
  * Case alternatives alt₁,alt₂,...:
-   Only one can be execuded, so
+   Only one can be executed, so
    Return (alt₁ ∪ alt₂ ∪...)
  * App e₁ e₂ (and analogously Case scrut alts), with non-trivial e₂:
    We get the results from both sides, with the argument evaluated at most once.
@@ -171,7 +170,7 @@ The interesting cases of the analysis:
    Tricky.
    We assume that it is really mutually recursive, i.e. that every variable
    calls one of the others, and that this is strongly connected (otherwise we
-   return an over-approximation, so that's ok), see note [Recursion and fixpointing].
+   return an over-approximation, so that's ok), see Note [Recursion and fixpointing].
 
    Let V = {v₁,...vₙ}.
    Assume that the vs have been analysed with an incoming demand and
@@ -278,7 +277,7 @@ together with what other functions.
 Note [Analysis type signature]
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
-The work-hourse of the analysis is the function `callArityAnal`, with the
+The workhorse of the analysis is the function `callArityAnal`, with the
 following type:
 
     type CallArityRes = (UnVarGraph, VarEnv Arity)
@@ -378,15 +377,14 @@ a body representing “all external calls”, which returns a pessimistic
 CallArityRes (the co-call graph is the complete graph, all arityies 0).
 
 Note [Trimming arity]
-~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-
+~~~~~~~~~~~~~~~~~~~~~
 In the Call Arity papers, we are working on an untyped lambda calculus with no
 other id annotations, where eta-expansion is always possible. But this is not
 the case for Core!
  1. We need to ensure the invariant
       callArity e <= typeArity (exprType e)
     for the same reasons that exprArity needs this invariant (see Note
-    [exprArity invariant] in GHC.Core.Opt.Arity).
+    [typeArity invariants] in GHC.Core.Opt.Arity).
 
     If we are not doing that, a too-high arity annotation will be stored with
     the id, confusing the simplifier later on.
@@ -434,12 +432,12 @@ choice, and hence Call Arity sets the call arity for join points as well.
 
 -- Main entry point
 
-callArityAnalProgram :: DynFlags -> CoreProgram -> CoreProgram
-callArityAnalProgram _dflags binds = binds'
+callArityAnalProgram :: CoreProgram -> CoreProgram
+callArityAnalProgram binds = binds'
   where
     (_, binds') = callArityTopLvl [] emptyVarSet binds
 
--- See Note [Analysing top-level-binds]
+-- See Note [Analysing top-level binds]
 callArityTopLvl :: [Var] -> VarSet -> [CoreBind] -> (CallArityRes, [CoreBind])
 callArityTopLvl exported _ []
     = ( calledMultipleTimes $ (emptyUnVarGraph, mkVarEnv $ [(v, 0) | v <- exported])
@@ -526,7 +524,7 @@ callArityAnal arity int (Case scrut bndr ty alts)
       (final_ae, Case scrut' bndr ty alts')
   where
     (alt_aes, alts') = unzip $ map go alts
-    go (Alt dc bndrs e) = let (ae, e') = callArityAnal arity int e
+    go (Alt dc bndrs e) = let (ae, e') = callArityAnal arity (int `delVarSetList` (bndr:bndrs)) e
                           in  (ae, Alt dc bndrs e')
     alt_ae = lubRess alt_aes
     (scrut_ae, scrut') = callArityAnal 0 int scrut
@@ -545,7 +543,7 @@ callArityAnal arity int (Let bind e)
 -- Which bindings should we look at?
 -- See Note [Which variables are interesting]
 isInteresting :: Var -> Bool
-isInteresting v = not $ null (typeArity (idType v))
+isInteresting v = typeArity (idType v) > 0
 
 interestingBinds :: CoreBind -> [Var]
 interestingBinds = filter isInteresting . bindersOf
@@ -568,7 +566,7 @@ callArityBind boring_vars ae_body int (NonRec v rhs)
     --          (vcat [ppr v, ppr ae_body, ppr int, ppr ae_rhs, ppr safe_arity])
     (final_ae, NonRec v' rhs')
   where
-    is_thunk = not (exprIsCheap rhs) -- see note [What is a thunk]
+    is_thunk = not (exprIsCheap rhs) -- see Note [What is a thunk]
     -- If v is boring, we will not find it in ae_body, but always assume (0, False)
     boring = v `elemVarSet` boring_vars
 
@@ -638,7 +636,7 @@ callArityBind boring_vars ae_body int b@(Rec binds)
 
             | otherwise
             -- We previously analyzed this with a different arity (or not at all)
-            = let is_thunk = not (exprIsCheap rhs) -- see note [What is a thunk]
+            = let is_thunk = not (exprIsCheap rhs) -- see Note [What is a thunk]
 
                   safe_arity | is_thunk    = 0  -- See Note [Thunks in recursive groups]
                              | otherwise   = new_arity
@@ -699,14 +697,14 @@ callArityRecEnv any_boring ae_rhss ae_body
 
 -- See Note [Trimming arity]
 trimArity :: Id -> Arity -> Arity
-trimArity v a = minimum [a, max_arity_by_type, max_arity_by_strsig]
+trimArity v a = minimum (a :| max_arity_by_type : max_arity_by_strsig : [])
   where
-    max_arity_by_type = length (typeArity (idType v))
+    max_arity_by_type = typeArity (idType v)
     max_arity_by_strsig
         | isDeadEndDiv result_info = length demands
         | otherwise = a
 
-    (demands, result_info) = splitStrictSig (idStrictness v)
+    (demands, result_info) = splitDmdSig (idDmdSig v)
 
 ---------------------------------------
 -- Functions related to CallArityRes --
@@ -730,7 +728,7 @@ resDel :: Var -> CallArityRes -> CallArityRes
 resDel v (!g, !ae) = (g `delNode` v, ae `delVarEnv` v)
 
 domRes :: CallArityRes -> UnVarSet
-domRes (_, ae) = varEnvDom ae
+domRes (_, ae) = varEnvDomain ae
 
 -- In the result, find out the minimum arity and whether the variable is called
 -- at most once.
