@@ -21,8 +21,10 @@ module GHC.Types.Id.Info (
         -- * The IdDetails type
         IdDetails(..), pprIdDetails, coVarDetails, isCoVarDetails,
         JoinArity, isJoinIdDetails_maybe,
+
         RecSelParent(..), recSelParentName, recSelFirstConName,
         recSelParentCons, idDetailsConcreteTvs,
+        RecSelInfo(..), conLikesRecSelInfo,
 
         -- * The IdInfo type
         IdInfo,         -- Abstract
@@ -35,7 +37,8 @@ module GHC.Types.Id.Info (
 
         -- ** Zapping various forms of Info
         zapLamInfo, zapFragileInfo,
-        zapDemandInfo, zapUsageInfo, zapUsageEnvInfo, zapUsedOnceInfo,
+        lazifyDemandInfo, floatifyDemandInfo,
+        zapUsageInfo, zapUsageEnvInfo, zapUsedOnceInfo,
         zapTailCallInfo, zapCallArityInfo, trimUnfolding,
 
         -- ** The ArityInfo type
@@ -110,11 +113,12 @@ import {-# SOURCE #-} GHC.Tc.Utils.TcType ( ConcreteTyVars, noConcreteTyVars )
 
 import GHC.Utils.Outputable
 import GHC.Utils.Panic
-import GHC.Stg.InferTags.TagSig
+import GHC.Stg.EnforceEpt.TagSig
 import GHC.StgToCmm.Types (LambdaFormInfo)
 
 import Data.Data ( Data )
 import Data.Word
+import Data.List as List( partition )
 
 -- infixl so you can say (id `set` a `set` b)
 infixl  1 `setRuleInfo`,
@@ -150,12 +154,9 @@ data IdDetails
     , sel_naughty    :: Bool    -- True <=> a "naughty" selector which can't actually exist, for example @x@ in:
                                 --    data T = forall a. MkT { x :: a }
                                 -- See Note [Naughty record selectors] in GHC.Tc.TyCl
-    , sel_cons       :: ([ConLike], [ConLike])
-                                -- If record selector is not defined for all constructors
-                                -- of a parent type, this is the pair of lists of constructors that
-                                -- it is and is not defined for. Otherwise, it's Nothing.
-                                -- Cached here based on the RecSelParent.
-    }                           -- See Note [Detecting incomplete record selectors] in GHC.HsToCore.Pmc
+    , sel_cons       :: RecSelInfo
+                        -- Partiality info, cached here based on the RecSelParent.
+    }
 
   | DataConWorkId DataCon       -- ^ The 'Id' is for a data constructor /worker/
   | DataConWrapId DataCon       -- ^ The 'Id' is for a data constructor /wrapper/
@@ -179,7 +180,7 @@ data IdDetails
         -- should be instantiated to concrete type variables?
         --
         -- See Note [Representation-polymorphism checking built-ins]
-        -- in GHC.Tc.Gen.Head.
+        -- in GHC.Tc.Utils.Concrete.
 
   -- | The 'Id' is for a primitive operator.
   | PrimOpId
@@ -192,7 +193,7 @@ data IdDetails
         -- type variables.
         --
         -- See Note [Representation-polymorphism checking built-ins]
-        -- in GHC.Tc.Gen.Head.
+        -- in GHC.Tc.Utils.Concrete.
 
   | FCallId ForeignCall         -- ^ The 'Id' is for a foreign call.
                                 -- Type will be simple: no type families, newtypes, etc
@@ -219,10 +220,15 @@ data IdDetails
         -- Worker like functions are create by W/W and SpecConstr and we can expect that they
         -- aren't used unapplied.
         -- See Note [CBV Function Ids]
-        -- See Note [Tag Inference]
+        -- See Note [EPT enforcement]
         -- The [CbvMark] is always empty (and ignored) until after Tidy for ids from the current
         -- module.
 
+data RecSelInfo
+  = RSI { rsi_def   :: [ConLike]   -- Record selector defined for these
+        , rsi_undef :: [ConLike]   -- Record selector not defined for these
+        }
+
 idDetailsConcreteTvs :: IdDetails -> ConcreteTyVars
 idDetailsConcreteTvs = \ case
     PrimOpId _ conc_tvs -> conc_tvs
@@ -231,16 +237,26 @@ idDetailsConcreteTvs = \ case
     DataConWrapId dc    -> dataConConcreteTyVars dc
     _                   -> noConcreteTyVars
 
+-- | The ConLikes that have *all* the given fields
+conLikesRecSelInfo :: [ConLike] -> [FieldLabelString] -> RecSelInfo
+conLikesRecSelInfo con_likes lbls
+  = RSI { rsi_def = defs, rsi_undef = undefs }
+  where
+    !(defs,undefs) = List.partition has_flds con_likes
+
+    has_flds dc = all (has_fld dc) lbls
+    has_fld dc lbl = any (\ fl -> flLabel fl == lbl) (conLikeFieldLabels dc)
+
 
 {- Note [CBV Function Ids]
-~~~~~~~~~~~~~~~~~~~~~~~~~~~
+~~~~~~~~~~~~~~~~~~~~~~~~~~
 A WorkerLikeId essentially allows us to constrain the calling convention
 for the given Id. Each such Id carries with it a list of CbvMarks
 with each element representing a value argument. Arguments who have
 a matching `MarkedCbv` entry in the list need to be passed evaluated+*properly tagged*.
 
 CallByValueFunIds give us additional expressiveness which we use to improve
-runtime. This is all part of the TagInference work. See also Note [Tag Inference].
+runtime. This is all part of the EPT enforcement work. See also Note [EPT enforcement].
 
 They allows us to express the fact that an argument is not only evaluated to WHNF once we
 entered it's RHS but also that an lifted argument is already *properly tagged* once we jump
@@ -260,7 +276,7 @@ The invariants around the arguments of call by value function like Ids are then:
   * Any `WorkerLikeId`
   * Some `JoinId` bindings.
 
-This works analogous to the Strict Field Invariant. See also Note [Strict Field Invariant].
+This works analogous to the EPT Invariant. See also Note [EPT enforcement].
 
 To make this work what we do is:
 * During W/W and SpecConstr any worker/specialized binding we introduce
@@ -855,11 +871,21 @@ zapLamInfo info@(IdInfo {occInfo = occ, demandInfo = demand})
 
     is_safe_dmd dmd = not (isStrUsedDmd dmd)
 
--- | Remove all demand info on the 'IdInfo'
-zapDemandInfo :: IdInfo -> Maybe IdInfo
-zapDemandInfo info = Just (info {demandInfo = topDmd})
+-- | Lazify (remove the top-level demand, only) the demand in `IdInfo`
+-- Keep nested demands; see Note [Floatifying demand info when floating]
+-- in GHC.Core.Opt.SetLevels
+lazifyDemandInfo :: IdInfo -> Maybe IdInfo
+lazifyDemandInfo info@(IdInfo { demandInfo = dmd })
+  = Just (info {demandInfo = lazifyDmd dmd })
 
--- | Remove usage (but not strictness) info on the 'IdInfo'
+-- | Floatify the demand in `IdInfo`
+-- But keep /nested/ demands; see Note [Floatifying demand info when floating]
+-- in GHC.Core.Opt.SetLevels
+floatifyDemandInfo :: IdInfo -> Maybe IdInfo
+floatifyDemandInfo info@(IdInfo { demandInfo = dmd })
+  = Just (info {demandInfo = floatifyDmd dmd })
+
+-- | Remove usage (but not strictness) info on the `IdInfo`
 zapUsageInfo :: IdInfo -> Maybe IdInfo
 zapUsageInfo info = Just (info {demandInfo = zapUsageDemand (demandInfo info)})
 
