@@ -12,7 +12,6 @@ import Settings.Warnings
 import qualified Context as Context
 import Rules.Libffi (libffiName)
 import qualified Data.Set as Set
-import System.Directory
 import Data.Version.Extra
 
 ghcBuilderArgs :: Args
@@ -36,6 +35,9 @@ compileAndLinkHs = (builder (Ghc CompileHs) ||^ builder (Ghc LinkHs)) ? do
     useColor <- shakeColor <$> expr getShakeOptions
     let hasVanilla = elem vanilla ways
         hasDynamic = elem dynamic ways
+    hieFiles <- ghcHieFiles <$> expr flavour
+    stage <- getStage
+    hie_path <- getHieBuildPath
     mconcat [ arg "-Wall"
             , arg "-Wcompat"
             , not useColor ? builder (Ghc CompileHs) ?
@@ -50,6 +52,10 @@ compileAndLinkHs = (builder (Ghc CompileHs) ||^ builder (Ghc LinkHs)) ? do
             , ghcLinkArgs
             , defaultGhcWarningsArgs
             , builder (Ghc CompileHs) ? arg "-c"
+            , hieFiles stage ? builder (Ghc CompileHs) ? mconcat
+                  [ arg "-fwrite-ide-info"
+                  , arg "-hiedir", arg hie_path
+                  ]
             , getInputs
             , arg "-o", arg =<< getOutput ]
 
