@@ -6,13 +6,17 @@ import {-# SOURCE #-} Settings.Default
 
 -- Please update doc/flavours.md when changing this file.
 performanceFlavour :: Flavour
-performanceFlavour = defaultFlavour
+performanceFlavour = splitSections $ enableLateCCS $ defaultFlavour
     { name = "perf"
-    , args = defaultBuilderArgs <> performanceArgs <> defaultPackageArgs }
+    , extraArgs = performanceArgs <> defaultHaddockExtraArgs }
 
 performanceArgs :: Args
 performanceArgs = sourceArgs SourceArgs
     { hsDefault  = pure ["-O", "-H64m"]
-    , hsLibrary  = notStage0 ? arg "-O2"
+    , hsLibrary  = orM [notStage0, cross] ? arg "-O2"
     , hsCompiler = pure ["-O2"]
-    , hsGhc      = mconcat [stage0 ? arg "-O", notStage0 ? arg "-O2"] }
+    , hsGhc      = mconcat
+                    [ andM [stage0, notCross] ? arg "-O"
+                    , orM  [notStage0, cross] ? arg "-O2"
+                    ]
+    }
