@@ -1,5 +1,3 @@
-{-# LANGUAGE DeriveFoldable         #-}
-{-# LANGUAGE DeriveFunctor          #-}
 {-# LANGUAGE DeriveGeneric          #-}
 {-# LANGUAGE DeriveTraversable      #-}
 {-# LANGUAGE FunctionalDependencies #-}
@@ -15,6 +13,7 @@ import Data.Text    (Text)
 import GHC.Generics (Generic)
 
 import qualified Data.Algorithm.Diff as Diff
+import qualified Data.Char           as C
 import qualified Data.Map            as Map
 import qualified Data.Set            as Set
 import qualified Data.Text           as T
@@ -34,12 +33,14 @@ data SPDXLicenseListVersion
     | SPDXLicenseListVersion_3_16
     | SPDXLicenseListVersion_3_23
     | SPDXLicenseListVersion_3_25
+    | SPDXLicenseListVersion_3_26
   deriving (Eq, Ord, Show, Enum, Bounded)
 
 allVers :: Set.Set SPDXLicenseListVersion
 allVers =  Set.fromList [minBound .. maxBound]
 
 prettyVer :: SPDXLicenseListVersion -> Text
+prettyVer SPDXLicenseListVersion_3_26 = "SPDX License List 3.26"
 prettyVer SPDXLicenseListVersion_3_25 = "SPDX License List 3.25"
 prettyVer SPDXLicenseListVersion_3_23 = "SPDX License List 3.23"
 prettyVer SPDXLicenseListVersion_3_16 = "SPDX License List 3.16"
@@ -50,6 +51,7 @@ prettyVer SPDXLicenseListVersion_3_2  = "SPDX License List 3.2"
 prettyVer SPDXLicenseListVersion_3_0  = "SPDX License List 3.0"
 
 suffixVer :: SPDXLicenseListVersion -> String
+suffixVer SPDXLicenseListVersion_3_26 = "_3_26"
 suffixVer SPDXLicenseListVersion_3_25 = "_3_25"
 suffixVer SPDXLicenseListVersion_3_23 = "_3_23"
 suffixVer SPDXLicenseListVersion_3_16 = "_3_16"
@@ -63,7 +65,7 @@ suffixVer SPDXLicenseListVersion_3_0  = "_3_0"
 -- Per version
 -------------------------------------------------------------------------------
 
-data PerV a = PerV a a a a a a a a
+data PerV a = PerV a a a a a a a a a
   deriving (Show, Functor, Foldable, Traversable)
 
 class Functor f => Representable i f | f -> i where
@@ -71,14 +73,15 @@ class Functor f => Representable i f | f -> i where
     tabulate :: (i -> a) -> f a
 
 instance Representable SPDXLicenseListVersion PerV where
-    index SPDXLicenseListVersion_3_0  (PerV x _ _ _ _ _ _ _) = x
-    index SPDXLicenseListVersion_3_2  (PerV _ x _ _ _ _ _ _) = x
-    index SPDXLicenseListVersion_3_6  (PerV _ _ x _ _ _ _ _) = x
-    index SPDXLicenseListVersion_3_9  (PerV _ _ _ x _ _ _ _) = x
-    index SPDXLicenseListVersion_3_10 (PerV _ _ _ _ x _ _ _) = x
-    index SPDXLicenseListVersion_3_16 (PerV _ _ _ _ _ x _ _) = x
-    index SPDXLicenseListVersion_3_23 (PerV _ _ _ _ _ _ x _) = x
-    index SPDXLicenseListVersion_3_25 (PerV _ _ _ _ _ _ _ x) = x
+    index SPDXLicenseListVersion_3_0  (PerV x _ _ _ _ _ _ _ _) = x
+    index SPDXLicenseListVersion_3_2  (PerV _ x _ _ _ _ _ _ _) = x
+    index SPDXLicenseListVersion_3_6  (PerV _ _ x _ _ _ _ _ _) = x
+    index SPDXLicenseListVersion_3_9  (PerV _ _ _ x _ _ _ _ _) = x
+    index SPDXLicenseListVersion_3_10 (PerV _ _ _ _ x _ _ _ _) = x
+    index SPDXLicenseListVersion_3_16 (PerV _ _ _ _ _ x _ _ _) = x
+    index SPDXLicenseListVersion_3_23 (PerV _ _ _ _ _ _ x _ _) = x
+    index SPDXLicenseListVersion_3_25 (PerV _ _ _ _ _ _ _ x _) = x
+    index SPDXLicenseListVersion_3_26 (PerV _ _ _ _ _ _ _ _ x) = x
 
     tabulate f = PerV
         (f SPDXLicenseListVersion_3_0)
@@ -89,6 +92,7 @@ instance Representable SPDXLicenseListVersion PerV where
         (f SPDXLicenseListVersion_3_16)
         (f SPDXLicenseListVersion_3_23)
         (f SPDXLicenseListVersion_3_25)
+        (f SPDXLicenseListVersion_3_26)
 
 -------------------------------------------------------------------------------
 -- Sorting
@@ -165,10 +169,10 @@ toConstructorName t = t
     f c   = c
 
     special :: Text -> Text
-    special "0BSD"          = "NullBSD"
-    special "389_exception" = "DS389_exception"
-    special "3D_Slicer_1_0" = "X3D_Slicer_1_0"
-    special u               = u
+    special u
+      | Just (c, _) <- T.uncons u
+      , C.isDigit c = "N_" <> u
+    special u = u
 
 mkList :: [Text] -> Text
 mkList []     = "    []"
