@@ -127,6 +127,9 @@ import Distribution.System
   ( Platform
   )
 import Distribution.Types.Dependency
+import Distribution.Types.DependencySatisfaction
+  ( DependencySatisfaction (..)
+  )
 import Distribution.Verbosity
   ( normal
   )
@@ -1092,7 +1095,7 @@ configuredPackageProblems
         case finalizePD
           specifiedFlags
           compSpec
-          (const True)
+          (const Satisfied)
           platform
           cinfo
           []
