@@ -44,6 +44,7 @@ module Distribution.Deprecated.ParseUtils
   , showFreeText
   , field
   , simpleField
+  , monoidField
   , listField
   , listFieldWithSep
   , spaceListField
@@ -52,6 +53,7 @@ module Distribution.Deprecated.ParseUtils
   , readPToMaybe
   , fieldParsec
   , simpleFieldParsec
+  , monoidFieldParsec
   , listFieldParsec
   , commaListFieldParsec
   , commaNewLineListFieldParsec
@@ -90,6 +92,7 @@ data PError
   = AmbiguousParse String LineNo
   | NoParse String LineNo
   | TabsError LineNo
+  | FieldShouldBeStanza String LineNo
   | FromString String (Maybe LineNo)
   deriving (Eq, Show)
 
@@ -185,6 +188,10 @@ locatedErrorMsg (NoParse f n) =
   , "Parse of field '" ++ f ++ "' failed."
   )
 locatedErrorMsg (TabsError n) = (Just n, "Tab used as indentation.")
+locatedErrorMsg (FieldShouldBeStanza name lineNumber) =
+  ( Just lineNumber
+  , "'" ++ name ++ "' is a stanza, not a field. Remove the trailing ':' to parse a stanza."
+  )
 locatedErrorMsg (FromString s n) = (n, s)
 
 syntaxError :: LineNo -> String -> ParseResult a
@@ -249,6 +256,32 @@ simpleFieldParsec
 simpleFieldParsec name showF readF get set =
   liftField get set $ fieldParsec name showF readF
 
+monoidField
+  :: Semigroup a
+  => String
+  -> (a -> Doc)
+  -> ReadP a a
+  -> (b -> a)
+  -> (a -> b -> b)
+  -> FieldDescr b
+monoidField name showF readF get set =
+  liftField get set' $ field name showF readF
+  where
+    set' xs b = set (get b <> xs) b
+
+monoidFieldParsec
+  :: Semigroup a
+  => String
+  -> (a -> Doc)
+  -> ParsecParser a
+  -> (b -> a)
+  -> (a -> b -> b)
+  -> FieldDescr b
+monoidFieldParsec name showF readF get set =
+  liftField get set' $ fieldParsec name showF readF
+  where
+    set' xs b = set (get b <> xs) b
+
 commaListFieldWithSepParsec
   :: Separator
   -> String
