@@ -1,6 +1,6 @@
 {-# LANGUAGE DataKinds #-}
+{-# LANGUAGE PatternSynonyms #-}
 {-# LANGUAGE RankNTypes #-}
-{-# LANGUAGE TypeApplications #-}
 
 -- | Functionality for invoking Haskell scripts with the correct
 -- package database setup.
@@ -26,7 +26,7 @@ import Distribution.Simple.Program
 import Distribution.Simple.Compiler
 import Distribution.Verbosity
 import Distribution.System
-import Distribution.Simple.Setup (Flag(..))
+import Distribution.Simple.Setup (pattern Flag)
 
 import qualified Data.Monoid as M
 
