@@ -1,19 +1,22 @@
 {-# LANGUAGE CPP #-}
 
 #ifdef STRICT
-import Data.IntMap.Strict as Data.IntMap hiding (showTree)
+import Data.IntMap.Strict as Data.IntMap
 import Data.IntMap.Strict.Internal (traverseMaybeWithKey)
 import Data.IntMap.Merge.Strict
 #else
-import Data.IntMap.Lazy as Data.IntMap hiding (showTree)
+import Data.IntMap.Lazy as Data.IntMap
 import Data.IntMap.Internal (traverseMaybeWithKey)
 import Data.IntMap.Merge.Lazy
 #endif
 import Data.IntMap.Internal.Debug (showTree)
-import IntMapValidity (valid)
+import Data.IntSet.Internal.IntTreeCommons (Prefix(..), nomatch)
+import IntMapValidity (hasPrefix, hasPrefixSimple, valid)
 
 import Control.Applicative (Applicative(..))
 import Control.Monad ((<=<))
+import qualified Data.Either as Either
+import qualified Data.Foldable as Foldable
 import Data.Monoid
 import Data.Maybe hiding (mapMaybe)
 import qualified Data.Maybe as Maybe (mapMaybe)
@@ -21,8 +24,8 @@ import Data.Ord
 import Data.Foldable (foldMap)
 import Data.Function
 import Data.Traversable (Traversable(traverse), foldMapDefault)
-import Prelude hiding (lookup, null, map, filter, foldr, foldl)
-import qualified Prelude (map)
+import Prelude hiding (lookup, null, map, filter, foldr, foldl, foldl')
+import qualified Prelude (map, filter)
 
 import Data.List (nub,sort)
 import qualified Data.List as List
@@ -31,7 +34,7 @@ import Test.Tasty
 import Test.Tasty.HUnit
 import Test.Tasty.QuickCheck
 import Test.QuickCheck.Function (apply)
-import Test.QuickCheck.Poly (A, B, C)
+import Test.QuickCheck.Poly (A, B, C, OrdA)
 
 default (Int)
 
@@ -102,7 +105,8 @@ main = defaultMain $ testGroup "intmap-properties"
              , testCase "fromAscListWithKey" test_fromAscListWithKey
              , testCase "fromDistinctAscList" test_fromDistinctAscList
              , testCase "filter" test_filter
-             , testCase "filterWithKey" test_filteWithKey
+             , testCase "filterKeys" test_filterKeys
+             , testCase "filterWithKey" test_filterWithKey
              , testCase "partition" test_partition
              , testCase "partitionWithKey" test_partitionWithKey
              , testCase "mapMaybe" test_mapMaybe
@@ -134,6 +138,7 @@ main = defaultMain $ testGroup "intmap-properties"
              , testCase "minimum" test_minimum
              , testCase "maximum" test_maximum
              , testProperty "valid"                prop_valid
+             , testProperty "hasPrefix"            prop_hasPrefix
              , testProperty "empty valid"          prop_emptyValid
              , testProperty "insert to singleton"  prop_singleton
              , testProperty "insert then lookup"   prop_insertLookup
@@ -148,6 +153,7 @@ main = defaultMain $ testGroup "intmap-properties"
              , testProperty "intersection model"   prop_intersectionModel
              , testProperty "intersectionWith model" prop_intersectionWithModel
              , testProperty "intersectionWithKey model" prop_intersectionWithKeyModel
+             , testProperty "symmetricDifference"  prop_symmetricDifference
              , testProperty "mergeWithKey model"   prop_mergeWithKeyModel
              , testProperty "merge valid"          prop_merge_valid
              , testProperty "mergeA effects"       prop_mergeA_effects
@@ -179,11 +185,17 @@ main = defaultMain $ testGroup "intmap-properties"
              , testProperty "deleteMin"            prop_deleteMinModel
              , testProperty "deleteMax"            prop_deleteMaxModel
              , testProperty "filter"               prop_filter
+             , testProperty "filterKeys"           prop_filterKeys
+             , testProperty "filterWithKey"        prop_filterWithKey
              , testProperty "partition"            prop_partition
+             , testProperty "takeWhileAntitone"    prop_takeWhileAntitone
+             , testProperty "dropWhileAntitone"    prop_dropWhileAntitone
+             , testProperty "spanAntitone"         prop_spanAntitone
              , testProperty "map"                  prop_map
              , testProperty "fmap"                 prop_fmap
              , testProperty "mapkeys"              prop_mapkeys
              , testProperty "split"                prop_splitModel
+             , testProperty "splitLookup"          prop_splitLookup
              , testProperty "splitRoot"            prop_splitRoot
              , testProperty "foldr"                prop_foldr
              , testProperty "foldr'"               prop_foldr'
@@ -205,26 +217,61 @@ main = defaultMain $ testGroup "intmap-properties"
              , testProperty "traverseMaybeWithKey identity"         prop_traverseMaybeWithKey_identity
              , testProperty "traverseMaybeWithKey->mapMaybeWithKey" prop_traverseMaybeWithKey_degrade_to_mapMaybeWithKey
              , testProperty "traverseMaybeWithKey->traverseWithKey" prop_traverseMaybeWithKey_degrade_to_traverseWithKey
+             , testProperty "isProperSubmapOfBy"   prop_isProperSubmapOfBy
+             , testProperty "isSubmapOfBy"         prop_isSubmapOfBy
+             , testProperty "insert"               prop_insert
+             , testProperty "delete"               prop_delete
+             , testProperty "insertWith"           prop_insertWith
+             , testProperty "insertWithKey"        prop_insertWithKey
+             , testProperty "insertLookupWithKey"  prop_insertLookupWithKey
+             , testProperty "adjust"               prop_adjust
+             , testProperty "adjustWithKey"        prop_adjustWithKey
+             , testProperty "update"               prop_update
+             , testProperty "updateWithKey"        prop_updateWithKey
+             , testProperty "updateLookupWithKey"  prop_updateLookupWithKey
+             , testProperty "differenceWith"       prop_differenceWith
+             , testProperty "differenceWithKey"    prop_differenceWithKey
+             , testProperty "partitionWithKey"     prop_partitionWithKey
+             , testProperty "updateMin"            prop_updateMin
+             , testProperty "updateMax"            prop_updateMax
+             , testProperty "updateMinWithKey"     prop_updateMinWithKey
+             , testProperty "updateMaxWithKey"     prop_updateMaxWithKey
+             , testProperty "minViewWithKey"       prop_minViewWithKey
+             , testProperty "maxViewWithKey"       prop_maxViewWithKey
+             , testProperty "minView"              prop_minView
+             , testProperty "maxView"              prop_maxView
+             , testProperty "mapEither"            prop_mapEither
+             , testProperty "mapEitherWithKey"     prop_mapEitherWithKey
+             , testProperty "mapAccum"             prop_mapAccum
+             , testProperty "mapAccumWithKey"      prop_mapAccumWithKey
+             , testProperty "mapAccumRWithKey"     prop_mapAccumRWithKey
+             , testProperty "mapKeysWith"          prop_mapKeysWith
+             , testProperty "mapKeysMonotonic"     prop_mapKeysMonotonic
+             , testProperty "compare"              prop_compare
              ]
 
-apply2 :: Fun (a, b) c -> a -> b -> c
-apply2 f a b = apply f (a, b)
-
-apply3 :: Fun (a, b, c) d -> a -> b -> c -> d
-apply3 f a b c = apply f (a, b, c)
-
-
 {--------------------------------------------------------------------
   Arbitrary, reasonably balanced trees
 --------------------------------------------------------------------}
 
 instance Arbitrary a => Arbitrary (IntMap a) where
-  arbitrary = fmap fromList arbitrary
+  arbitrary = oneof [go arbitrary, go (getLarge <$> arbitrary)]
+    where
+      go kgen = fromList <$> listOf ((,) <$> kgen <*> arbitrary)
+  shrink = fmap fromList . shrink . toAscList
 
 newtype NonEmptyIntMap a = NonEmptyIntMap {getNonEmptyIntMap :: IntMap a} deriving (Eq, Show)
 
 instance Arbitrary a => Arbitrary (NonEmptyIntMap a) where
-  arbitrary = fmap (NonEmptyIntMap . fromList . getNonEmpty) arbitrary
+  arbitrary = oneof [go arbitrary, go (getLarge <$> arbitrary)]
+    where
+      go kgen = NonEmptyIntMap . fromList <$> listOf1 ((,) <$> kgen <*> arbitrary)
+  shrink =
+    fmap (NonEmptyIntMap . fromList) .
+    List.filter (not . List.null) .
+    shrink .
+    toAscList .
+    getNonEmptyIntMap
 
 
 ------------------------------------------------------------------------
@@ -854,8 +901,13 @@ test_filter = do
     filter (> "x") (fromList [(5,"a"), (-3,"b")]) @?= empty
     filter (< "a") (fromList [(5,"a"), (-3,"b")]) @?= empty
 
-test_filteWithKey :: Assertion
-test_filteWithKey = do
+test_filterKeys :: Assertion
+test_filterKeys = do
+    filterKeys (> 4) (fromList [(5,"a"), (3,"b")])  @?= singleton 5 "a"
+    filterKeys (> 4) (fromList [(5,"a"), (-3,"b")]) @?= singleton 5 "a"
+
+test_filterWithKey :: Assertion
+test_filterWithKey = do
     filterWithKey (\k _ -> k > 4) (fromList [(5,"a"), (3,"b")])  @?= singleton 5 "a"
     filterWithKey (\k _ -> k > 4) (fromList [(5,"a"), (-3,"b")]) @?= singleton 5 "a"
 
@@ -995,6 +1047,9 @@ test_isProperSubmapOfBy = do
     isProperSubmapOfBy (==) (fromList [(-1,1),(2,2)]) (fromList [(-1,1)]) @?= False
     isProperSubmapOfBy (<)  (fromList [(-1,1)])       (fromList [(-1,1),(2,2)]) @?= False
 
+    -- See Github #1007
+    isProperSubmapOfBy (==) (fromList [(-3,1),(-1,1)]) (fromList [(-3,1),(-1,1),(0,1)]) @?= True
+
 test_isProperSubmapOf :: Assertion
 test_isProperSubmapOf = do
     isProperSubmapOf (fromList [(1,1)]) (fromList [(1,1),(2,2)]) @?= True
@@ -1005,6 +1060,9 @@ test_isProperSubmapOf = do
     isProperSubmapOf (fromList [(-1,1),(2,2)]) (fromList [(-1,1),(2,2)]) @?= False
     isProperSubmapOf (fromList [(-1,1),(2,2)]) (fromList [(-1,1)]) @?= False
 
+    -- See Github #1007
+    isProperSubmapOf (fromList [(-3,1),(-1,1)]) (fromList [(-3,1),(-1,1),(0,1)]) @?= True
+
 ----------------------------------------------------------------
 -- Min/Max
 
@@ -1060,6 +1118,9 @@ test_updateMin = do
     updateMin (\ a -> Just ("X" ++ a)) (fromList [(5,"a"), (-3,"b")]) @?= fromList [(-3, "Xb"), (5, "a")]
     updateMin (\ _ -> Nothing)         (fromList [(5,"a"), (-3,"b")]) @?= singleton 5 "a"
 
+    updateMin (\ a -> Just ("X" ++ a)) (empty :: SMap) @?= empty
+    updateMin (\ _ -> Nothing)         (empty :: SMap) @?= empty
+
 test_updateMax :: Assertion
 test_updateMax = do
     updateMax (\ a -> Just ("X" ++ a)) (fromList [(5,"a"), (3,"b")]) @?= fromList [(3, "b"), (5, "Xa")]
@@ -1068,6 +1129,9 @@ test_updateMax = do
     updateMax (\ a -> Just ("X" ++ a)) (fromList [(5,"a"), (-3,"b")]) @?= fromList [(-3, "b"), (5, "Xa")]
     updateMax (\ _ -> Nothing)         (fromList [(5,"a"), (-3,"b")]) @?= singleton (-3) "b"
 
+    updateMax (\ a -> Just ("X" ++ a)) (empty :: SMap) @?= empty
+    updateMax (\ _ -> Nothing)         (empty :: SMap) @?= empty
+
 test_updateMinWithKey :: Assertion
 test_updateMinWithKey = do
     updateMinWithKey (\ k a -> Just ((show k) ++ ":" ++ a)) (fromList [(5,"a"), (3,"b")]) @?= fromList [(3,"3:b"), (5,"a")]
@@ -1076,6 +1140,9 @@ test_updateMinWithKey = do
     updateMinWithKey (\ k a -> Just ((show k) ++ ":" ++ a)) (fromList [(5,"a"), (-3,"b")]) @?= fromList [(-3,"-3:b"), (5,"a")]
     updateMinWithKey (\ _ _ -> Nothing)                     (fromList [(5,"a"), (-3,"b")]) @?= singleton 5 "a"
 
+    updateMinWithKey (\ k a -> Just ((show k) ++ ":" ++ a)) (empty :: SMap) @?= empty
+    updateMinWithKey (\ _ _ -> Nothing)                     (empty :: SMap) @?= empty
+
 test_updateMaxWithKey :: Assertion
 test_updateMaxWithKey = do
     updateMaxWithKey (\ k a -> Just ((show k) ++ ":" ++ a)) (fromList [(5,"a"), (3,"b")]) @?= fromList [(3,"b"), (5,"5:a")]
@@ -1084,6 +1151,9 @@ test_updateMaxWithKey = do
     updateMaxWithKey (\ k a -> Just ((show k) ++ ":" ++ a)) (fromList [(5,"a"), (-3,"b")]) @?= fromList [(-3,"b"), (5,"5:a")]
     updateMaxWithKey (\ _ _ -> Nothing)                     (fromList [(5,"a"), (-3,"b")]) @?= singleton (-3) "b"
 
+    updateMaxWithKey (\ k a -> Just ((show k) ++ ":" ++ a)) (empty :: SMap) @?= empty
+    updateMaxWithKey (\ _ _ -> Nothing)                     (empty :: SMap) @?= empty
+
 test_minView :: Assertion
 test_minView = do
     minView (fromList [(5,"a"), (3,"b")]) @?= Just ("b", singleton 5 "a")
@@ -1146,6 +1216,10 @@ forValidUnitTree f = forValid f
 prop_valid :: Property
 prop_valid = forValidUnitTree $ \t -> valid t
 
+prop_hasPrefix :: Int -> NonZero Int -> Property
+prop_hasPrefix i (NonZero p) =
+  hasPrefix i (Prefix p) === hasPrefixSimple i (Prefix p)
+
 ----------------------------------------------------------------
 -- QuickCheck
 ----------------------------------------------------------------
@@ -1229,6 +1303,17 @@ prop_intersectionWithKeyModel xs ys
           ys' = List.nubBy ((==) `on` fst) ys
           f k l r = k + 2 * l + 3 * r
 
+prop_symmetricDifference :: IMap -> IMap -> Property
+prop_symmetricDifference m1 m2 =
+  valid m3 .&&.
+  toAscList m3 ===
+  List.sort (   List.filter ((`notElem` fmap fst kys) . fst) kxs
+             ++ List.filter ((`notElem` fmap fst kxs) . fst) kys)
+  where
+    m3 = symmetricDifference m1 m2
+    kxs = toAscList m1
+    kys = toAscList m2
+
 prop_disjoint :: UMap -> UMap -> Property
 prop_disjoint m1 m2 = disjoint m1 m2 === null (intersection m1 m2)
 
@@ -1403,31 +1488,17 @@ prop_findWithDefault xs n x =
       m = fromList xs'
   in all (\k -> findWithDefault x k m == maybe x id (List.lookup k xs')) (n : List.map fst xs')
 
-test_lookupSomething :: (Int -> IntMap Int -> Maybe (Int, Int)) -> (Int -> Int -> Bool) -> [(Int, Int)] -> Bool
-test_lookupSomething lookup' cmp xs =
-  let odd_sorted_xs = filter_odd $ sort $ List.nubBy ((==) `on` fst) xs
-      t = fromList odd_sorted_xs
-      test k = case List.filter ((`cmp` k) . fst) odd_sorted_xs of
-                 []             -> lookup' k t == Nothing
-                 cs | 0 `cmp` 1 -> lookup' k t == Just (last cs) -- we want largest such element
-                    | otherwise -> lookup' k t == Just (head cs) -- we want smallest such element
-  in all test (List.map fst xs)
+prop_lookupLT :: Int -> IntMap A -> Bool
+prop_lookupLT k m = lookupLT k m == List.find ((<k) . fst) (toDescList m)
 
-  where filter_odd [] = []
-        filter_odd [_] = []
-        filter_odd (_ : o : xs) = o : filter_odd xs
+prop_lookupGT :: Int -> IntMap A -> Bool
+prop_lookupGT k m = lookupGT k m == List.find ((>k) . fst) (toList m)
 
-prop_lookupLT :: [(Int, Int)] -> Bool
-prop_lookupLT = test_lookupSomething lookupLT (<)
+prop_lookupLE :: Int -> IntMap A -> Bool
+prop_lookupLE k m = lookupLE k m == List.find ((<=k) . fst) (toDescList m)
 
-prop_lookupGT :: [(Int, Int)] -> Bool
-prop_lookupGT = test_lookupSomething lookupGT (>)
-
-prop_lookupLE :: [(Int, Int)] -> Bool
-prop_lookupLE = test_lookupSomething lookupLE (<=)
-
-prop_lookupGE :: [(Int, Int)] -> Bool
-prop_lookupGE = test_lookupSomething lookupGE (>=)
+prop_lookupGE :: Int -> IntMap A -> Bool
+prop_lookupGE k m = lookupGE k m == List.find ((>=k) . fst) (toList m)
 
 prop_lookupMin :: IntMap Int -> Property
 prop_lookupMin im = lookupMin im === listToMaybe (toAscList im)
@@ -1460,6 +1531,18 @@ prop_filter p ys = length ys > 0 ==>
   in  valid m .&&.
       m === fromList (List.filter (apply p . snd) xs)
 
+prop_filterKeys :: Fun Int Bool -> IMap -> Property
+prop_filterKeys fun m =
+  valid m' .&&. toList m' === Prelude.filter (apply fun . fst) (toList m)
+  where
+    m' = filterKeys (apply fun) m
+
+prop_filterWithKey :: Fun (Int, Int) Bool -> IMap -> Property
+prop_filterWithKey fun m =
+  valid m' .&&. toList m' === Prelude.filter (apply fun) (toList m)
+  where
+    m' = filterWithKey (applyFun2 fun) m
+
 prop_partition :: Fun Int Bool -> [(Int, Int)] -> Property
 prop_partition p ys = length ys > 0 ==>
   let xs = List.nubBy ((==) `on` fst) ys
@@ -1469,6 +1552,26 @@ prop_partition p ys = length ys > 0 ==>
       m === let (a,b) = (List.partition (apply p . snd) xs)
             in (fromList a, fromList b)
 
+prop_takeWhileAntitone :: Int -> [(Int, Int)] -> Property
+prop_takeWhileAntitone x ys =
+  let l = takeWhileAntitone (<x) (fromList ys)
+  in  valid l .&&.
+      l === fromList (List.filter ((<x) . fst) ys)
+
+prop_dropWhileAntitone :: Int -> [(Int, Int)] -> Property
+prop_dropWhileAntitone x ys =
+  let r = dropWhileAntitone (<x) (fromList ys)
+  in  valid r .&&.
+      r === fromList (List.filter ((>=x) . fst) ys)
+
+prop_spanAntitone :: Int -> [(Int, Int)] -> Property
+prop_spanAntitone x ys =
+  let (l, r) = spanAntitone (<x) (fromList ys)
+  in  valid l .&&.
+      valid r .&&.
+      l === fromList (List.filter ((<x) . fst) ys) .&&.
+      r === fromList (List.filter ((>=x) . fst) ys)
+
 prop_map :: Fun Int Int -> [(Int, Int)] -> Property
 prop_map f ys = length ys > 0 ==>
   let xs = List.nubBy ((==) `on` fst) ys
@@ -1496,6 +1599,16 @@ prop_splitModel n ys = length ys > 0 ==>
       toAscList l === sort [(k, v) | (k,v) <- xs, k < n] .&&.
       toAscList r === sort [(k, v) | (k,v) <- xs, k > n]
 
+prop_splitLookup :: Int -> [(Int, Int)] -> Property
+prop_splitLookup n ys =
+    let xs = List.nubBy ((==) `on` fst) ys
+        (l, x, r) = splitLookup n (fromList xs)
+    in  valid l .&&.
+        valid r .&&.
+        x === List.lookup n xs .&&.
+        toAscList l === sort [(k, v) | (k,v) <- xs, k < n] .&&.
+        toAscList r === sort [(k, v) | (k,v) <- xs, k > n]
+
 prop_splitRoot :: IMap -> Bool
 prop_splitRoot s = loop ls && (s == unions ls)
  where
@@ -1607,3 +1720,267 @@ prop_traverseMaybeWithKey_degrade_to_traverseWithKey fun mp =
         -- so this also checks the order of traversing is the same.
   where f k v = (show k, applyFun2 fun k v)
         g k v = fmap Just $ f k v
+
+prop_isProperSubmapOfBy :: Fun (A, A) Bool -> IntMap A -> IntMap A -> Property
+prop_isProperSubmapOfBy f m1 m2 =
+  isProperSubmapOfBy (applyFun2 f) m1 m2 ===
+  (length xs == size m1 && size m1 < size m2)
+  where
+    xs = List.intersectBy
+           (\(k1,x1) (k2,x2) -> k1 == k2 && applyFun2 f x1 x2)
+           (assocs m1) (assocs m2)
+
+prop_isSubmapOfBy :: Fun (A, A) Bool -> IntMap A -> IntMap A -> Property
+prop_isSubmapOfBy f m1 m2 =
+  isSubmapOfBy (applyFun2 f) m1 m2 ===
+  (length xs == size m1)
+  where
+    xs = List.intersectBy
+           (\(k1,x1) (k2,x2) -> k1 == k2 && applyFun2 f x1 x2)
+           (assocs m1) (assocs m2)
+
+prop_insert :: Int -> A -> IntMap A -> Property
+prop_insert k x m = valid m' .&&. toList m' === kxs'
+  where
+    m' = insert k x m
+    kxs = toList m
+    kxs' = List.insertBy (comparing fst) (k,x) $
+           maybe kxs (\y -> kxs List.\\ [(k,y)]) (List.lookup k kxs)
+
+prop_delete :: Int -> IntMap A -> Property
+prop_delete k m = valid m' .&&. toList m' === kxs'
+  where
+    m' = delete k m
+    kxs = toList m
+    kxs' = maybe kxs (\v -> kxs List.\\ [(k,v)]) (List.lookup k kxs)
+
+prop_insertWith :: Fun (A, A) A -> Int -> A -> IntMap A -> Property
+prop_insertWith f k x m = valid m' .&&. toList m' === kxs'
+  where
+    m' = insertWith (applyFun2 f) k x m
+    kxs = toList m
+    kxs' = if k `List.elem` (fmap fst kxs)
+           then let g (k',x') = (k', if k == k' then applyFun2 f x x' else x')
+                in fmap g kxs
+           else List.insertBy (comparing fst) (k,x) kxs
+
+prop_insertWithKey :: Fun (Int, A, A) A -> Int -> A -> IntMap A -> Property
+prop_insertWithKey f k x m = valid m' .&&. toList m' === kxs'
+  where
+    m' = insertWithKey (applyFun3 f) k x m
+    kxs = toList m
+    kxs' = if k `List.elem` (fmap fst kxs)
+           then let g (k',x') = (k', if k == k' then applyFun3 f k x x' else x')
+                in fmap g kxs
+           else List.insertBy (comparing fst) (k,x) kxs
+
+prop_insertLookupWithKey
+  :: Fun (Int, A, A) A -> Int -> A -> IntMap A -> Property
+prop_insertLookupWithKey f k x m =
+  valid m' .&&.
+  r === (lookup k m, insertWith (applyFun3 f k) k x m)
+  where
+    r@(_, m') = insertLookupWithKey (applyFun3 f) k x m
+
+prop_adjust :: Fun A A -> Int -> IntMap A -> Property
+prop_adjust f k m =
+  valid m' .&&.
+  m' === maybe m (\x -> insert k (applyFun f x) m) (lookup k m)
+  where
+    m' = adjust (applyFun f) k m
+
+prop_adjustWithKey :: Fun (Int, A) A -> Int -> IntMap A -> Property
+prop_adjustWithKey f k m =
+  valid m' .&&.
+  m' === maybe m (\x -> insert k (applyFun2 f k x) m) (lookup k m)
+  where
+    m' = adjustWithKey (applyFun2 f) k m
+
+prop_update :: Fun A (Maybe A) -> Int -> IntMap A -> Property
+prop_update f k m = valid m' .&&. m' === m''
+  where
+    m' = update (applyFun f) k m
+    m'' = case lookup k m of
+      Nothing -> m
+      Just x -> case applyFun f x of
+        Nothing -> delete k m
+        Just x' -> insert k x' m
+
+prop_updateWithKey :: Fun (Int, A) (Maybe A) -> Int -> IntMap A -> Property
+prop_updateWithKey f k m = valid m' .&&. m' === m''
+  where
+    m' = updateWithKey (applyFun2 f) k m
+    m'' = case lookup k m of
+      Nothing -> m
+      Just x -> case applyFun2 f k x of
+        Nothing -> delete k m
+        Just x' -> insert k x' m
+
+prop_updateLookupWithKey
+  :: Fun (Int, A) (Maybe A) -> Int -> IntMap A -> Property
+prop_updateLookupWithKey f k m = valid m' .&&. r === (lookup k m, m'')
+  where
+    r@(_, m') = updateLookupWithKey (applyFun2 f) k m
+    m'' = updateWithKey (applyFun2 f) k m
+
+prop_differenceWith
+  :: Fun (A, B) (Maybe A) -> IntMap A -> IntMap B -> Property
+prop_differenceWith f m1 m2 =
+  valid m' .&&.
+  toList m' === Maybe.mapMaybe f' kxs
+  where
+    m' = differenceWith (applyFun2 f) m1 m2
+    kxs = toList m1
+    kys = toList m2
+    f' (k,x) = case List.lookup k kys of
+      Nothing -> Just (k,x)
+      Just y -> (,) k <$> applyFun2 f x y
+
+prop_differenceWithKey
+  :: Fun (Int, A, B) (Maybe A) -> IntMap A -> IntMap B -> Property
+prop_differenceWithKey f m1 m2 =
+  valid m' .&&.
+  toList m' === Maybe.mapMaybe f' kxs
+  where
+    m' = differenceWithKey (applyFun3 f) m1 m2
+    kxs = toList m1
+    kys = toList m2
+    f' (k,x) = case List.lookup k kys of
+      Nothing -> Just (k,x)
+      Just y -> (,) k <$> applyFun3 f k x y
+
+prop_partitionWithKey :: Fun (Int, A) Bool -> IntMap A -> Property
+prop_partitionWithKey f m =
+  valid m1 .&&.
+  valid m2 .&&.
+  (toList m1, toList m2) === List.partition (applyFun f) (toList m)
+  where
+    (m1, m2) = partitionWithKey (applyFun2 f) m
+
+prop_updateMin :: Fun A (Maybe A) -> IntMap A -> Property
+prop_updateMin f m =
+  valid m' .&&.
+  toList m' === case toList m of
+    [] -> []
+    (k,x):xs -> Foldable.toList ((,) k <$> applyFun f x) ++ xs
+  where
+    m' = updateMin (applyFun f) m
+
+prop_updateMax :: Fun A (Maybe A) -> IntMap A -> Property
+prop_updateMax f m =
+  valid m' .&&.
+  toDescList m' === case toDescList m of
+    [] -> []
+    (k,x):xs -> Foldable.toList ((,) k <$> applyFun f x) ++ xs
+  where
+    m' = updateMax (applyFun f) m
+
+prop_updateMinWithKey :: Fun (Int, A) (Maybe A) -> IntMap A -> Property
+prop_updateMinWithKey f m =
+  valid m' .&&.
+  toList m' === case toList m of
+    [] -> []
+    (k,x):xs -> Foldable.toList ((,) k <$> applyFun2 f k x) ++ xs
+  where
+    m' = updateMinWithKey (applyFun2 f) m
+
+prop_updateMaxWithKey :: Fun (Int, A) (Maybe A) -> IntMap A -> Property
+prop_updateMaxWithKey f m =
+  valid m' .&&.
+  toDescList m' === case toDescList m of
+    [] -> []
+    (k,x):xs -> Foldable.toList ((,) k <$> applyFun2 f k x) ++ xs
+  where
+    m' = updateMaxWithKey (applyFun2 f) m
+
+prop_minViewWithKey :: IntMap A -> Property
+prop_minViewWithKey m = case minViewWithKey m of
+  Nothing -> property $ null m
+  Just ((k,x),m') -> valid m .&&. toList m === (k,x) : toList m'
+
+prop_maxViewWithKey :: IntMap A -> Property
+prop_maxViewWithKey m = case maxViewWithKey m of
+  Nothing -> property $ null m
+  Just ((k,x),m') -> valid m .&&. toDescList m === (k,x) : toDescList m'
+
+prop_minView :: IntMap A -> Property
+prop_minView m = case minView m of
+  Nothing -> property $ null m
+  Just (x,m') | ~(_,x'):xs' <- toList m ->
+    valid m' .&&. x == x' .&&. toList m' === xs'
+
+prop_maxView :: IntMap A -> Property
+prop_maxView m = case maxView m of
+  Nothing -> property $ null m
+  Just (x,m') | ~(_,x'):xs' <- toDescList m ->
+    valid m' .&&. x == x' .&&. toDescList m' === xs'
+
+prop_mapEither :: Fun A (Either B C) -> IntMap A -> Property
+prop_mapEither f m =
+  valid m1 .&&.
+  valid m2 .&&.
+  (toList m1, toList m2) === Either.partitionEithers (List.map f' (toList m))
+  where
+    (m1, m2) = mapEither (applyFun f) m
+    f' (k,x) = case applyFun f x of
+      Left y -> Left (k,y)
+      Right z -> Right (k,z)
+
+prop_mapEitherWithKey :: Fun (Int, A) (Either B C) -> IntMap A -> Property
+prop_mapEitherWithKey f m =
+  valid m1 .&&.
+  valid m2 .&&.
+  (toList m1, toList m2) === Either.partitionEithers (List.map f' (toList m))
+  where
+    (m1, m2) = mapEitherWithKey (applyFun2 f) m
+    f' (k,x) = case applyFun2 f k x of
+      Left y -> Left (k,y)
+      Right z -> Right (k,z)
+
+prop_mapAccum :: Fun (A, B) (A, C) -> A -> IntMap B -> Property
+prop_mapAccum f z0 m =
+  valid m' .&&.
+  (z, toList m') === List.mapAccumL f' z0 (toList m)
+  where
+    (z, m') = mapAccum (applyFun2 f) z0 m
+    f' z' (k,x) = (,) k <$> applyFun2 f z' x
+
+prop_mapAccumWithKey :: Fun (A, Int, B) (A, C) -> A -> IntMap B -> Property
+prop_mapAccumWithKey f z0 m =
+  valid m' .&&.
+  (z, toList m') === List.mapAccumL f' z0 (toList m)
+  where
+    (z, m') = mapAccumWithKey (applyFun3 f) z0 m
+    f' z' (k,x) = (,) k <$> applyFun3 f z' k x
+
+prop_mapAccumRWithKey :: Fun (A, Int, B) (A, C) -> A -> IntMap B -> Property
+prop_mapAccumRWithKey f z0 m =
+  valid m' .&&.
+  (z, toList m') === List.mapAccumR f' z0 (toList m)
+  where
+    (z, m') = mapAccumRWithKey (applyFun3 f) z0 m
+    f' z' (k,x) = (,) k <$> applyFun3 f z' k x
+
+prop_mapKeysWith :: Fun (A, A) A -> Fun Int Int -> IntMap A -> Property
+prop_mapKeysWith f g m =
+  valid m' .&&.
+  toList m' ===
+    toList (fromListWith (applyFun2 f) [(applyFun g k, x) | (k,x) <- toList m])
+  where
+    m' = mapKeysWith (applyFun2 f) (applyFun g) m
+
+prop_mapKeysMonotonic :: Positive Int -> Int -> IntMap A -> Property
+prop_mapKeysMonotonic (Positive a) b m =
+  all ok (keys m) ==>
+    valid m' .&&.
+    toList m' == [(f k, x) | (k,x) <- toList m]
+  where
+    m' = mapKeysMonotonic f m
+    f x = x * a + b
+    ok x =  -- must not overflow
+      fromIntegral (minBound :: Int) <= y && y <= fromIntegral (maxBound :: Int)
+      where
+        y = fromIntegral a * fromIntegral x + fromIntegral b :: Integer
+
+prop_compare :: IntMap OrdA -> IntMap OrdA -> Property
+prop_compare m1 m2 = compare m1 m2 === compare (toList m1) (toList m2)
