@@ -1,25 +1,25 @@
 {-# LANGUAGE CPP #-}
 import qualified Data.IntSet as IntSet
-import Data.List (nub,sort)
+import Data.List (nub, sort, sortBy)
 import qualified Data.List as List
-import Data.Monoid (mempty)
 import Data.Maybe
 import Data.Set
-import Prelude hiding (lookup, null, map, filter, foldr, foldl, all, take, drop, splitAt)
+import Data.Set.Internal (link, merge)
+import Prelude hiding (lookup, null, map, filter, foldr, foldl, foldl', all, take, drop, splitAt)
 import Test.Tasty
 import Test.Tasty.HUnit
 import Test.Tasty.QuickCheck
 import Test.QuickCheck.Function (apply)
 import Control.Monad.Trans.State.Strict
 import Control.Monad.Trans.Class
-import Control.Monad (liftM, liftM3)
 import Data.Functor.Identity
 import Data.Foldable (all)
+import Data.Ord (Down(..), comparing)
 import Control.Applicative (liftA2)
+import Data.List.NonEmpty (NonEmpty(..))
+import qualified Data.List.NonEmpty as NE
 
-#if __GLASGOW_HASKELL__ >= 806
-import Utils.NoThunks (whnfHasNoThunks)
-#endif
+import Utils.ArbitrarySetMap (mkArbSet, setFromList)
 
 main :: IO ()
 main = defaultMain $ testGroup "set-properties"
@@ -67,11 +67,15 @@ main = defaultMain $ testGroup "set-properties"
                    , testProperty "prop_DescList" prop_DescList
                    , testProperty "prop_AscDescList" prop_AscDescList
                    , testProperty "prop_fromList" prop_fromList
-                   , testProperty "prop_fromListDesc" prop_fromListDesc
+                   , testProperty "prop_fromAscList" prop_fromAscList
+                   , testProperty "prop_fromDistinctAscList" prop_fromDistinctAscList
+                   , testProperty "prop_fromDescList" prop_fromDescList
+                   , testProperty "prop_fromDistinctDescList" prop_fromDistinctDescList
                    , testProperty "prop_isProperSubsetOf" prop_isProperSubsetOf
                    , testProperty "prop_isProperSubsetOf2" prop_isProperSubsetOf2
                    , testProperty "prop_isSubsetOf" prop_isSubsetOf
                    , testProperty "prop_isSubsetOf2" prop_isSubsetOf2
+                   , testProperty "prop_symmetricDifference" prop_symmetricDifference
                    , testProperty "prop_disjoint" prop_disjoint
                    , testProperty "prop_size" prop_size
                    , testProperty "prop_lookupMax" prop_lookupMax
@@ -103,10 +107,18 @@ main = defaultMain $ testGroup "set-properties"
                    , testProperty "powerSet"             prop_powerSet
                    , testProperty "cartesianProduct"     prop_cartesianProduct
                    , testProperty "disjointUnion"        prop_disjointUnion
-#if __GLASGOW_HASKELL__ >= 806
-                   , testProperty "strict foldr"         prop_strictFoldr'
-                   , testProperty "strict foldr"         prop_strictFoldl'
-#endif
+                   , testProperty "eq" prop_eq
+                   , testProperty "compare" prop_compare
+                   , testProperty "intersections" prop_intersections
+                   , testProperty "intersections_lazy" prop_intersections_lazy
+                   , testProperty "insert" prop_insert
+                   , testProperty "delete" prop_delete
+                   , testProperty "deleteMin" prop_deleteMin
+                   , testProperty "deleteMax" prop_deleteMax
+                   , testProperty "findIndex" prop_findIndex
+                   , testProperty "lookupIndex" prop_lookupIndex
+                   , testProperty "elemAt" prop_elemAt
+                   , testProperty "deleteAt" prop_deleteAt
                    ]
 
 -- A type with a peculiar Eq instance designed to make sure keys
@@ -219,7 +231,7 @@ instance IsInt a => Arbitrary (Set a) where
         middle <- choose (-positionFactor * (sz + 1), positionFactor * (sz + 1))
         let shift = (sz * (gapRange) + 1) `quot` 2
             start = middle - shift
-        t <- evalStateT (mkArb step sz) start
+        t <- evalStateT (mkArbSet step sz) start
         if valid t then pure t else error "Test generated invalid tree!")
     where
       step = do
@@ -229,47 +241,6 @@ instance IsInt a => Arbitrary (Set a) where
         put i'
         pure (fromInt i')
 
-class Monad m => MonadGen m where
-  liftGen :: Gen a -> m a
-instance MonadGen Gen where
-  liftGen = id
-instance MonadGen m => MonadGen (StateT s m) where
-  liftGen = lift . liftGen
-
--- | Given an action that produces successively larger elements and
--- a size, produce a set of arbitrary shape with exactly that size.
-mkArb :: MonadGen m => m a -> Int -> m (Set a)
-mkArb step n
-  | n <= 0 = return Tip
-  | n == 1 = singleton `liftM` step
-  | n == 2 = do
-     dir <- liftGen arbitrary
-     p <- step
-     q <- step
-     if dir
-       then return (Bin 2 q (singleton p) Tip)
-       else return (Bin 2 p Tip (singleton q))
-  | otherwise = do
-      -- This assumes a balance factor of delta = 3
-      let upper = (3*(n - 1)) `quot` 4
-      let lower = (n + 2) `quot` 4
-      ln <- liftGen $ choose (lower, upper)
-      let rn = n - ln - 1
-      liftM3 (\lt x rt -> Bin n x lt rt) (mkArb step ln) step (mkArb step rn)
-
--- | Given a strictly increasing list of elements, produce an arbitrarily
--- shaped set with exactly those elements.
-setFromList :: [a] -> Gen (Set a)
-setFromList xs = flip evalStateT xs $ mkArb step (length xs)
-  where
-    step = do
-      xxs <- get
-      case xxs of
-        x : xs -> do
-          put xs
-          pure x
-        [] -> error "setFromList"
-
 data TwoSets = TwoSets (Set Int) (Set Int) deriving (Show)
 
 data TwoLists a = TwoLists [a] [a]
@@ -335,31 +306,17 @@ prop_NotMember xs n =
   let m  = fromList xs
   in all (\k -> k `notMember` m == (k `notElem` xs)) (n : xs)
 
-test_LookupSomething :: (Int -> Set Int -> Maybe Int) -> (Int -> Int -> Bool) -> [Int] -> Bool
-test_LookupSomething lookup' cmp xs =
-  let odd_sorted_xs = filter_odd $ nub $ sort xs
-      t = fromList odd_sorted_xs
-      test x = case List.filter (`cmp` x) odd_sorted_xs of
-                 []             -> lookup' x t == Nothing
-                 cs | 0 `cmp` 1 -> lookup' x t == Just (last cs) -- we want largest such element
-                    | otherwise -> lookup' x t == Just (head cs) -- we want smallest such element
-  in all test xs
+prop_LookupLT :: Int -> Set Int -> Property
+prop_LookupLT x xs = lookupLT x xs === List.find (<x) (toDescList xs)
 
-  where filter_odd [] = []
-        filter_odd [_] = []
-        filter_odd (_ : o : xs) = o : filter_odd xs
+prop_LookupGT :: Int -> Set Int -> Property
+prop_LookupGT x xs = lookupGT x xs === List.find (>x) (toList xs)
 
-prop_LookupLT :: [Int] -> Bool
-prop_LookupLT = test_LookupSomething lookupLT (<)
+prop_LookupLE :: Int -> Set Int -> Property
+prop_LookupLE x xs = lookupLE x xs === List.find (<=x) (toDescList xs)
 
-prop_LookupGT :: [Int] -> Bool
-prop_LookupGT = test_LookupSomething lookupGT (>)
-
-prop_LookupLE :: [Int] -> Bool
-prop_LookupLE = test_LookupSomething lookupLE (<=)
-
-prop_LookupGE :: [Int] -> Bool
-prop_LookupGE = test_LookupSomething lookupGE (>=)
+prop_LookupGE :: Int -> Set Int -> Property
+prop_LookupGE x xs = lookupGE x xs === List.find (>=x) (toList xs)
 
 prop_InsertValid :: Int -> Property
 prop_InsertValid k = forValidUnitTree $ \t -> valid (insert k t)
@@ -485,6 +442,16 @@ prop_Int :: [Int] -> [Int] -> Bool
 prop_Int xs ys = toAscList (intersection (fromList xs) (fromList ys))
                  == List.sort (nub ((List.intersect) (xs)  (ys)))
 
+prop_symmetricDifference :: Set Int -> Set Int -> Property
+prop_symmetricDifference xs ys =
+  valid zs .&&.
+  toAscList zs ===
+  List.sort (List.filter (`notElem` xs') ys' ++ List.filter (`notElem` ys') xs')
+  where
+    zs = symmetricDifference xs ys
+    xs' = toAscList xs
+    ys' = toAscList ys
+
 prop_disjoint :: Set Int -> Set Int -> Bool
 prop_disjoint a b = a `disjoint` b == null (a `intersection` b)
 
@@ -513,21 +480,43 @@ prop_AscDescList xs = toAscList s == reverse (toDescList s)
 
 prop_fromList :: [Int] -> Property
 prop_fromList xs =
-           t === fromAscList sort_xs .&&.
-           t === fromDistinctAscList nub_sort_xs .&&.
+           valid t .&&.
            t === List.foldr insert empty xs
   where t = fromList xs
-        sort_xs = sort xs
-        nub_sort_xs = List.map List.head $ List.group sort_xs
 
-prop_fromListDesc :: [Int] -> Property
-prop_fromListDesc xs =
-           t === fromDescList sort_xs .&&.
-           t === fromDistinctDescList nub_sort_xs .&&.
-           t === List.foldr insert empty xs
-  where t = fromList xs
-        sort_xs = reverse (sort xs)
-        nub_sort_xs = List.map List.head $ List.group sort_xs
+prop_fromAscList :: [Int] -> Property
+prop_fromAscList xs =
+    valid t .&&.
+    toList t === nubSortedXs
+  where
+    sortedXs = sort xs
+    nubSortedXs = List.map NE.head $ NE.group sortedXs
+    t = fromAscList sortedXs
+
+prop_fromDistinctAscList :: [Int] -> Property
+prop_fromDistinctAscList xs =
+    valid t .&&.
+    toList t === nubSortedXs
+  where
+    nubSortedXs = List.map NE.head $ NE.group $ sort xs
+    t = fromDistinctAscList nubSortedXs
+
+prop_fromDescList :: [Int] -> Property
+prop_fromDescList xs =
+    valid t .&&.
+    toList t === reverse nubDownSortedXs
+  where
+    downSortedXs = sortBy (comparing Down) xs
+    nubDownSortedXs = List.map NE.head $ NE.group downSortedXs
+    t = fromDescList downSortedXs
+
+prop_fromDistinctDescList :: [Int] -> Property
+prop_fromDistinctDescList xs =
+    valid t .&&.
+    toList t === reverse nubDownSortedXs
+  where
+    nubDownSortedXs = List.map NE.head $ NE.group $ sortBy (comparing Down) xs
+    t = fromDistinctDescList nubDownSortedXs
 
 {--------------------------------------------------------------------
   Set operations are like IntSet operations
@@ -669,16 +658,16 @@ prop_spanAntitone xs' = valid tw .&&. valid dw
     xs = fromList xs'
     (tw, dw) = spanAntitone isLeft xs
 
-prop_powerSet :: Set Int -> Property
-prop_powerSet xs = valid ps .&&. ps === ps'
-  where
-    xs' = take 10 xs
-
-    ps = powerSet xs'
-    ps' = fromList . fmap fromList $ lps (toList xs')
-
-    lps [] = [[]]
-    lps (y : ys) = fmap (y:) (lps ys) ++ lps ys
+prop_powerSet :: Property
+prop_powerSet = forAll (resize 10 arbitrary :: Gen (Set Int)) $ \xs ->
+   -- We don't actually have to check on the values directly, because the power
+   -- set is the *only* one that can be produced by a function with the type of
+   -- `powerSet` and satisfy the criteria below. In particular, the `valid ps`
+   -- test ensures that we haven't duplicated any subsets, while the size test
+   -- ensures that we haven't omitted any. Parametricity ensures that we
+   -- haven't produced any elements out of thin air.
+   let ps = powerSet xs
+   in valid ps .&&. all valid ps .&&. size ps === 2^size xs
 
 prop_cartesianProduct :: Set Int -> Set Int -> Property
 prop_cartesianProduct xs ys =
@@ -694,12 +683,58 @@ isLeft :: Either a b -> Bool
 isLeft (Left _) = True
 isLeft _ = False
 
-#if __GLASGOW_HASKELL__ >= 806
-prop_strictFoldr' :: Set Int -> Property
-prop_strictFoldr' m = whnfHasNoThunks (foldr' (:) [] m)
-#endif
+prop_eq :: Set Int -> Set Int -> Property
+prop_eq s1 s2 = (s1 == s2) === (toList s1 == toList s2)
 
-#if __GLASGOW_HASKELL__ >= 806
-prop_strictFoldl' :: Set Int -> Property
-prop_strictFoldl' m = whnfHasNoThunks (foldl' (flip (:)) [] m)
-#endif
+prop_compare :: Set Int -> Set Int -> Property
+prop_compare s1 s2 = compare s1 s2 === compare (toList s1) (toList s2)
+
+prop_intersections :: (Set Int, [Set Int]) -> Property
+prop_intersections (s, ss) =
+  intersections ss' === List.foldl' intersection s ss
+  where
+    ss' = s :| ss -- Work around missing Arbitrary NonEmpty instance
+
+prop_intersections_lazy :: [Set Int] -> Property
+prop_intersections_lazy ss = intersections ss' === empty
+  where
+    ss' = NE.fromList $ ss ++ [empty] ++ error "too strict"
+                           --- ^ result will certainly be empty at this point,
+                           --    so the rest of the list should not be demanded.
+
+prop_insert :: Int -> Set Int -> Property
+prop_insert x s =
+  valid s' .&&.
+  toList s' === (if x `List.elem` xs then xs else List.insert x xs)
+  where
+    s' = insert x s
+    xs = toList s
+
+prop_delete :: Int -> Set Int -> Property
+prop_delete x s = valid s' .&&. toList s' === toList s List.\\ [x]
+  where
+    s' = delete x s
+
+prop_deleteMin :: Set Int -> Property
+prop_deleteMin s = toList (deleteMin s) === if null s then [] else tail (toList s)
+
+prop_deleteMax :: Set Int -> Property
+prop_deleteMax s = toList (deleteMax s) === if null s then [] else init (toList s)
+
+prop_findIndex :: Int -> Set Int -> Property
+prop_findIndex x s = x `member` s ==>
+  Just (findIndex x s) === List.findIndex (==x) (toList s)
+
+prop_lookupIndex :: Int -> Set Int -> Property
+prop_lookupIndex x s = lookupIndex x s === List.findIndex (==x) (toList s)
+
+prop_elemAt :: Int -> Set Int -> Property
+prop_elemAt i s = 0 <= i && i < size s ==>
+  elemAt i s === toList s !! i
+
+prop_deleteAt :: Int -> Set Int -> Property
+prop_deleteAt i s = 0 <= i && i < size s ==>
+  valid s' .&&.
+  toList s' === [x | (j, x) <- zip [0..] (toList s), i /= j]
+  where
+    s' = deleteAt i s
