@@ -6,7 +6,9 @@
 module Utils.Containers.Internal.State where
 
 import Control.Monad (ap, liftM2)
-import Control.Applicative (Applicative(..), liftA)
+import Control.Applicative (liftA)
+import Utils.Containers.Internal.Prelude
+import Prelude ()
 
 newtype State s a = State {runState :: s -> (s, a)}
 
@@ -14,9 +16,7 @@ instance Functor (State s) where
     fmap = liftA
 
 instance Monad (State s) where
-    {-# INLINE return #-}
     {-# INLINE (>>=) #-}
-    return = pure
     m >>= k = State $ \ s -> case runState m s of
         (s', x) -> runState (k x) s'
 
@@ -26,9 +26,7 @@ instance Applicative (State s) where
     (<*>) = ap
     m *> n = State $ \s -> case runState m s of
       (s', _) -> runState n s'
-#if MIN_VERSION_base(4,10,0)
     liftA2 = liftM2
-#endif
 
 execState :: State s a -> s -> a
 execState m x = snd (runState m x)
