@@ -14,41 +14,27 @@ void init_ghc_hs_iface(void);
 #define CLOSURE(module, symbol) \
     extern StgClosure ghczminternal_##module##_##symbol;
 
-#define PRIMCLOSURE(module, symbol) \
-    extern StgClosure ghczmprim_##module##_##symbol;
-
 #define UNDEF_CLOSURE(module, symbol)
 
 #define INFO_TBL(module, symbol) \
     extern const StgInfoTable ghczminternal_##module##_##symbol;
 
-#define PRIM_INFO_TBL(module, symbol) \
-    extern const StgInfoTable ghczmprim_##module##_##symbol;
-
 #include "RtsIfaceSymbols.h"
 
 #undef CLOSURE
-#undef PRIMCLOSURE
 #undef UNDEF_CLOSURE
 #undef INFO_TBL
-#undef PRIM_INFO_TBL
 
 // HsIface definition
 #define CLOSURE(module, symbol) \
     .symbol = &ghczminternal_##module##_##symbol,
 
-#define PRIMCLOSURE(module, symbol) \
-    .symbol = &ghczmprim_##module##_##symbol,
-
 #define UNDEF_CLOSURE(module, symbol) \
     .symbol = NULL,
 
 #define INFO_TBL(module, symbol) \
     .symbol = &ghczminternal_##module##_##symbol,
 
-#define PRIM_INFO_TBL(module, symbol) \
-    .symbol = &ghczmprim_##module##_##symbol,
-
 static HsIface the_ghc_hs_iface = {
 #include "RtsIfaceSymbols.h"
 };
