@@ -11,13 +11,13 @@ The overall structure of the GHC Prelude is a bit tricky.
 So the rough structure is as follows, in (linearised) dependency order
 
 
-GHC.Prim        Has no implementation.  It defines built-in things, and
-                by importing it you bring them into scope.
-                The source file is GHC.Prim.hi-boot, which is just
-                copied to make GHC.Prim.hi
+GHC.Internal.Prim        Has no implementation.  It defines built-in things, and
+                         by importing it you bring them into scope.
+                         The source file is GHC.Internal.Prim.hi-boot, which is just
+                         copied to make GHC.Internal.Prim.hi
 
 GHC.Internal.Base        Classes: Eq, Ord, Functor, Monad
-                Types:   List, (), Int, Bool, Ordering, Char, String
+                Types:   List, (), Int, Bool, Ordering, Char, String, NonEmpty
 
 GHC.Internal.Data.Tuple      Types: tuples, plus instances for GHC.Internal.Base classes
 
@@ -29,6 +29,13 @@ GHC.Internal.Data.Maybe      Type: Maybe, plus instances for GHC.Internal.Base c
 
 GHC.Internal.List        List functions
 
+GHC.Internal.Data.NonEmpty   Orphan instances for GHC.Internal.Base.NonEmpty of
+                             GHC.Internal.Base classes (other than Eq and Ord)
+                             plus function map
+
+GHC.Internal.Data.List.NonEmpty   Re-export GHC.Internal.Data.NonEmpty plus
+                                  functions zip and zipWith
+
 GHC.Internal.Num         Class: Num, plus instances for Int
                 Type:  Integer, plus instances for all classes so far (Eq, Ord, Num, Show)
 
@@ -99,20 +106,20 @@ Other Prelude modules are much easier with fewer complex dependencies.
 module GHC.Internal.Base
         (
         module GHC.Internal.Base,
-        module GHC.Classes,
-        module GHC.CString,
-        module GHC.Magic,
-        module GHC.Magic.Dict,
-        module GHC.Types,
-        module GHC.Prim,         -- Re-export GHC.Prim, GHC.Prim.Ext,
-        module GHC.Prim.Ext,     -- GHC.Prim.PtrEq and [boot] GHC.Internal.Err
-        module GHC.Prim.PtrEq,   -- to avoid lots of people having to
-        module GHC.Internal.Err, -- import these modules explicitly
+        module GHC.Internal.Classes,
+        module GHC.Internal.CString,
+        module GHC.Internal.Magic,
+        module GHC.Internal.Magic.Dict,
+        module GHC.Internal.Types,
+        module GHC.Internal.Prim,         -- Re-export GHC.Internal.Prim, GHC.Internal.Prim.Ext,
+        module GHC.Internal.Prim.Ext,     -- GHC.Internal.Prim.PtrEq and [boot] GHC.Internal.Err
+        module GHC.Internal.Prim.PtrEq,   -- to avoid lots of people having to
+        module GHC.Internal.Err,          -- import these modules explicitly
         module GHC.Internal.Maybe
   )
         where
 
-import GHC.Types hiding (
+import GHC.Internal.Types hiding (
   Unit#,
   Solo#,
   Tuple0#,
@@ -243,7 +250,7 @@ import GHC.Types hiding (
   Sum62#,
   Sum63#,
   )
-import GHC.Classes hiding (
+import GHC.Internal.Classes hiding (
   CUnit,
   CSolo,
   CTuple0,
@@ -312,21 +319,21 @@ import GHC.Classes hiding (
   CTuple63,
   CTuple64,
   )
-import GHC.CString
-import GHC.Magic
-import GHC.Magic.Dict
-import GHC.Prim hiding (dataToTagSmall#, dataToTagLarge#, whereFrom#)
+import GHC.Internal.CString
+import GHC.Internal.Magic
+import GHC.Internal.Magic.Dict
+import GHC.Internal.Prim hiding (dataToTagSmall#, dataToTagLarge#, whereFrom#)
   -- Hide dataToTag# ops because they are expected to break for
   -- GHC-internal reasons in the near future, and shouldn't
   -- be exposed from base (not even GHC.Exts)
 
-import GHC.Prim.Ext
-import GHC.Prim.PtrEq
+import GHC.Internal.Prim.Ext
+import GHC.Internal.Prim.PtrEq
 import GHC.Internal.Err
 import GHC.Internal.Maybe
 import {-# SOURCE #-} GHC.Internal.IO (mkUserError, mplusIO)
 
-import GHC.Tuple (Solo (MkSolo))
+import GHC.Internal.Tuple (Solo (MkSolo))
 
 -- See Note [Semigroup stimes cycle]
 import {-# SOURCE #-} GHC.Internal.Num (Num (..))
@@ -371,11 +378,11 @@ Such implicit dependencies can be introduced in at least the following ways:
 
 W1:
   Common awkward dependencies:
-   * TypeRep metadata introduces references to GHC.Types in EVERY module.
-   * A String literal introduces a reference to GHC.CString, for either
+   * TypeRep metadata introduces references to GHC.Internal.Types in EVERY module.
+   * A String literal introduces a reference to GHC.Internal.CString, for either
      unpackCString# or unpackCStringUtf8# depending on its contents.
-   * Tuple-notation introduces references to GHC.Tuple.
-   * Constraint tuples introduce references to GHC.Classes.
+   * Tuple-notation introduces references to GHC.Internal.Tuple.
+   * Constraint tuples introduce references to GHC.Internal.Classes.
    * Short lists like [3,8,2] produce references to GHC.Internal.Base.build
 
   A module can transitively depend on all of these by importing any of
@@ -387,7 +394,7 @@ W1:
    * Most modules in ghc-internal import GHC.Internal.Base.
    * Most modules in compiler/ import GHC.Prelude, which imports Prelude.
    * Most hs-boot files that would otherwise have no imports can get
-     away with just importing GHC.Types.
+     away with just importing GHC.Internal.Types.
 
   Unfortunately, the requirement to transitively import these modules
   when they are implicitly used is obscure and causes only /intermittent/
@@ -401,7 +408,8 @@ W2:
   missing record fields, and missing class instance methods all
   introduce references to GHC.Internal.Control.Exception.Base.
 
-  These constructs are therefore not allowed in ghc-prim or ghc-bignum.
+  These constructs are therefore not allowed in packages below ghc-internal.
+  This was the case when ghc-prim and ghc-bignum were separate of ghc-internal.
   But since they generally have bad code smell and are avoided by
   developers anyway, this restriction has not been very burdensome.
 
@@ -442,7 +450,7 @@ W4:
 
 W5:
   If no explicit "default" declaration is present, the assumed
-  "default (Integer, Double)" creates a dependency on GHC.Num.Integer
+  "default (Integer, Double)" creates a dependency on GHC.Internal.Bignum.Integer
   for the Integer type if defaulting is ever attempted during
   type-checking.  (This doesn't apply to hs-boot files, which can't
   be given "default" declarations anyway.)
@@ -753,10 +761,6 @@ needed to make foldr/build forms efficient are turned off, we'll get reasonably
 efficient translations anyway.
 -}
 
--- | @since base-4.9.0.0
-instance Semigroup (NonEmpty a) where
-        (a :| as) <> ~(b :| bs) = a :| (as ++ b : bs)
-
 -- | @since base-4.9.0.0
 instance Semigroup b => Semigroup (a -> b) where
         f <> g = \x -> f x <> g x
@@ -972,7 +976,7 @@ class Functor f where
     -- | 'fmap' is used to apply a function of type @(a -> b)@ to a value of type @f a@,
     -- where f is a functor, to produce a value of type @f b@.
     -- Note that for any type constructor with more than one parameter (e.g., 'Either'),
-    -- only the last type parameter can be modified with `fmap` (e.g., `b` in `Either a b`).
+    -- only the last type parameter can be modified with `fmap` (e.g., `b` in @Either a b@).
     --
     -- Some type constructors with two parameters or more have a @'Data.Bifunctor'@ instance that allows
     -- both the last and the penultimate parameters to be mapped over.
@@ -1707,25 +1711,6 @@ data NonEmpty a = a :| [a]
            , Ord -- ^ @since base-4.9.0.0
            )
 
--- | @since base-4.9.0.0
-instance Functor NonEmpty where
-  fmap f ~(a :| as) = f a :| fmap f as
-  b <$ ~(_ :| as)   = b   :| (b <$ as)
-
--- | @since base-4.9.0.0
-instance Applicative NonEmpty where
-  pure a = a :| []
-  (<*>) = ap
-  liftA2 = liftM2
-
--- | @since base-4.9.0.0
-instance Monad NonEmpty where
-  ~(a :| as) >>= f = b :| (bs ++ bs')
-    where b :| bs = f a
-          bs' = as >>= toList . f
-          toList ~(c :| cs) = c : cs
-
-
 ----------------------------------------------
 -- The list type
 
@@ -2107,6 +2092,9 @@ id x                    =  x
 
 -- Assertion function.  This simply ignores its boolean argument.
 -- The compiler may rewrite it to @('assertError' line)@.
+-- The Haddock below is attached to `assert`, since that is
+-- what occurs in source programs.
+-- See Note [Overview of assertions] in GHC.Tc.Gen.Head
 
 -- | If the first argument evaluates to 'True', then the result is the
 -- second argument.  Otherwise an 'Control.Exception.AssertionFailed' exception
@@ -2115,14 +2103,9 @@ id x                    =  x
 --
 -- Assertions can normally be turned on or off with a compiler flag
 -- (for GHC, assertions are normally on unless optimisation is turned on
--- with @-O@ or the @-fignore-asserts@
--- option is given).  When assertions are turned off, the first
--- argument to 'assert' is ignored, and the second argument is
--- returned as the result.
-
---      SLPJ: in 5.04 etc 'assert' is in GHC.Prim,
---      but from Template Haskell onwards it's simply
---      defined here in Base.hs
+-- with @-O@ or the @-fignore-asserts@ option is given). When assertions
+-- are turned off, the first argument to 'assert' is ignored, and the second
+-- argument is returned as the result.
 assert :: Bool -> a -> a
 assert _pred r = r
 
