@@ -24,9 +24,5 @@ module GHC.Internal.Data.Coerce
           -- @since base-4.7.0.0
           coerce, Coercible
         ) where
-import GHC.Prim (coerce)
-import GHC.Types (Coercible)
-
--- The import of GHC.Internal.Base is for build ordering; see Notes in GHC.Internal.Base for
--- more info.
-import GHC.Internal.Base ()
+import GHC.Internal.Prim (coerce)
+import GHC.Internal.Types (Coercible)
