@@ -64,6 +64,7 @@ module System.Console.Haskeline(
                     defaultPrefs,
                     runInputTWithPrefs,
                     runInputTBehaviorWithPrefs,
+                    withRunInBase,
                     -- ** History
                     -- $history
                     getHistory,
@@ -143,6 +144,10 @@ They return `Nothing` if they encounter the end of input.  More specifically:
 
 If @'autoAddHistory' == 'True'@ and the line input is nonblank (i.e., is not all
 spaces), it will be automatically added to the history.
+
+To include ANSI escape sequences in the input prompt, terminate them by @\STX@.
+See <https://github.com/haskell/haskeline/wiki/ControlSequencesInPrompt> for
+more information.
 -}
 getInputLine :: (MonadIO m, MonadMask m)
             => String -- ^ The input prompt
@@ -329,7 +334,7 @@ promptedInput doTerm doFile prompt = do
 of type 'Interrupt'.  For example:
 
 > tryAction :: InputT IO ()
-> tryAction = handle (\Interrupt -> outputStrLn "Cancelled.")
+> tryAction = handleInterrupt (outputStrLn "Cancelled.")
 >                $ withInterrupt $ someLongAction
 
 The action can handle the interrupt itself; a new 'Interrupt' exception will be thrown
@@ -337,7 +342,7 @@ every time Ctrl-C is pressed.
 
 > tryAction :: InputT IO ()
 > tryAction = withInterrupt loop
->     where loop = handle (\Interrupt -> outputStrLn "Cancelled; try again." >> loop)
+>     where loop = handleInterrupt (outputStrLn "Cancelled; try again." >> loop)
 >                    someLongAction
 
 This behavior differs from GHC's built-in Ctrl-C handling, which
@@ -350,9 +355,10 @@ withInterrupt act = do
     rterm <- InputT ask
     wrapInterrupt rterm act
 
--- | Catch and handle an exception of type 'Interrupt'.
+-- | Catch and handle an exception of type 'Interrupt'. See 'withInterrupt' for
+-- more explanation.
 --
--- > handleInterrupt f = handle $ \Interrupt -> f
+-- > handleInterrupt (outputStrLn "Ctrl+C was pressed, aborting!") someLongAction
 handleInterrupt :: MonadMask m => m a -> m a -> m a
 handleInterrupt f = handle $ \Interrupt -> f
 
