@@ -9,9 +9,11 @@
 #include "Rts.h"
 #include "RtsAPI.h"
 #include "RtsUtils.h"
+#include "RtsFlags.h"
 
 #include "Capability.h"
 #include "Printer.h"
+#include "AllocArray.h"
 
 StgWord heap_view_closureSize(StgClosure *closure) {
     ASSERT(LOOKS_LIKE_CLOSURE_PTR(closure));
@@ -173,7 +175,7 @@ StgWord collect_pointers(StgClosure *closure, StgClosure *ptrs[]) {
         case IND:
         case IND_STATIC:
         case BLACKHOLE:
-            ptrs[nptrs++] = (StgClosure *)(((StgInd *)closure)->indirectee);
+            ptrs[nptrs++] = (StgClosure *) ACQUIRE_LOAD(&((StgInd *)closure)->indirectee);
             break;
 
         case MUT_ARR_PTRS_CLEAN:
@@ -245,6 +247,16 @@ StgWord collect_pointers(StgClosure *closure, StgClosure *ptrs[]) {
             // See the note in AP_STACK about the stack chunk.
             break;
 
+        case BLOCKING_QUEUE:
+        {
+            StgBlockingQueue *bq = (StgBlockingQueue *) closure;
+            ptrs[nptrs++] = (StgClosure *) bq->link;
+            ptrs[nptrs++] = bq->bh;
+            ptrs[nptrs++] = (StgClosure *) bq->owner;
+            ptrs[nptrs++] = (StgClosure *) bq->queue;
+            break;
+        }
+
         default:
             fprintf(stderr,"closurePtrs: Cannot handle type %s yet\n",
                            closure_type_names[info->type]);
@@ -267,18 +279,14 @@ StgMutArrPtrs *heap_view_closurePtrs(Capability *cap, StgClosure *closure) {
     StgClosure **ptrs = (StgClosure **) stgMallocBytes(sizeof(StgClosure *) * size, "heap_view_closurePtrs");
     StgWord nptrs = collect_pointers(closure, ptrs);
 
-    size = nptrs + mutArrPtrsCardTableSize(nptrs);
-    StgMutArrPtrs *arr =
-        (StgMutArrPtrs *)allocate(cap, sizeofW(StgMutArrPtrs) + size);
-    TICK_ALLOC_PRIM(sizeofW(StgMutArrPtrs), nptrs, 0);
-    SET_HDR(arr, &stg_MUT_ARR_PTRS_FROZEN_CLEAN_info, cap->r.rCCCS);
-    arr->ptrs = nptrs;
-    arr->size = size;
+    StgMutArrPtrs *arr = allocateMutArrPtrs(cap, nptrs, cap->r.rCCCS);
+    if (RTS_UNLIKELY(arr == NULL)) goto end;
+    SET_INFO((StgClosure *) arr, &stg_MUT_ARR_PTRS_FROZEN_CLEAN_info);
 
     for (StgWord i = 0; i<nptrs; i++) {
         arr->payload[i] = ptrs[i];
     }
+end:
     stgFree(ptrs);
-
     return arr;
 }
