@@ -9,7 +9,7 @@
 
 #if defined(PROFILING)
 
-#include "PosixSource.h"
+#include "rts/PosixSource.h"
 #include "Rts.h"
 
 #include "Profiling.h"
@@ -70,7 +70,7 @@ processHeapClosureForDead( const StgClosure *c )
     if (IS_FORWARDING_PTR(info)) {
         // The size of the evacuated closure is currently stored in
         // the LDV field.  See SET_EVACUAEE_FOR_LDV() in
-        // includes/StgLdvProf.h.
+        // rts/include/StgLdvProf.h.
         return LDVW(c);
     }
     info = INFO_PTR_TO_STRUCT(info);
@@ -119,6 +119,7 @@ processHeapClosureForDead( const StgClosure *c )
     case FUN_2_0:
     case FUN_1_1:
     case FUN_0_2:
+    case CONTINUATION:
     case BLACKHOLE:
     case BLOCKING_QUEUE:
         /*
@@ -153,6 +154,7 @@ processHeapClosureForDead( const StgClosure *c )
     case CATCH_STM_FRAME:
     case CATCH_RETRY_FRAME:
     case ATOMICALLY_FRAME:
+    case ANN_FRAME:
         // others
     case INVALID_OBJECT:
     case COMPACT_NFDATA:
