@@ -105,12 +105,12 @@
  */
 
 #if defined(freebsd_HOST_OS)
-extern void iconvctl();
-extern void iconv_open_into();
-extern void iconv_open();
-extern void iconv_close();
-extern void iconv_canonicalize();
-extern void iconv();
+extern void iconvctl(void);
+extern void iconv_open_into(void);
+extern void iconv_open(void);
+extern void iconv_close(void);
+extern void iconv_canonicalize(void);
+extern void iconv(void);
 #endif
 
 /*
@@ -974,10 +974,20 @@ SymbolAddr* lookupSymbol( SymbolName* lbl )
     // lookupDependentSymbol directly.
     SymbolAddr* r = lookupDependentSymbol(lbl, NULL, NULL);
     if (!r) {
-        errorBelch("^^ Could not load '%s', dependency unresolved. "
-                   "See top entry above.\n", lbl);
-        IF_DEBUG(linker, printLoadedObjects());
-        fflush(stderr);
+        if (!RtsFlags.MiscFlags.linkerOptimistic) {
+          errorBelch("^^ Could not load '%s', dependency unresolved. "
+                     "See top entry above. You might consider using --optimistic-linking\n",
+                     lbl);
+          IF_DEBUG(linker, printLoadedObjects());
+          fflush(stderr);
+        } else {
+          // if --optimistic-linking is passed into the RTS we allow the linker
+          // to optimistically continue
+          errorBelch("^^ Could not load '%s', dependency unresolved, "
+                     "optimistically continuing\n",
+                     lbl);
+          r = (void*) 0xDEADBEEF;
+        }
     }
 
     if (!runPendingInitializers()) {
