@@ -10,13 +10,13 @@
 
 #include <stdio.h>
 
-#include "BeginPrivate.h"
 #include "Rts.h"
-
 #if defined(DEBUG)
 #include "Arena.h"
 #endif
 
+#include "BeginPrivate.h"
+
 #if defined(PROFILING)
 #define PROFILING_ONLY(s) s
 #else
@@ -55,6 +55,10 @@ extern Arena *prof_arena;
 void debugCCS( CostCentreStack *ccs );
 #endif
 
+#if defined(THREADED_RTS)
+extern Mutex ccs_mutex;
+#endif
+
 #endif
 
 #include "EndPrivate.h"
