@@ -177,6 +177,7 @@ isRetainer( const StgClosure *c )
     case FUN_0_2:
         // partial applications
     case PAP:
+    case CONTINUATION:
         // indirection
     // IND_STATIC used to be an error, but at the moment it can happen
     // as isAlive doesn't look through IND_STATIC as it ignores static
@@ -216,6 +217,7 @@ isRetainer( const StgClosure *c )
     case RET_SMALL:
     case RET_BIG:
     case RET_FUN:
+    case ANN_FRAME:
         // other cases
     case IND:
     case INVALID_OBJECT:
@@ -390,11 +392,11 @@ computeRetainerSet( traverseState *ts )
     //
     // The following code assumes that WEAK objects are considered to be roots
     // for retainer profiling.
-    for (n = 0; n < n_capabilities; n++) {
+    for (n = 0; n < getNumCapabilities(); n++) {
         // NB: after a GC, all nursery weak_ptr_lists have been migrated
         // to the global lists living in the generations
-        ASSERT(capabilities[n]->weak_ptr_list_hd == NULL);
-        ASSERT(capabilities[n]->weak_ptr_list_tl == NULL);
+        ASSERT(getCapability(n)->weak_ptr_list_hd == NULL);
+        ASSERT(getCapability(n)->weak_ptr_list_tl == NULL);
     }
     for (g = 0; g < RtsFlags.GcFlags.generations; g++) {
         for (weak = generations[g].weak_ptr_list; weak != NULL; weak = weak->link) {
@@ -413,7 +415,7 @@ computeRetainerSet( traverseState *ts )
 
 /* -----------------------------------------------------------------------------
  * Perform retainer profiling.
- * N is the oldest generation being profilied, where the generations are
+ * N is the oldest generation being profiled, where the generations are
  * numbered starting at 0.
  * Invariants:
  * Note:
