@@ -9,6 +9,7 @@
 #include "rts/PosixSource.h"
 #include "Rts.h"
 #include "RtsUtils.h"
+#include "ghcversion.h"
 
 #include "eventlog/EventLog.h"
 
@@ -177,7 +178,7 @@ rtsFatalInternalErrorFn(const char *s, va_list ap)
      libdwFree(session);
 #endif
      fprintf(stderr, "\n");
-     fprintf(stderr, "    (GHC version %s for %s)\n", ProjectVersion, xstr(HostPlatform_TYPE));
+     fprintf(stderr, "    (GHC version %s for %s)\n", __GLASGOW_HASKELL_FULL_VERSION__, xstr(HostPlatform_TYPE));
      fprintf(stderr, "    Please report this as a GHC bug:  https://www.haskell.org/ghc/reportabug\n");
      fflush(stderr);
   }
@@ -186,7 +187,12 @@ rtsFatalInternalErrorFn(const char *s, va_list ap)
 #endif
 
 #if defined(TRACING)
-  if (RtsFlags.TraceFlags.tracing == TRACE_EVENTLOG) endEventLogging();
+  if (RtsFlags.TraceFlags.tracing == TRACE_EVENTLOG) {
+    // Use flushAllCapsEventsBufs rather than endEventLogging here since
+    // the latter insists on acquiring all capabilities to flush the eventlog;
+    // this would deadlock if we barfed during a GC.
+    flushAllCapsEventsBufs();
+  }
 #endif
 
   abort();
@@ -321,27 +327,18 @@ rtsDebugMsgFn(const char *s, va_list ap)
 }
 
 
-// Used in stg_badAlignment_entry defined in StgStartup.cmm.
-void rtsBadAlignmentBarf(void) STG_NORETURN;
-
 void
 rtsBadAlignmentBarf(void)
 {
     barf("Encountered incorrectly aligned pointer. This can't be good.");
 }
 
-// Used by code generator
-void rtsOutOfBoundsAccess(void) STG_NORETURN;
-
 void
 rtsOutOfBoundsAccess(void)
 {
     barf("Encountered out of bounds array access.");
 }
 
-// Used by code generator
-void rtsMemcpyRangeOverlap(void) STG_NORETURN;
-
 void
 rtsMemcpyRangeOverlap(void)
 {
