@@ -52,11 +52,7 @@ Mutex all_tasks_mutex;
 // A thread-local-storage key that we can use to get access to the
 // current thread's Task structure.
 #if defined(THREADED_RTS)
-# if CC_SUPPORTS_TLS
 __thread Task *my_task;
-# else
-ThreadLocalKey currentTaskKey;
-# endif
 #else
 Task *my_task;
 #endif
@@ -75,9 +71,6 @@ initTaskManager (void)
         peakWorkerCount = 0;
         tasksInitialized = 1;
 #if defined(THREADED_RTS)
-#if !CC_SUPPORTS_TLS
-        newThreadLocalKey(&currentTaskKey);
-#endif
         initMutex(&all_tasks_mutex);
 #endif
     }
@@ -109,9 +102,6 @@ freeTaskManager (void)
 
 #if defined(THREADED_RTS)
     closeMutex(&all_tasks_mutex);
-#if !CC_SUPPORTS_TLS
-    freeThreadLocalKey(&currentTaskKey);
-#endif
 #endif
 
     tasksInitialized = 0;
