@@ -9,10 +9,10 @@
 
 #if defined(PROFILING)
 
-#include <string.h>
 #include "rts/PosixSource.h"
 #include "Rts.h"
 #include "sm/Storage.h"
+#include <string.h>
 
 #include "TraverseHeap.h"
 
@@ -48,7 +48,7 @@ static void debug(const char *s, ...)
     va_end(ap);
 }
 #else
-#define debug(...)
+static void debug(const char *s STG_UNUSED, ...) {}
 #endif
 
 // number of blocks allocated for one stack
@@ -529,6 +529,7 @@ traverseGetChildren(StgClosure *c, StgClosure **first_child, bool *other_childre
     case RET_BCO:
     case RET_SMALL:
     case RET_BIG:
+    case ANN_FRAME:
         // invalid objects
     case IND:
     case INVALID_OBJECT:
@@ -832,6 +833,7 @@ traversePop(traverseState *ts, StgClosure **c, StgClosure **cp, stackData *data,
         case RET_BCO:
         case RET_SMALL:
         case RET_BIG:
+        case ANN_FRAME:
             // invalid objects
         case IND:
         case INVALID_OBJECT:
@@ -965,6 +967,7 @@ traversePushStack(traverseState *ts, StgClosure *cp, stackElement *sep,
         case CATCH_RETRY_FRAME:
         case ATOMICALLY_FRAME:
         case RET_SMALL:
+        case ANN_FRAME:
             bitmap = BITMAP_BITS(info->i.layout.bitmap);
             size   = BITMAP_SIZE(info->i.layout.bitmap);
             p++;
@@ -1239,12 +1242,15 @@ inner_loop:
         traversePushClosure(ts, (StgClosure *) tso->blocked_exceptions, c, sep, child_data);
         traversePushClosure(ts, (StgClosure *) tso->bq, c, sep, child_data);
         traversePushClosure(ts, (StgClosure *) tso->trec, c, sep, child_data);
-        if (   tso->why_blocked == BlockedOnMVar
-               || tso->why_blocked == BlockedOnMVarRead
-               || tso->why_blocked == BlockedOnBlackHole
-               || tso->why_blocked == BlockedOnMsgThrowTo
-            ) {
+        switch (ACQUIRE_LOAD(&tso->why_blocked)) {
+        case BlockedOnMVar:
+        case BlockedOnMVarRead:
+        case BlockedOnBlackHole:
+        case BlockedOnMsgThrowTo:
             traversePushClosure(ts, tso->block_info.closure, c, sep, child_data);
+            break;
+        default:
+            break;
         }
         goto loop;
     }
