@@ -60,38 +60,37 @@ void initAdjustors(void) {
 
 void*
 createAdjustor(StgStablePtr hptr, StgFunPtr wptr,
-               char *typeString STG_UNUSED
-    )
+               char *typeString STG_UNUSED)
 {
-            // The adjustor puts the following things on the stack:
-            // 1.) %ebp link
-            // 2.) padding and (a copy of) the arguments
-            // 3.) a dummy argument
-            // 4.) hptr
-            // 5.) return address (for returning to the adjustor)
-            // All these have to add up to a multiple of 16.
+    // The adjustor puts the following things on the stack:
+    // 1. %ebp link
+    // 2. padding and (a copy of) the arguments
+    // 3. a dummy argument
+    // 4. hptr
+    // 5. return address (for returning to the adjustor)
+    // All these have to add up to a multiple of 16.
 
-        int sz = totalArgumentSize(typeString);
-            // first, include everything in frame_size
-        StgInt frame_size = sz * 4 + 16;
-            // align to 16 bytes
-        frame_size = (frame_size + 15) & ~15;
-            // only count 2.) and 3.) as part of frame_size
-        frame_size -= 12;
+    int sz = totalArgumentSize(typeString);
+        // first, include everything in frame_size
+    StgInt frame_size = sz * 4 + 16;
+        // align to 16 bytes
+    frame_size = (frame_size + 15) & ~15;
+        // only count 2.) and 3.) as part of frame_size
+    frame_size -= 12;
 
-        struct CCallContext context = {
-            .hptr = hptr,
-            .wptr = wptr,
-            .frame_size = frame_size,
-            .argument_size = sz,
-        };
-        return alloc_adjustor(ccall_pool, &context);
+    struct CCallContext context = {
+        .hptr = hptr,
+        .wptr = wptr,
+        .frame_size = frame_size,
+        .argument_size = sz,
+    };
+    return alloc_adjustor(ccall_pool, &context);
 }
 
 void
 freeHaskellFunctionPtr(void* ptr)
 {
-    struct AdjustorContext context;
+    struct CCallContext context;
     free_adjustor(ptr, &context);
     freeStablePtr(context.hptr);
 }
