@@ -16,13 +16,12 @@ SymbolExtra* makeArmSymbolExtra( ObjectCode const* oc,
                                  unsigned long target,
                                  bool fromThumb,
                                  bool toThumb );
-#else
+#elif defined(powerpc_HOST_ARCH) || defined(x86_64_HOST_ARCH) || defined(riscv64_HOST_ARCH)
 SymbolExtra* makeSymbolExtra( ObjectCode const* oc,
                               unsigned long symbolNumber,
                               unsigned long target );
 
-#endif /* arm_HOST_ARCH */
-
+#endif /* powerpc_HOST_ARCH || x86_64_HOST_ARCH */
 #endif /* NEED_SYMBOL_EXTRAS */
 
 #include "EndPrivate.h"
