@@ -7,18 +7,19 @@
  *
  * --------------------------------------------------------------------------*/
 
-#include "PosixSource.h"
+#include "rts/PosixSource.h"
 
-/* We've defined _POSIX_SOURCE via "PosixSource.h", and yet still use
+/* We've defined _POSIX_SOURCE via "rts/PosixSource.h", and yet still use
    some non-POSIX features.  With _POSIX_SOURCE defined, visibility of
-   non-POSIX extension prototypes requires _DARWIN_C_SOURCE on Mac OS X and
-   __BSD_VISIBLE on FreeBSD and DragonflyBSD.  Otherwise, for example, code
-   using pthread_setname_np(3) and variants will not compile.  We must
-   therefore define the additional macros that expose non-POSIX APIs early,
-   before any of the relevant system headers are included via "Rts.h".
+   non-POSIX extension prototypes requires _DARWIN_C_SOURCE on Mac OS X,
+   __BSD_VISIBLE on FreeBSD and DragonflyBSD, and _NETBSD_SOURCE on
+   NetBSD.  Otherwise, for example, code using pthread_setname_np(3) and
+   variants will not compile.  We must therefore define the additional
+   macros that expose non-POSIX APIs early, before any of the relevant
+   system headers are included via "Rts.h".
 
    An alternative approach could be to write portable wrappers or stubs for all
-   the non-posix functions in a C-module that does not include "PosixSource.h",
+   the non-posix functions in a C-module that does not include "rts/PosixSource.h",
    and then use only POSIX features and the portable wrapper functions in all
    other C-modules. */
 #include "ghcconfig.h"
@@ -28,6 +29,9 @@
 #if defined(darwin_HOST_OS)
 #define _DARWIN_C_SOURCE 1
 #endif
+#if defined(netbsd_HOST_OS)
+#define _NETBSD_SOURCE 1
+#endif
 
 #include "Rts.h"
 
@@ -51,7 +55,7 @@
 #include <string.h>
 #endif
 
-#if defined(darwin_HOST_OS) || defined(freebsd_HOST_OS)
+#if defined(darwin_HOST_OS) || defined(freebsd_HOST_OS) || defined(netbsd_HOST_OS)
 #include <sys/types.h>
 #include <sys/sysctl.h>
 #endif
@@ -182,20 +186,69 @@ shutdownThread(void)
   pthread_exit(NULL);
 }
 
+struct ThreadDesc {
+    OSThreadProc *startProc;
+    void *param;
+    char *name;
+};
+
+// N.B. Darwin's pthread_setname_np only allows the name of the
+// calling thread to be set. Consequently we must use this
+// trampoline.
+static void *
+start_thread (void *param)
+{
+    struct ThreadDesc *desc = (struct ThreadDesc *) param;
+    OSThreadProc *startProc = desc->startProc;
+    void *startParam = desc->param;
+
+#if defined(HAVE_PTHREAD_SET_NAME_NP)
+    pthread_set_name_np(pthread_self(), desc->name);
+#elif defined(HAVE_PTHREAD_SETNAME_NP)
+    pthread_setname_np(pthread_self(), desc->name);
+#elif defined(HAVE_PTHREAD_SETNAME_NP_DARWIN)
+    pthread_setname_np(desc->name);
+#elif defined(HAVE_PTHREAD_SETNAME_NP_NETBSD)
+    pthread_setname_np(pthread_self(), "%s", desc->name);
+#endif
+
+    stgFree(desc->name);
+    stgFree(desc);
+
+    return startProc(startParam);
+}
+
+/* Note: at least on Linux/Glibc, `pthread_setname_np` restricts the name of
+ * a thread to 16 bytes, including the terminating null byte. Hence, make sure
+ * to only pass in names of up to 15 characters. Otherwise,
+ * `pthread_setname_np` when called in `start_thread` will fail with `ERANGE`,
+ * which is not checked for, and the thread won't be named at all.
+ */
 int
-createOSThread (OSThreadId* pId, char *name STG_UNUSED,
+createOSThread (OSThreadId* pId, const char *name,
                 OSThreadProc *startProc, void *param)
 {
-  int result = pthread_create(pId, NULL, startProc, param);
+  int result = createAttachedOSThread(pId, name, startProc, param);
   if (!result) {
     pthread_detach(*pId);
-#if defined(HAVE_PTHREAD_SET_NAME_NP)
-    pthread_set_name_np(*pId, name);
-#elif defined(HAVE_PTHREAD_SETNAME_NP)
-    pthread_setname_np(*pId, name);
-#elif defined(HAVE_PTHREAD_SETNAME_NP_DARWIN)
-    pthread_setname_np(name);
-#endif
+  }
+  return result;
+}
+
+int
+createAttachedOSThread (OSThreadId *pId, const char *name,
+                        OSThreadProc *startProc, void *param)
+{
+  struct ThreadDesc *desc = stgMallocBytes(sizeof(struct ThreadDesc), "createAttachedOSThread");
+  desc->startProc = startProc;
+  desc->param = param;
+  desc->name = stgMallocBytes(strlen(name) + 1, "createAttachedOSThread");
+  strcpy(desc->name, name);
+
+  int result = pthread_create(pId, NULL, start_thread, desc);
+  if (result) {
+      stgFree(desc->name);
+      stgFree(desc);
   }
   return result;
 }
@@ -217,7 +270,7 @@ osThreadIsAlive(OSThreadId id STG_UNUSED)
 void
 initMutex(Mutex* pMut)
 {
-#if defined(DEBUG)
+#if defined(ASSERTS_ENABLED)
     pthread_mutexattr_t attr;
     pthread_mutexattr_init(&attr);
     pthread_mutexattr_settype(&attr,PTHREAD_MUTEX_ERRORCHECK);
@@ -233,43 +286,6 @@ closeMutex(Mutex* pMut)
     pthread_mutex_destroy(pMut);
 }
 
-void
-newThreadLocalKey (ThreadLocalKey *key)
-{
-    int r;
-    if ((r = pthread_key_create(key, NULL)) != 0) {
-        barf("newThreadLocalKey: %s", strerror(r));
-    }
-}
-
-void *
-getThreadLocalVar (ThreadLocalKey *key)
-{
-    return pthread_getspecific(*key);
-    // Note: a return value of NULL can indicate that either the key
-    // is not valid, or the key is valid and the data value has not
-    // yet been set.  We need to use the latter case, so we cannot
-    // detect errors here.
-}
-
-void
-setThreadLocalVar (ThreadLocalKey *key, void *value)
-{
-    int r;
-    if ((r = pthread_setspecific(*key,value)) != 0) {
-        barf("setThreadLocalVar: %s", strerror(r));
-    }
-}
-
-void
-freeThreadLocalKey (ThreadLocalKey *key)
-{
-    int r;
-    if ((r = pthread_key_delete(*key)) != 0) {
-        barf("freeThreadLocalKey: %s", strerror(r));
-    }
-}
-
 #if defined(THREADED_RTS)
 
 static void *
@@ -335,6 +351,15 @@ getNumberOfProcessors (void)
             if(sysctlbyname("hw.ncpu",&nproc,&size,NULL,0) != 0)
                 nproc = 1;
         }
+#elif defined(netbsd_HOST_OS)
+        int mib[2] = { CTL_HW, HW_NCPUONLINE };
+        size_t size = sizeof(nproc);
+        if (sysctl(mib, 2, &nproc, &size, NULL, 0) != 0) {
+            mib[1] = HW_NCPU;
+            if (sysctl(mib, 2, &nproc, &size, NULL, 0) != 0) {
+                nproc = 1;
+            }
+        }
 #elif defined(HAVE_SYSCONF) && defined(_SC_NPROCESSORS_ONLN)
         // N.B. This is the number of physical processors.
         nproc = sysconf(_SC_NPROCESSORS_ONLN);
@@ -389,7 +414,7 @@ setThreadAffinity (uint32_t n, uint32_t m)
 #elif defined(darwin_HOST_OS) && defined(THREAD_AFFINITY_POLICY)
 // Schedules the current thread in the affinity set identified by tag n.
 void
-setThreadAffinity (uint32_t n, uint32_t m GNUC3_ATTRIBUTE(__unused__))
+setThreadAffinity (uint32_t n, uint32_t m STG_UNUSED)
 {
     thread_affinity_policy_data_t policy;
 
