@@ -19,7 +19,7 @@
 #include "RtsUtils.h"
 #include "Capability.h"
 #include "Select.h"
-#include "IOManager.h"
+#include "IOManagerInternals.h"
 #include "Stats.h"
 #include "GetTime.h"
 
@@ -36,7 +36,7 @@
 
 #include "Clock.h"
 
-#if !defined(THREADED_RTS)
+#if defined(IOMGR_ENABLED_SELECT)
 
 // The target time for a threadDelay is stored in a one-word quantity
 // in the TSO (tso->block_info.target).  On a 32-bit machine we
@@ -217,7 +217,7 @@ static enum FdState fdPollWriteState (int fd)
  *
  */
 void
-awaitEvent(Capability *cap, bool wait)
+awaitCompletedTimeoutsOrIOSelect(Capability *cap, bool wait)
 {
     CapIOManager *iomgr = cap->iomgr;
     StgTSO *tso, *prev, *next;
@@ -417,7 +417,7 @@ awaitEvent(Capability *cap, bool wait)
                   }
                   break;
               default:
-                  barf("awaitEvent");
+                  barf("awaitCompletedTimeoutsOrIOSelect");
               }
 
               switch (fd_state) {
@@ -463,4 +463,4 @@ awaitEvent(Capability *cap, bool wait)
                   && emptyRunQueue(cap));
 }
 
-#endif /* THREADED_RTS */
+#endif /* IOMGR_ENABLED_SELECT */
