@@ -274,19 +274,9 @@ STATIC_INLINE uint32_t
 log_2(W_ n)
 {
     ASSERT(n > 0 && n < (1<<NUM_FREE_LISTS));
-#if defined(__GNUC__)
     return CLZW(n) ^ (sizeof(StgWord)*8 - 1);
     // generates good code on x86.  __builtin_clz() compiles to bsr+xor, but
     // we want just bsr, so the xor here cancels out gcc's xor.
-#else
-    W_ i, x;
-    x = n;
-    for (i=0; i < NUM_FREE_LISTS; i++) {
-        x = x >> 1;
-        if (x == 0) return i;
-    }
-    return NUM_FREE_LISTS;
-#endif
 }
 
 // log base 2 (ceiling), needs to support up to (2^NUM_FREE_LISTS)-1
@@ -294,18 +284,8 @@ STATIC_INLINE uint32_t
 log_2_ceil(W_ n)
 {
     ASSERT(n > 0 && n < (1<<NUM_FREE_LISTS));
-#if defined(__GNUC__)
     uint32_t r = log_2(n);
     return (n & (n-1)) ? r+1 : r;
-#else
-    W_ i, x;
-    x = 1;
-    for (i=0; i < MAX_FREE_LIST; i++) {
-        if (x >= n) return i;
-        x = x << 1;
-    }
-    return MAX_FREE_LIST;
-#endif
 }
 
 STATIC_INLINE void
@@ -394,6 +374,53 @@ split_block_low (bdescr *bd, W_ n)
 }
 
 
+// A variant of `split_block_high` where we keep both blocks.
+// The returned block has size `n`, which is split off `bd`.
+static bdescr *
+split_block_high_no_free (bdescr *bd, W_ n)
+{
+    ASSERT(bd->blocks > n);
+
+    bdescr* ret = bd + bd->blocks - n; // take n blocks off the end
+    ret->blocks = n;
+    ret->start = ret->free = bd->start + (bd->blocks - n)*BLOCK_SIZE_W;
+    ret->link = NULL;
+
+    bd->blocks -= n;
+
+    setup_tail(ret);
+    setup_tail(bd);
+
+    return ret;
+}
+
+// Allocate a MBlock worth of `n` block sized chunks aligned at `n`-block boundry.
+// This returns a linked list of `bdescr` of length `BLOCKS_PER_MBLOCK / n`.
+// We assume relevant locks are held.
+bdescr *
+allocMBlockAlignedGroupOnNode(uint32_t node, W_ n)
+{
+    bdescr *bd = allocGroupOnNode(node, BLOCKS_PER_MBLOCK);
+
+    // Free unaligned blocks, as we can't use these.
+    ASSERT(bd->blocks == BLOCKS_PER_MBLOCK);
+    bd = split_block_high(bd, bd->blocks - bd->blocks % n);
+    ASSERT(bd->blocks % n == 0);
+
+    bdescr *last = NULL;
+    bdescr *chunk = NULL;
+    // Chain the aligned groups together into a linked-list
+    while (bd->blocks > n) {
+      chunk = split_block_high_no_free(bd, n);
+      chunk->link = last;
+      last = chunk;
+    }
+    bd->link = chunk;
+
+    return bd;
+}
+
+
 /* Find a fitting block for the allocation request in the given free list.
    Returns:
      - not NULL: when an exact match was found in the free list.
@@ -1138,14 +1165,14 @@ static void sortDeferredList(bdescr** head) {
     }
 }
 
-void deferMBlockFreeing() {
+void deferMBlockFreeing(void) {
     if(defer_mblock_frees) {
         barf("MBlock freeing is already deferred");
     }
     defer_mblock_frees = true;
 }
 
-void commitMBlockFreeing() {
+void commitMBlockFreeing(void) {
     if(! defer_mblock_frees) {
         barf("MBlock freeing was never deferred");
     }
@@ -1405,3 +1432,17 @@ reportUnmarkedBlocks (void)
 }
 
 #endif
+
+void clear_free_list(void) {
+    for (uint32_t node = 0; node < n_numa_nodes; ++node) {
+        for (bdescr *bd = free_mblock_list[node]; bd != NULL; bd = bd->link) {
+            clear_blocks(bd);
+        }
+
+        for (int ln = 0; ln < NUM_FREE_LISTS; ++ln) {
+            for (bdescr *bd = free_list[node][ln]; bd != NULL; bd = bd->link) {
+                clear_blocks(bd);
+            }
+        }
+    }
+}
