@@ -16,6 +16,7 @@
 #include <stdio.h>
 #include "Schedule.h"
 #include "Capability.h"
+#include "IOManagerInternals.h"
 #include "win32/AsyncMIO.h"
 #include "win32/MIOManager.h"
 
@@ -150,7 +151,7 @@ addDoProcRequest(void* proc, void* param)
 
 
 int
-startupAsyncIO()
+startupAsyncIO(void)
 {
     if (!StartIOManager()) {
         return 0;
@@ -294,7 +295,7 @@ start:
             for(tso = iomgr->blocked_queue_hd; tso != END_TSO_QUEUE;
                   tso = tso->_link) {
 
-                switch(tso->why_blocked) {
+                switch(ACQUIRE_LOAD(&tso->why_blocked)) {
                 case BlockedOnRead:
                 case BlockedOnWrite:
                 case BlockedOnDoProc:
