@@ -3,12 +3,16 @@
 {-# LANGUAGE GADTs #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 
-import Data.List
+import Data.List (sortBy, intercalate, isPrefixOf)
 import Data.Data
+import Control.Monad.IO.Class
 import GHC.Types.SrcLoc
 import GHC hiding (moduleName)
 import GHC.Hs.Dump
+import GHC.Driver.Env
 import GHC.Driver.Session
+import GHC.Driver.Ppr
+import GHC.Driver.Make
 import GHC.Utils.Outputable hiding (space)
 import System.Environment( getArgs )
 import System.Exit
@@ -42,7 +46,7 @@ testOneFile libdir fileName = do
        let
          origAst = showPprUnsafe
                      $ showAstData BlankSrcSpan BlankEpAnnotations
-                     $ eraseLayoutInfo (pm_parsed_source p)
+                     $ eraseEpLayout (pm_parsed_source p)
          pped    = pragmas ++ "\n" ++ pp (pm_parsed_source p)
          pragmas = getPragmas (pm_parsed_source p)
 
@@ -58,7 +62,7 @@ testOneFile libdir fileName = do
        let newAstStr :: String
            newAstStr = showPprUnsafe
                          $ showAstData BlankSrcSpan BlankEpAnnotations
-                         $ eraseLayoutInfo (pm_parsed_source p')
+                         $ eraseEpLayout (pm_parsed_source p')
        writeBinFile newAstFile newAstStr
 
        if origAst == newAstStr
@@ -76,34 +80,24 @@ testOneFile libdir fileName = do
 
 parseOneFile :: FilePath -> FilePath -> IO ParsedModule
 parseOneFile libdir fileName = do
-       let modByFile m =
-             case ml_hs_file $ ms_location m of
-               Nothing -> False
-               Just fn -> fn == fileName
        runGhc (Just libdir) $ do
          dflags <- getSessionDynFlags
          let dflags2 = dflags `gopt_set` Opt_KeepRawTokenStream
          _ <- setSessionDynFlags dflags2
-         addTarget Target { targetId = TargetFile fileName Nothing
-                          , targetAllowObjCode = True
-                          , targetContents = Nothing }
-         _ <- load LoadAllTargets
-         graph <- getModuleGraph
-         let
-           modSum = case filter modByFile (mgModSummaries graph) of
-                     [x] -> x
-                     xs -> error $ "Can't find module, got:"
-                              ++ show (map (ml_hs_file . ms_location) xs)
-         parseModule modSum
+         hsc_env <- getSession
+         mms <- liftIO $ summariseFile hsc_env (hsc_home_unit hsc_env) mempty fileName Nothing Nothing
+         case mms of
+           Left _err -> error "parseOneFile"
+           Right ms -> parseModule ms
 
-getPragmas :: Located HsModule -> String
-getPragmas (L _ (HsModule { hsmodAnn = anns'})) = pragmaStr
+getPragmas :: Located (HsModule GhcPs) -> String
+getPragmas (L _ (HsModule { hsmodExt = XModulePs { hsmodAnn = anns' } })) = pragmaStr
   where
     tokComment (L _ (EpaComment (EpaBlockComment s) _)) = s
     tokComment (L _ (EpaComment (EpaLineComment  s) _)) = s
     tokComment _ = ""
 
-    cmp (L l1 _) (L l2 _) = compare (anchor l1) (anchor l2)
+    cmp (L l1 _) (L l2 _) = compare (epaLocationRealSrcSpan l1) (epaLocationRealSrcSpan l2)
     comments' = map tokComment $ sortBy cmp $ priorComments $ epAnnComments anns'
     pragmas = filter (\c -> isPrefixOf "{-#" c ) comments'
     pragmaStr = intercalate "\n" pragmas
@@ -111,14 +105,14 @@ getPragmas (L _ (HsModule { hsmodAnn = anns'})) = pragmaStr
 pp :: (Outputable a) => a -> String
 pp a = showPprUnsafe a
 
-eraseLayoutInfo :: ParsedSource -> ParsedSource
-eraseLayoutInfo = everywhere go
+eraseEpLayout :: ParsedSource -> ParsedSource
+eraseEpLayout = everywhere go
   where
     go :: forall a. Typeable a => a -> a
     go x =
-      case eqT @a @LayoutInfo of
+      case eqT @a @EpLayout of
         Nothing -> x
-        Just Refl -> NoLayoutInfo
+        Just Refl -> EpNoLayout
 
 -- ---------------------------------------------------------------------
 
