@@ -10,13 +10,6 @@
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 
--- We never want to link against terminfo while bootstrapping.
-#if defined(BOOTSTRAPPING)
-#if defined(WITH_TERMINFO)
-#undef WITH_TERMINFO
-#endif
-#endif
-
 -- Fine if this comes from make/Hadrian or the pre-built base.
 #include <ghcplatform.h>
 
@@ -30,11 +23,12 @@
 
 module Main (main) where
 
+import Debug.Trace
 import qualified GHC.Unit.Database as GhcPkg
 import GHC.Unit.Database hiding (mkMungePathUrl)
 import GHC.HandleEncoding
 import GHC.BaseDir (getBaseDir)
-import GHC.Settings.Utils (getTargetArchOS, maybeReadFuzzy)
+import GHC.Settings.Utils (getTargetArchOS, maybeReadFuzzy, getGlobalPackageDb, RawSettings)
 import GHC.Platform.Host (hostPlatformArchOS)
 import GHC.UniqueSubdir (uniqueSubdir)
 import qualified GHC.Data.ShortText as ST
@@ -62,7 +56,8 @@ import System.Directory ( getXdgDirectory, createDirectoryIfMissing, getAppUserD
                           getModificationTime, XdgDirectory ( XdgData ) )
 import Text.Printf
 
-import Prelude
+import Prelude hiding (Foldable(..))
+import Data.Foldable (Foldable(..))
 
 import System.Console.GetOpt
 import qualified Control.Exception as Exception
@@ -78,9 +73,10 @@ import System.Exit ( exitWith, ExitCode(..) )
 import System.Environment ( getArgs, getProgName, getEnv )
 import System.IO
 import System.IO.Error
+import GHC.IO           ( catchException )
 import GHC.IO.Exception (IOErrorType(InappropriateType))
 import Data.List ( group, sort, sortBy, nub, partition, find
-                 , intercalate, intersperse, foldl', unfoldr
+                 , intercalate, intersperse, unfoldr
                  , isInfixOf, isSuffixOf, isPrefixOf, stripPrefix )
 import Control.Concurrent
 import qualified Data.Foldable as F
@@ -92,6 +88,7 @@ import qualified Data.ByteString as BS
 #if defined(mingw32_HOST_OS)
 import GHC.ConsoleHandler
 #else
+import System.Environment (lookupEnv)
 import System.Posix hiding (fdToHandle)
 #endif
 
@@ -99,20 +96,6 @@ import System.Posix hiding (fdToHandle)
 import qualified System.Info(os)
 #endif
 
-#if defined(WITH_TERMINFO)
-import System.Console.Terminfo as Terminfo
-#endif
-
-#if defined(mingw32_HOST_OS)
-# if defined(i386_HOST_ARCH)
-#  define WINDOWS_CCONV stdcall
-# elif defined(x86_64_HOST_ARCH)
-#  define WINDOWS_CCONV ccall
-# else
-#  error Unknown mingw32 arch
-# endif
-#endif
-
 -- | Short-circuit 'any' with a \"monadic predicate\".
 anyM :: (Monad m) => (a -> m Bool) -> [a] -> m Bool
 anyM _ [] = return False
@@ -389,7 +372,7 @@ runit verbosity cli nonopts = do
 
         splitFields fields = unfoldr splitComma (',':fields)
           where splitComma "" = Nothing
-                splitComma fs = Just $ break (==',') (tail fs)
+                splitComma (_ : fs) = Just $ break (==',') fs
 
         -- | Parses a glob into a predicate which tests if a string matches
         -- the glob.  Returns Nothing if the string in question is not a glob.
@@ -589,6 +572,21 @@ allPackagesInStack = concatMap packages
 stackUpTo :: FilePath -> PackageDBStack -> PackageDBStack
 stackUpTo to_modify = dropWhile ((/= to_modify) . location)
 
+readFromSettingsFile :: FilePath
+                      -> (FilePath -> RawSettings -> Either String b)
+                      -> IO (Either String b)
+readFromSettingsFile settingsFile f = do
+  settingsStr <- readFile settingsFile
+  pure $ do
+    mySettings <- case maybeReadFuzzy settingsStr of
+      Just s -> pure $ Map.fromList s
+      -- It's excusable to not have a settings file (for now at
+      -- least) but completely inexcusable to have a malformed one.
+      Nothing -> Left $ "Can't parse settings file " ++ show settingsFile
+    case f settingsFile mySettings of
+      Right archOS -> Right archOS
+      Left e -> Left e
+
 getPkgDatabases :: Verbosity
                 -> GhcPkg.DbOpenMode mode DbModifySelector
                 -> Bool    -- use the user db
@@ -612,24 +610,38 @@ getPkgDatabases verbosity mode use_user use_cache expand_vars my_flags = do
   -- location is passed to the binary using the --global-package-db flag by the
   -- wrapper script.
   let err_msg = "missing --global-package-db option, location of global package database unknown\n"
-  global_conf <-
+  (top_dir, global_conf) <-
      case [ f | FlagGlobalConfig f <- my_flags ] of
         -- See Note [Base Dir] for more information on the base dir / top dir.
         [] -> do mb_dir <- getBaseDir
                  case mb_dir of
                    Nothing  -> die err_msg
                    Just dir -> do
-                     r <- lookForPackageDBIn dir
-                     case r of
-                       Nothing -> die ("Can't find package database in " ++ dir)
-                       Just path -> return path
-        fs -> return (last fs)
+                     -- Look for where it is given in the settings file, if marked there.
+                     let settingsFile = dir </> "settings"
+                     exists_settings_file <- doesFileExist settingsFile
+                     erel_db <-
+                      if exists_settings_file
+                          then readFromSettingsFile settingsFile getGlobalPackageDb
+                          else pure (Left ("Settings file doesn't exist: " ++ settingsFile))
 
-  -- The value of the $topdir variable used in some package descriptions
-  -- Note that the way we calculate this is slightly different to how it
-  -- is done in ghc itself. We rely on the convention that the global
-  -- package db lives in ghc's libdir.
-  top_dir <- absolutePath (takeDirectory global_conf)
+                     case erel_db of
+                      Right rel_db -> return (dir, dir </> rel_db)
+                      -- If the version of GHC doesn't have this field or the settings file
+                      -- doesn't exist for some reason, look in the libdir.
+                      Left err -> do
+                        r <- lookForPackageDBIn dir
+                        case r of
+                          Nothing -> die (unlines [err, ("Fallback: Can't find package database in " ++ dir)])
+                          Just path -> return (dir, path)
+        fs -> do
+          -- The value of the $topdir variable used in some package descriptions
+          -- Note that the way we calculate this is slightly different to how it
+          -- is done in ghc itself. We rely on the convention that the global
+          -- package db lives in ghc's libdir.
+          let pkg_db = last fs
+          top_dir <- absolutePath (takeDirectory pkg_db)
+          return (top_dir, pkg_db)
 
   let no_user_db = FlagNoUserDb `elem` my_flags
 
@@ -640,7 +652,7 @@ getPkgDatabases verbosity mode use_user use_cache expand_vars my_flags = do
     case [ f | FlagUserConfig f <- my_flags ] of
       _ | no_user_db -> return Nothing
       [] -> do
-        -- See Note [Settings File] about this file, and why we need GHC to share it with us.
+        -- See Note [Settings file] about this file, and why we need GHC to share it with us.
         let settingsFile = top_dir </> "settings"
         exists_settings_file <- doesFileExist settingsFile
         targetArchOS <- case exists_settings_file of
@@ -648,16 +660,11 @@ getPkgDatabases verbosity mode use_user use_cache expand_vars my_flags = do
             warn $ "WARNING: settings file doesn't exist " ++ show settingsFile
             warn "cannot know target platform so guessing target == host (native compiler)."
             pure hostPlatformArchOS
-          True -> do
-            settingsStr <- readFile settingsFile
-            mySettings <- case maybeReadFuzzy settingsStr of
-              Just s -> pure $ Map.fromList s
-              -- It's excusable to not have a settings file (for now at
-              -- least) but completely inexcusable to have a malformed one.
-              Nothing -> die $ "Can't parse settings file " ++ show settingsFile
-            case getTargetArchOS settingsFile mySettings of
-              Right archOS -> pure archOS
+          True ->
+            readFromSettingsFile settingsFile getTargetArchOS >>= \case
+              Right v -> pure v
               Left e -> die e
+
         let subdir = uniqueSubdir targetArchOS
 
             getFirstSuccess :: [IO a] -> IO (Maybe a)
@@ -768,7 +775,7 @@ getPkgDatabases verbosity mode use_user use_cache expand_vars my_flags = do
                 else do
                   db <- readParseDatabase verbosity mb_user_conf
                                           mode use_cache db_path
-                    `Exception.catch` couldntOpenDbForModification db_path
+                    `catchException` couldntOpenDbForModification db_path
                   let ro_db = db { packageDbLock = GhcPkg.DbOpenReadOnly }
                   return (ro_db, Just db)
           | db_path <- final_stack ]
@@ -1291,7 +1298,7 @@ updateDBCache verbosity db db_stack = do
       hasAnyAbiDepends x = length (abiDepends x) > 0
 
   -- warn when we find any (possibly-)bogus abi-depends fields;
-  -- Note [Recompute abi-depends]
+  -- See Note [Recompute abi-depends]
   when (verbosity >= Normal) $ do
     let definitelyBrokenPackages =
           nub
@@ -1332,7 +1339,6 @@ updateDBCache verbosity db db_stack = do
     GhcPkg.DbOpenReadWrite lock -> GhcPkg.unlockPackageDb lock
 
 type PackageCacheFormat = GhcPkg.GenericUnitInfo
-                            ComponentId
                             PackageIdentifier
                             PackageName
                             UnitId
@@ -1341,7 +1347,6 @@ type PackageCacheFormat = GhcPkg.GenericUnitInfo
 
 {- Note [Recompute abi-depends]
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-
 Like most fields, `ghc-pkg` relies on who-ever is performing package
 registration to fill in fields; this includes the `abi-depends` field present
 for the package.
@@ -1392,7 +1397,7 @@ recomputeValidAbiDeps db pkg =
 -- Ghc.PackageDb to store into the database)
 fromPackageCacheFormat :: PackageCacheFormat -> GhcPkg.DbUnitInfo
 fromPackageCacheFormat = GhcPkg.mapGenericUnitInfo
-     mkUnitId' mkComponentId' mkPackageIdentifier' mkPackageName' mkModuleName' mkModule'
+     mkUnitId' mkPackageIdentifier' mkPackageName' mkModuleName' mkModule'
    where
      displayBS :: Pretty a => a -> BS.ByteString
      displayBS            = toUTF8BS . display
@@ -1413,7 +1418,7 @@ convertPackageInfoToCacheFormat :: InstalledPackageInfo -> PackageCacheFormat
 convertPackageInfoToCacheFormat pkg =
     GhcPkg.GenericUnitInfo {
        GhcPkg.unitId             = installedUnitId pkg,
-       GhcPkg.unitInstanceOf     = installedComponentId pkg,
+       GhcPkg.unitInstanceOf     = mkUnitId (unComponentId (installedComponentId pkg)),
        GhcPkg.unitInstantiations = instantiatedWith pkg,
        GhcPkg.unitPackageId      = sourcePackageId pkg,
        GhcPkg.unitPackageName    = packageName pkg,
@@ -1583,43 +1588,35 @@ listPackages verbosity my_flags mPackageName mModuleName = do
 
   if simple_output then show_simple stack else do
 
-#if !defined(WITH_TERMINFO)
+#if defined(mingw32_HOST_OS)
     mapM_ show_normal stack
 #else
     let
-       show_colour withF db@PackageDB{ packages = pkg_confs } =
-           if null pkg_confs
-           then termText (location db) <#> termText "\n    (no packages)\n"
-           else
-               mconcat $ map (<#> termText "\n") $
-                           (termText (location db)
-                            : map (termText "    " <#>) (map pp_pkg pkg_confs))
-          where
-                   pp_pkg p
-                     | installedUnitId p `elem` broken = withF Red  doc
-                     | exposed p                       = doc
-                     | otherwise                       = withF Blue doc
-                     where doc | verbosity >= Verbose
-                               = termText (printf "%s (%s)" pkg (display (installedUnitId p)))
-                               | otherwise
-                               = termText pkg
-                            where
-                            pkg = display (mungedId p)
+      show_colour PackageDB{ location = db_name, packages = pkg_confs } =
+          do hPutStrLn stdout db_name
+             if null pkg_confs
+                 then hPutStrLn stdout "    (no packages)"
+                 else hPutStrLn stdout $ unlines (map ("    " ++) (map pp_pkg pkg_confs))
+           where
+                 pp_pkg p
+                   | installedUnitId p `elem` broken = printf "\ESC[31m%s\ESC[0m" doc -- red color
+                   | exposed p = doc
+                   | otherwise = printf "\ESC[34m%s\ESC[0m" doc -- blue color
+                   where doc | verbosity >= Verbose = printf "%s (%s)" pkg (display (installedUnitId p))
+                             | otherwise            = pkg
+                          where
+                          pkg = display (mungedId p)
 
     is_tty <- hIsTerminalDevice stdout
-    if not is_tty
-       then mapM_ show_normal stack
-       else do tty <- Terminfo.setupTermFromEnv
-               case Terminfo.getCapability tty withForegroundColor of
-                   Nothing -> mapM_ show_normal stack
-                   Just w  -> runTermOutput tty $ mconcat $
-                                                  map (show_colour w) stack
+    -- Equivalent of https://hackage.haskell.org/package/ansi-terminal/docs/System-Console-ANSI.html#v:hSupportsANSI
+    term <- lookupEnv "TERM"
+    mapM_ (if is_tty && term /= Just "dumb" then show_colour else show_normal) stack
 #endif
 
 simplePackageList :: [Flag] -> [InstalledPackageInfo] -> IO ()
 simplePackageList my_flags pkgs = do
    let showPkg :: InstalledPackageInfo -> String
-       showPkg | FlagShowUnitIds `elem` my_flags = display . installedUnitId
+       showPkg | FlagShowUnitIds `elem` my_flags = traceId . display . installedUnitId
                | FlagNamesOnly `elem` my_flags   = display . mungedName . mungedId
                | otherwise                       = display . mungedId
        strs = map showPkg pkgs
@@ -1963,10 +1960,11 @@ checkUnitId ipi db_stack update = do
 
 checkDuplicates :: PackageDBStack -> InstalledPackageInfo
                 -> Bool -> Bool-> Validate ()
-checkDuplicates db_stack pkg multi_instance update = do
+checkDuplicates [] _ _ _ = pure ()
+checkDuplicates (hd : _) pkg multi_instance update = do
   let
         pkgid = mungedId pkg
-        pkgs  = packages (head db_stack)
+        pkgs  = packages hd
   --
   -- Check whether this package id already exists in this DB
   --
@@ -2040,8 +2038,12 @@ checkHSLib _verbosity dirs lib = do
   let filenames = ["lib" ++ lib ++ ".a",
                    "lib" ++ lib ++ "_p.a",
                    "lib" ++ lib ++ "-ghc" ++ GHC.Version.cProjectVersion ++ ".so",
+                   "lib" ++ lib ++ "_p" ++ "-ghc" ++ GHC.Version.cProjectVersion ++ ".so",
                    "lib" ++ lib ++ "-ghc" ++ GHC.Version.cProjectVersion ++ ".dylib",
-                            lib ++ "-ghc" ++ GHC.Version.cProjectVersion ++ ".dll"]
+                   "lib" ++ lib ++ "_p" ++ "-ghc" ++ GHC.Version.cProjectVersion ++ ".dylib",
+                   lib ++ "-ghc" ++ GHC.Version.cProjectVersion ++ ".dll",
+                   lib ++ "_p" ++ "-ghc" ++ GHC.Version.cProjectVersion ++ ".dll"
+                  ]
   b <- liftIO $ doesFileExistOnPath filenames dirs
   when (not b) $
     verror ForceFiles ("cannot find any of " ++ show filenames ++
@@ -2077,7 +2079,7 @@ checkModuleFile pkg modl =
       -- there's no interface file for GHC.Prim
       unless (modl == ModuleName.fromString "GHC.Prim") $ do
       let files = [ ModuleName.toFilePath modl <.> extension
-                  | extension <- ["hi", "p_hi", "dyn_hi" ] ]
+                  | extension <- ["hi", "p_hi", "dyn_hi", "p_dyn_hi"] ]
       b <- liftIO $ doesFileExistOnPath files (importDirs pkg)
       when (not b) $
          verror ForceFiles ("cannot find any of " ++ show files)
@@ -2254,7 +2256,7 @@ installSignalHandlers = do
 #endif
 
 catchIO :: IO a -> (Exception.IOException -> IO a) -> IO a
-catchIO = Exception.catch
+catchIO = catchException
 
 tryIO :: IO a -> IO (Either Exception.IOException a)
 tryIO = Exception.try
