@@ -1,33 +1,41 @@
 {-# LANGUAGE OverloadedStrings #-}
+
 module Haddock.Backends.Hyperlinker.Utils
-    ( hypSrcDir, hypSrcModuleFile, hypSrcModuleFile'
-    , hypSrcModuleUrl, hypSrcModuleUrl'
-    , hypSrcNameUrl
-    , hypSrcLineUrl
-    , hypSrcModuleNameUrl, hypSrcModuleLineUrl
-    , hypSrcModuleUrlFormat
-    , hypSrcModuleNameUrlFormat, hypSrcModuleLineUrlFormat
-    , spliceURL, spliceURL'
+  ( hypSrcDir
+  , hypSrcModuleFile
+  , hypSrcModuleFile'
+  , hypSrcModuleUrl
+  , hypSrcModuleUrl'
+  , hypSrcNameUrl
+  , hypSrcLineUrl
+  , hypSrcModuleNameUrl
+  , hypSrcModuleLineUrl
+  , hypSrcModuleUrlFormat
+  , hypSrcModuleNameUrlFormat
+  , hypSrcModuleLineUrlFormat
+  , hypSrcModuleUrlToNameFormat
+  , hypSrcModuleUrlToLineFormat
+  , hypSrcPkgUrlToModuleFormat
+  , spliceURL
+  , spliceURL'
 
     -- * HIE file processing
-    , PrintedType
-    , recoverFullIfaceTypes
-    ) where
-
-import Haddock.Utils
-import Haddock.Backends.Xhtml.Utils
-
-import GHC
-import GHC.Iface.Ext.Types ( HieAST(..), HieType(..), HieArgs(..), TypeIndex, HieTypeFlat )
-import GHC.Iface.Type
-import GHC.Types.Name      ( getOccFS, getOccString )
-import GHC.Driver.Ppr      ( showSDoc )
-import GHC.Types.Var       ( VarBndr(..), visArg, invisArg, TypeOrConstraint(..) )
-
-import System.FilePath.Posix ((</>), (<.>))
+  , PrintedType
+  , recoverFullIfaceTypes
+  ) where
 
 import qualified Data.Array as A
+import GHC
+import GHC.Iface.Ext.Types (HieAST (..), HieArgs (..), HieType (..), HieTypeFlat, TypeIndex)
+import GHC.Iface.Type
+import GHC.Types.Name (getOccFS, getOccString)
+import GHC.Types.Var (TypeOrConstraint (..), VarBndr (..), invisArg, visArg)
+import GHC.Utils.Outputable (SDocContext)
+import qualified GHC.Utils.Outputable as Outputable
+import System.FilePath.Posix ((<.>), (</>))
 
+import Haddock.Backends.Xhtml.Utils
+import Haddock.Utils
 
 {-# INLINE hypSrcDir #-}
 hypSrcDir :: FilePath
@@ -38,8 +46,12 @@ hypSrcModuleFile :: Module -> FilePath
 hypSrcModuleFile m = moduleNameString (moduleName m) <.> "html"
 
 hypSrcModuleFile' :: ModuleName -> FilePath
-hypSrcModuleFile' mdl = spliceURL'
-    (Just mdl) Nothing Nothing moduleFormat
+hypSrcModuleFile' mdl =
+  spliceURL'
+    (Just mdl)
+    Nothing
+    Nothing
+    moduleFormat
 
 hypSrcModuleUrl :: Module -> String
 hypSrcModuleUrl = hypSrcModuleFile
@@ -72,6 +84,15 @@ hypSrcModuleNameUrlFormat = hypSrcModuleUrlFormat ++ "#" ++ nameFormat
 hypSrcModuleLineUrlFormat :: String
 hypSrcModuleLineUrlFormat = hypSrcModuleUrlFormat ++ "#" ++ lineFormat
 
+hypSrcModuleUrlToNameFormat :: String -> String
+hypSrcModuleUrlToNameFormat url = url ++ "#" ++ nameFormat
+
+hypSrcModuleUrlToLineFormat :: String -> String
+hypSrcModuleUrlToLineFormat url = url ++ "#" ++ lineFormat
+
+hypSrcPkgUrlToModuleFormat :: String -> String
+hypSrcPkgUrlToModuleFormat url = url </> moduleFormat
+
 moduleFormat :: String
 moduleFormat = "%{MODULE}.html"
 
@@ -81,7 +102,6 @@ nameFormat = "%{NAME}"
 lineFormat :: String
 lineFormat = "line-%{LINE}"
 
-
 -- * HIE file processing
 
 -- This belongs in GHC.Iface.Ext.Utils...
@@ -105,17 +125,19 @@ type PrintedType = String
 -- multiple calls to 'recoverFullType' don't share intermediate results. This
 -- function fixes that.
 recoverFullIfaceTypes
-  :: DynFlags
-  -> A.Array TypeIndex HieTypeFlat -- ^ flat types
-  -> HieAST TypeIndex              -- ^ flattened AST
-  -> HieAST PrintedType       -- ^ full AST
-recoverFullIfaceTypes df flattened ast = fmap (printed A.!) ast
-    where
-
+  :: SDocContext
+  -> A.Array TypeIndex HieTypeFlat
+  -- ^ flat types
+  -> HieAST TypeIndex
+  -- ^ flattened AST
+  -> HieAST PrintedType
+  -- ^ full AST
+recoverFullIfaceTypes sDocContext flattened ast = fmap (printed A.!) ast
+  where
     -- Splitting this out into its own array is also important: we don't want
     -- to pretty print the same type many times
     printed :: A.Array TypeIndex PrintedType
-    printed = fmap (showSDoc df . pprIfaceType) unflattened
+    printed = fmap (Outputable.renderWithContext sDocContext . pprIfaceType) unflattened
 
     -- The recursion in 'unflattened' is crucial - it's what gives us sharing
     -- between the IfaceType's produced
@@ -124,15 +146,16 @@ recoverFullIfaceTypes df flattened ast = fmap (printed A.!) ast
 
     -- Unfold an 'HieType' whose subterms have already been unfolded
     go :: HieType IfaceType -> IfaceType
-    go (HTyVarTy n) = IfaceTyVar (getOccFS n)
+    go (HTyVarTy n) = IfaceTyVar (mkIfLclName $ getOccFS n)
     go (HAppTy a b) = IfaceAppTy a (hieToIfaceArgs b)
     go (HLitTy l) = IfaceLitTy l
-    go (HForAllTy ((n,k),af) t) = let b = (getOccFS n, k)
-                                  in IfaceForAllTy (Bndr (IfaceTvBndr b) af) t
-    go (HFunTy w a b)  = IfaceFunTy (visArg TypeLike)   w a b          -- t1 -> t2
-    go (HQualTy con b) = IfaceFunTy (invisArg TypeLike) many_ty con b  -- c => t
+    go (HForAllTy ((n, k), af) t) =
+      let b = (mkIfLclName $ getOccFS n, k)
+       in IfaceForAllTy (Bndr (IfaceTvBndr b) af) t
+    go (HFunTy w a b) = IfaceFunTy (visArg TypeLike) w a b -- t1 -> t2
+    go (HQualTy con b) = IfaceFunTy (invisArg TypeLike) many_ty con b -- c => t
     go (HCastTy a) = a
-    go HCoercionTy = IfaceTyVar "<coercion type>"
+    go HCoercionTy = IfaceTyVar $ mkIfLclName "<coercion type>"
     go (HTyConApp a xs) = IfaceTyConApp a (hieToIfaceArgs xs)
 
     -- This isn't fully faithful - we can't produce the 'Inferred' case
@@ -140,5 +163,5 @@ recoverFullIfaceTypes df flattened ast = fmap (printed A.!) ast
     hieToIfaceArgs (HieArgs args) = go' args
       where
         go' [] = IA_Nil
-        go' ((True ,x):xs) = IA_Arg x Required $ go' xs
-        go' ((False,x):xs) = IA_Arg x Specified $ go' xs
+        go' ((True, x) : xs) = IA_Arg x Required $ go' xs
+        go' ((False, x) : xs) = IA_Arg x Specified $ go' xs
