@@ -27,7 +27,7 @@ import Data.List (sort)
 import Data.List.NonEmpty (NonEmpty (..))
 import qualified Data.Map as Map
 import qualified Data.Maybe as Maybe
-import GHC hiding (fromMaybeContext)
+import GHC hiding (HsTypeGhcPsExt (..), fromMaybeContext)
 import GHC.Core.Type (Specificity (..))
 import GHC.Data.FastString (unpackFS)
 import GHC.Types.Name (getOccString, nameOccName, tidyNameOcc)
@@ -244,7 +244,7 @@ isSimpleSig
           }
         )
     )
-    | Map.null argDocs = Just (map unLoc lnames, unLoc (dropWildCards t))
+    | Map.null argDocs = Just (map unLoc lnames, unLoc (dropWildCardsI t))
 isSimpleSig _ = Nothing
 
 isExportModule :: ExportItem DocNameI -> Maybe Module
@@ -327,7 +327,7 @@ ppDecl decl pats (doc, fnArgsDoc) instances subdocs _fxts = case unLoc decl of
   TyClD _ d@DataDecl{} -> ppDataDecl pats instances subdocs (Just doc) d unicode
   TyClD _ d@SynDecl{} -> ppTySyn (doc, fnArgsDoc) d unicode
   TyClD _ d@ClassDecl{} -> ppClassDecl instances doc subdocs d unicode
-  SigD _ (TypeSig _ lnames ty) -> ppFunSig Nothing (doc, fnArgsDoc) (map unLoc lnames) (dropWildCards ty) unicode
+  SigD _ (TypeSig _ lnames ty) -> ppFunSig Nothing (doc, fnArgsDoc) (map unLoc lnames) (dropWildCardsI ty) unicode
   SigD _ (PatSynSig _ lnames ty) -> ppLPatSig (doc, fnArgsDoc) (map unLoc lnames) ty unicode
   ForD _ d -> ppFor (doc, fnArgsDoc) d unicode
   InstD _ _ -> empty
@@ -362,7 +362,7 @@ ppFamDecl
   -> Bool
   -- ^ unicode
   -> LaTeX
-ppFamDecl associated doc instances decl unicode =
+ppFamDecl associated doc instances decl@(FamDecl{}) unicode =
   declWithDoc
     (ppFamHeader (tcdFam decl) unicode associated <+> whereBit)
     (if null body then Nothing else Just (vcat body))
@@ -401,6 +401,9 @@ ppFamDecl associated doc instances decl unicode =
 
     instancesBit = ppDocInstances unicode instances
 
+ppFamDecl _ _ _ _ _ = error "ppFamDecl"
+  -- Should never be called on a non-FamDecl
+
 -- | Print the LHS of a type\/data family declaration.
 ppFamHeader
   :: FamilyDecl DocNameI
@@ -475,7 +478,11 @@ ppTySyn
               : ppDocBinder name
               : map ppWcSymName (tyvarNames ltyvars)
           )
-      full = hdr <+> char '=' <+> ppLType unicode ltype
+      full = hdr <+> def
+      def = case unLoc ltype of
+        XHsType (HsRedacted k) ->
+          dcolon unicode <+> ppType unicode k
+        _ -> equals <+> ppLType unicode ltype
 ppTySyn _ _ _ = error "declaration not supported by ppTySyn"
 
 -------------------------------------------------------------------------------
@@ -594,7 +601,7 @@ ppSubSigLike unicode typ argDocs subdocs leader = do_sig_args 0 leader typ
       , decltt (ppLContextNoArrow lctxt unicode) <+> nl
       )
         : do_largs n (darrow unicode) ltype
-    do_args n leader (HsFunTy _ _w (L _ (HsRecTy _ fields)) r) =
+    do_args n leader (HsFunTy _ _w (L _ (XHsType (HsRecTy fields))) r) =
       [ (decltt ldr, latex <+> nl)
       | (L _ field, ldr) <- zip fields (leader <+> gadtOpen : repeat gadtComma)
       , let latex = ppSideBySideField subdocs unicode field
@@ -811,10 +818,11 @@ ppInstDecl :: Bool -> InstHead DocNameI -> LaTeX
 ppInstDecl unicode (InstHead{..}) = case ihdInstType of
   ClassInst ctx _ _ _ -> keyword "instance" <+> ppContextNoLocs ctx unicode <+> typ
   TypeInst rhs -> keyword "type" <+> keyword "instance" <+> typ <+> tibody rhs
-  DataInst dd ->
+  DataInst dd@(DataDecl {}) ->
     let cons = dd_cons (tcdDataDefn dd)
         pref = case cons of NewTypeCon _ -> keyword "newtype"; DataTypeCons _ _ -> keyword "data"
      in pref <+> keyword "instance" <+> typ
+  DataInst _ -> error "ppInstDecl"
   where
     typ = ppAppNameTypes ihdClsName ihdTypes unicode
     tibody = maybe empty (\t -> equals <+> ppType unicode t)
@@ -849,7 +857,7 @@ ppDataDecl
   -> Bool
   -- ^ unicode
   -> LaTeX
-ppDataDecl pats instances subdocs doc dataDecl unicode =
+ppDataDecl pats instances subdocs doc dataDecl@(DataDecl {}) unicode =
   declWithDoc
     (ppDataHeader dataDecl unicode <+> whereBit)
     (if null body then Nothing else Just (vcat body))
@@ -890,6 +898,8 @@ ppDataDecl pats instances subdocs doc dataDecl unicode =
               $$ text "\\end{tabulary}\\par"
 
     instancesBit = ppDocInstances unicode instances
+ppDataDecl _ _ _ _ _ _ = error "ppDataDecl"
+  -- Should never be called on a non-DataDecl
 
 -- ppConstrHdr is for (non-GADT) existentials constructors' syntax
 ppConstrHdr
@@ -952,13 +962,13 @@ ppSideBySideConstr subdocs unicode leader (L _ con) =
               header_ = ppConstrHdr forall_ tyVars context unicode
            in case det of
                 -- Prefix constructor, e.g. 'Just a'
-                PrefixCon _ args
+                PrefixCon args
                   | hasArgDocs -> header_ <+> ppOcc
                   | otherwise ->
                       hsep
                         [ header_
                         , ppOcc
-                        , hsep (map (ppLParendType unicode . hsScaledThing) args)
+                        , hsep (map (ppLParendType unicode . hsConDeclFieldToHsTypeNoMult) args)
                         ]
                 -- Record constructor, e.g. 'Identity { runIdentity :: a }'
                 RecCon _ -> header_ <+> ppOcc
@@ -968,9 +978,9 @@ ppSideBySideConstr subdocs unicode leader (L _ con) =
                   | otherwise ->
                       hsep
                         [ header_
-                        , ppLParendType unicode (hsScaledThing arg1)
+                        , ppLParendType unicode (hsConDeclFieldToHsTypeNoMult arg1)
                         , ppOccInfix
-                        , ppLParendType unicode (hsScaledThing arg2)
+                        , ppLParendType unicode (hsConDeclFieldToHsTypeNoMult arg2)
                         ]
       ConDeclGADT{}
         | hasArgDocs || not (isEmpty fieldPart) -> ppOcc
@@ -987,15 +997,15 @@ ppSideBySideConstr subdocs unicode leader (L _ con) =
         -- GADT record declarations
         RecConGADT _ _ -> doConstrArgsWithDocs []
         -- GADT prefix data constructors
-        PrefixConGADT _ args | hasArgDocs -> doConstrArgsWithDocs (map hsScaledThing args)
+        PrefixConGADT _ args | hasArgDocs -> doConstrArgsWithDocs (map hsConDeclFieldToHsTypeNoMult args)
         _ -> empty
       ConDeclH98{con_args = con_args'} -> case con_args' of
         -- H98 record declarations
         RecCon (L _ fields) -> doRecordFields fields
         -- H98 prefix data constructors
-        PrefixCon _ args | hasArgDocs -> doConstrArgsWithDocs (map hsScaledThing args)
+        PrefixCon args | hasArgDocs -> doConstrArgsWithDocs (map hsConDeclFieldToHsTypeNoMult args)
         -- H98 infix data constructor
-        InfixCon arg1 arg2 | hasArgDocs -> doConstrArgsWithDocs (map hsScaledThing [arg1, arg2])
+        InfixCon arg1 arg2 | hasArgDocs -> doConstrArgsWithDocs (map hsConDeclFieldToHsTypeNoMult [arg1, arg2])
         _ -> empty
 
     doRecordFields fields =
@@ -1024,23 +1034,30 @@ ppSideBySideConstr subdocs unicode leader (L _ con) =
           >>= fmap _doc . combineDocumentation . fst
 
 -- | Pretty-print a record field
-ppSideBySideField :: [(DocName, DocForDecl DocName)] -> Bool -> ConDeclField DocNameI -> LaTeX
-ppSideBySideField subdocs unicode (ConDeclField _ names ltype _) =
+ppSideBySideField :: [(DocName, DocForDecl DocName)] -> Bool -> HsConDeclRecField DocNameI -> LaTeX
+ppSideBySideField subdocs unicode (HsConDeclRecField _ names ltype) =
   decltt
     ( cat (punctuate comma (map (ppBinder . rdrNameOcc . foExt . unLoc) names))
+        <+> ppRecFieldMultAnn unicode ltype
         <+> dcolon unicode
-        <+> ppLType unicode ltype
+        <+> ppLType unicode (hsConDeclFieldToHsTypeNoMult ltype)
     )
     <-> rDoc mbDoc
   where
-    -- don't use cd_fld_doc for same reason we don't use con_doc above
-    -- Where there is more than one name, they all have the same documentation
     mbDoc = lookup (unLoc . foLabel . unLoc $ name) subdocs >>= fmap _doc . combineDocumentation . fst
     name =
       case Maybe.listToMaybe names of
         Nothing -> error "No names. An invariant was broken. Please report this to the Haddock project"
         Just hd -> hd
 
+-- don't use cdf_doc for same reason we don't use con_doc above
+-- Where there is more than one name, they all have the same documentation
+ppRecFieldMultAnn :: Bool -> HsConDeclField DocNameI -> LaTeX
+ppRecFieldMultAnn unicode (CDF { cdf_multiplicity = ann }) = case ann of
+  HsUnannotated _ -> empty
+  HsLinearAnn _ -> text "%1"
+  HsExplicitMult _ mult -> multAnnotation <> ppr_mono_lty mult unicode
+
 -- | Pretty-print a bundled pattern synonym
 ppSideBySidePat
   :: [LocatedN DocName]
@@ -1187,9 +1204,9 @@ pp_hs_context cxt unicode = parenList (map (ppType unicode) cxt)
 
 -------------------------------------------------------------------------------
 
-ppBang :: HsBang -> LaTeX
-ppBang (HsBang _ SrcStrict) = char '!'
-ppBang (HsBang _ SrcLazy) = char '~'
+ppBang :: HsSrcBang -> LaTeX
+ppBang (HsSrcBang _ _ SrcStrict) = char '!'
+ppBang (HsSrcBang _ _ SrcLazy) = char '~'
 ppBang _ = empty
 
 tupleParens :: HsTupleSort -> [LaTeX] -> LaTeX
@@ -1305,10 +1322,9 @@ ppr_mono_ty (HsFunTy _ mult ty1 ty2) u =
     ]
   where
     arr = case mult of
-      HsLinearArrow _ -> lollipop u
-      HsUnrestrictedArrow _ -> arrow u
+      HsLinearAnn _ -> lollipop u
+      HsUnannotated _ -> arrow u
       HsExplicitMult _ m -> multAnnotation <> ppr_mono_lty m u <+> arrow u
-ppr_mono_ty (HsBangTy _ b ty) u = ppBang b <> ppLParendType u ty
 ppr_mono_ty (HsTyVar _ NotPromoted (L _ name)) _ = ppDocName name
 ppr_mono_ty (HsTyVar _ IsPromoted (L _ name)) _ = char '\'' <> ppDocName name
 ppr_mono_ty (HsTupleTy _ con tys) u = tupleParens con (map (ppLType u) tys)
@@ -1317,8 +1333,9 @@ ppr_mono_ty (HsKindSig _ ty kind) u = parens (ppr_mono_lty ty u <+> dcolon u <+>
 ppr_mono_ty (HsListTy _ ty) u = brackets (ppr_mono_lty ty u)
 ppr_mono_ty (HsIParamTy _ (L _ n) ty) u = ppIPName n <+> dcolon u <+> ppr_mono_lty ty u
 ppr_mono_ty (HsSpliceTy v _) _ = dataConCantHappen v
-ppr_mono_ty (HsRecTy{}) _ = text "{..}"
-ppr_mono_ty (XHsType{}) _ = error "ppr_mono_ty HsCoreTy"
+ppr_mono_ty (XHsType (HsBangTy b ty)) u = ppBang b <> ppLParendType u ty
+ppr_mono_ty (XHsType HsRecTy{}) _ = text "{..}"
+ppr_mono_ty (XHsType HsCoreTy{}) _ = error "ppr_mono_ty HsCoreTy"
 ppr_mono_ty (HsExplicitListTy _ IsPromoted tys) u = Pretty.quote $ brackets $ hsep $ punctuate comma $ map (ppLType u) tys
 ppr_mono_ty (HsExplicitListTy _ NotPromoted tys) u = brackets $ hsep $ punctuate comma $ map (ppLType u) tys
 ppr_mono_ty (HsExplicitTupleTy _ IsPromoted tys) u = Pretty.quote $ parenList $ map (ppLType u) tys
@@ -1347,6 +1364,7 @@ ppr_mono_ty (HsDocTy _ ty _) unicode =
 ppr_mono_ty (HsWildCardTy _) _ = char '_'
 ppr_mono_ty (HsTyLit _ t) u = ppr_tylit t u
 ppr_mono_ty (HsStarTy _ isUni) unicode = starSymbol (isUni || unicode)
+ppr_mono_ty (XHsType HsRedacted{}) _ = error "ppr_mono_ty: HsRedacted can't be used here"
 
 ppr_tylit :: HsTyLit DocNameI -> Bool -> LaTeX
 ppr_tylit (HsNumTy _ n) _ = integer n
