@@ -23,6 +23,7 @@ module Haddock.Convert
   ( tyThingToLHsDecl
   , synifyInstHead
   , synifyFamInst
+  , synifyKindSig
   , PrintRuntimeReps (..)
   ) where
 
@@ -45,6 +46,7 @@ import GHC.Builtin.Types
   , promotedNilDataCon
   , unitTy
   )
+
 import GHC.Builtin.Types.Prim (alphaTyVars)
 import GHC.Core.Class
 import GHC.Core.Coercion.Axiom
@@ -121,7 +123,7 @@ tyThingToLHsDecl prr t = case t of
                 HsBndrKind _ kind -> HsKindSig noAnn (noLocA (cvt' bvar)) kind
 
             cvt' :: HsBndrVar GhcRn -> HsType GhcRn
-            cvt' (HsBndrVar _ nm)   = HsTyVar noAnn NotPromoted nm
+            cvt' (HsBndrVar _ nm)   = HsTyVar noAnn NotPromoted (fmap noUserRdr nm)
             cvt' (HsBndrWildCard _) = HsWildCardTy noExtField
 
             -- \| Convert a LHsTyVarBndr to an equivalent LHsType.
@@ -176,7 +178,7 @@ tyThingToLHsDecl prr t = case t of
                       $ snd
                       $ classTvsFds cl
                 , tcdSigs =
-                    noLocA (MinimalSig (noAnn, NoSourceText) . noLocA . fmap noLocA $ classMinimalDef cl)
+                    noLocA (MinimalSig (noAnn, NoSourceText) . noLocA $ classMinimalDef cl)
                       : [ noLocA tcdSig
                         | clsOp <- classOpItems cl
                         , tcdSig <- synifyTcIdSig vs clsOp
@@ -476,15 +478,41 @@ synifyDataCon use_gadt_syntax dc =
     -- con_qvars means a different thing depending on gadt-syntax
     (_univ_tvs, ex_tvs, _eq_spec, theta, arg_tys, res_ty) = dataConFullSig dc
     user_tvbndrs = dataConUserTyVarBinders dc -- Used for GADT data constructors
+
+    split_invis_tvbs :: [TyVarBinder] -> ([InvisTVBinder], [TyVarBinder])
+    split_invis_tvbs (Bndr tv (Invisible spec) : bs) =
+      let ~(invis, other) = split_invis_tvbs bs
+      in (Bndr tv spec : invis, other)
+    split_invis_tvbs bs = ([], bs)
+
+    split_req_tvbs :: [TyVarBinder] -> ([ReqTVBinder], [TyVarBinder])
+    split_req_tvbs (Bndr tv Required : bs) =
+      let ~(req, other) = split_req_tvbs bs
+      in (Bndr tv () : req, other)
+    split_req_tvbs bs = ([], bs)
+
+    (outer_tvbs, inner_tvbs) = split_invis_tvbs user_tvbndrs
+
     outer_bndrs
-      | null user_tvbndrs =
+      | null outer_tvbs =
           HsOuterImplicit{hso_ximplicit = []}
       | otherwise =
           HsOuterExplicit
             { hso_xexplicit = noExtField
-            , hso_bndrs = map synifyTyVarBndr user_tvbndrs
+            , hso_bndrs = map synifyTyVarBndr outer_tvbs
             }
 
+    inner_bndrs = mk_telescopes inner_tvbs
+
+    mk_telescopes bs
+      | (invis, other) <- split_invis_tvbs bs, not (null invis)
+      = mkHsForAllInvisTele noAnn (map synifyTyVarBndr invis) : mk_telescopes other
+
+      | (req, other) <- split_req_tvbs bs, not (null req)
+      = mkHsForAllVisTele noAnn (map synifyTyVarBndr req) : mk_telescopes other
+
+      | otherwise = []
+
     -- skip any EqTheta, use 'orig'inal syntax
     ctx
       | null theta = Nothing
@@ -492,11 +520,10 @@ synifyDataCon use_gadt_syntax dc =
 
     linear_tys =
       zipWith
-        ( \ty bang ->
-            let tySyn = synifyType WithinType [] (scaledThing ty)
-             in case bang of
-                  (HsSrcBang _ (HsBang NoSrcUnpack NoSrcStrict)) -> tySyn
-                  (HsSrcBang src bang') -> noLocA $ HsBangTy (noAnn, src) bang' tySyn
+        ( \(Scaled mult ty) (HsSrcBang st unp str) ->
+            let tySyn = synifyType WithinType [] ty
+                multSyn = synifyMultRec [] mult
+            in CDF (noAnn, st) unp str multSyn tySyn Nothing
         )
         arg_tys
         (dataConSrcBangs dc)
@@ -504,25 +531,24 @@ synifyDataCon use_gadt_syntax dc =
     field_tys = zipWith con_decl_field (dataConFieldLabels dc) linear_tys
     con_decl_field fl synTy =
       noLocA $
-        ConDeclField
-          noAnn
-          [noLocA $ FieldOcc (mkVarUnqual $ field_label $ flLabel fl) (noLocA  (flSelector fl))]
+        HsConDeclRecField
+          noExtField
+          [noLocA $ FieldOcc (mkVarUnqual $ field_label $ flLabel fl) (noLocA (flSelector fl))]
           synTy
-          Nothing
 
     mk_h98_arg_tys :: Either String (HsConDeclH98Details GhcRn)
     mk_h98_arg_tys = case (use_named_field_syntax, use_infix_syntax) of
       (True, True) -> Left "synifyDataCon: contradiction!"
       (True, False) -> return $ RecCon (noLocA field_tys)
-      (False, False) -> return $ PrefixCon noTypeArgs (map hsUnrestricted linear_tys)
+      (False, False) -> return $ PrefixCon linear_tys
       (False, True) -> case linear_tys of
-        [a, b] -> return $ InfixCon (hsUnrestricted a) (hsUnrestricted b)
+        [a, b] -> return $ InfixCon a b
         _ -> Left "synifyDataCon: infix with non-2 args?"
 
     mk_gadt_arg_tys :: HsConDeclGADTDetails GhcRn
     mk_gadt_arg_tys
       | use_named_field_syntax = RecConGADT noExtField (noLocA field_tys)
-      | otherwise = PrefixConGADT noExtField (map hsUnrestricted linear_tys)
+      | otherwise = PrefixConGADT noExtField linear_tys
    in
     -- finally we get synifyDataCon's result!
     if use_gadt_syntax
@@ -533,7 +559,8 @@ synifyDataCon use_gadt_syntax dc =
             ConDeclGADT
               { con_g_ext = noExtField
               , con_names = pure name
-              , con_bndrs = noLocA outer_bndrs
+              , con_outer_bndrs = noLocA outer_bndrs
+              , con_inner_bndrs = inner_bndrs
               , con_mb_cxt = ctx
               , con_g_args = hat
               , con_res_ty = synifyType WithinType [] res_ty
@@ -712,7 +739,7 @@ synifyType
   -> Type
   -- ^ the type to convert
   -> LHsType GhcRn
-synifyType _ _ (TyVarTy tv) = noLocA $ HsTyVar noAnn NotPromoted $ noLocA (getName tv)
+synifyType _ _ (TyVarTy tv) = noLocA $ HsTyVar noAnn NotPromoted $ noLocA (noUserRdr $ getName tv)
 synifyType _ vs (TyConApp tc tys) =
   maybe_sig res_ty
   where
@@ -723,7 +750,7 @@ synifyType _ vs (TyConApp tc tys) =
       , [TyConApp rep [TyConApp lev []]] <- tys
       , rep `hasKey` boxedRepDataConKey
       , lev `hasKey` liftedDataConKey =
-          noLocA (HsTyVar noAnn NotPromoted (noLocA liftedTypeKindTyConName))
+          noLocA (HsTyVar noAnn NotPromoted (noLocA $ noUserRdr liftedTypeKindTyConName))
       -- Use non-prefix tuple syntax where possible, because it looks nicer.
       | Just sort <- tyConTuple_maybe tc
       , tyConArity tc == tys_len =
@@ -757,7 +784,7 @@ synifyType _ vs (TyConApp tc tys) =
                   | L _ (HsExplicitListTy _ IsPromoted tTy') <- stripKindSig tTy ->
                       noLocA $ HsExplicitListTy noExtField IsPromoted (hTy : tTy')
                   | otherwise ->
-                      noLocA $ HsOpTy noExtField IsPromoted hTy (noLocA $ getName tc) tTy
+                      noLocA $ HsOpTy noExtField IsPromoted hTy (noLocA $ noUserRdr $ getName tc) tTy
       -- ditto for implicit parameter tycons
       | tc `hasKey` ipClassKey
       , [name, ty] <- tys
@@ -771,7 +798,7 @@ synifyType _ vs (TyConApp tc tys) =
               noExtField
               NotPromoted
               (synifyType WithinType vs ty1)
-              (noLocA eqTyConName)
+              (noLocA $ noUserRdr eqTyConName)
               (synifyType WithinType vs ty2)
       -- and infix type operators
       | isSymOcc (nameOccName (getName tc))
@@ -781,14 +808,14 @@ synifyType _ vs (TyConApp tc tys) =
                 noExtField
                 prom
                 (synifyType WithinType vs ty1)
-                (noLocA $ getName tc)
+                (noLocA $ noUserRdr $ getName tc)
                 (synifyType WithinType vs ty2)
             )
             tys_rest
       -- Most TyCons:
       | otherwise =
           mk_app_tys
-            (HsTyVar noAnn prom $ noLocA (getName tc))
+            (HsTyVar noAnn prom $ noLocA (noUserRdr $ getName tc))
             vis_tys
       where
         !prom = if isPromotedDataCon tc then IsPromoted else NotPromoted
@@ -828,7 +855,7 @@ synifyType s vs funty@(FunTy af w t1 t2)
   where
     s1 = synifyType WithinType vs t1
     s2 = synifyType WithinType vs t2
-    w' = synifyMult vs w
+    w' = synifyMultArrow vs w
 synifyType s vs forallty@(ForAllTy (Bndr _ argf) _ty) =
   case argf of
     Required -> synifyVisForAllType vs forallty
@@ -984,10 +1011,15 @@ noKindTyVars ts (FunTy _ w t1 t2) =
 noKindTyVars ts (CastTy t _) = noKindTyVars ts t
 noKindTyVars _ _ = emptyVarSet
 
-synifyMult :: [TyVar] -> Mult -> HsArrow GhcRn
-synifyMult vs t = case t of
-  OneTy -> HsLinearArrow noExtField
-  ManyTy -> HsUnrestrictedArrow noExtField
+synifyMultArrow :: [TyVar] -> Mult -> HsMultAnn GhcRn
+synifyMultArrow vs t = case t of
+  OneTy -> HsLinearAnn noExtField
+  ManyTy -> HsUnannotated noExtField
+  ty -> HsExplicitMult noExtField (synifyType WithinType vs ty)
+
+synifyMultRec :: [TyVar] -> Mult -> HsMultAnn GhcRn
+synifyMultRec vs t = case t of
+  OneTy -> HsUnannotated noExtField
   ty -> HsExplicitMult noExtField (synifyType WithinType vs ty)
 
 synifyPatSynType :: PatSyn -> LHsType GhcRn
